#!/usr/bin/env python
# coding: utf-8


#!/home/atms/.virtualenvs/ebkus27/bin/python


from __future__ import unicode_literals
import warnings
warnings.filterwarnings("ignore")
"""
Skript zur Unterstützung beim Entwickeln und Testen von EBKuS

Das Skript kann auf verschiedenen Rechnern im Netzwerk ausgeführt werden, wobei alle
Daten (außer der Installation selbst) im Verzeichnis test_installationen des Entwicklungsrechners
gelagert werden. Dieses Verzeichnis muss auf jedem Testrechner eingebunden werden (Samba, sshfs, oä)

ebkusdev.py <cmd> [options] [args]

revert  [name]
         - geht auf den Basissnapshot der installierten Version
           zurück, mit Name auf die betreffende.
           Findet selbständig raus, auf welcher Platform
           aufgerufen, und welche Version dort installiert ist.

snapshot [--basis] [--name <name>]
	  - macht snapshot der aktuellen Installation in das
            <Versionsverzeichnis>/snapshots/<hostname>. Vorhandener snapshot
	    mit demselben Namen wird überschrieben.
	  --basis: Dem generierten Namen wird 'basis__' vorangestellt.
            Damit erkennt revert den snapshot, zu dem revertiert wird, wenn kein Name
            genannt wird.
          --name: explizit einen Namen vergeben, der dem generiert Namen präfigiert wird.
            Ohne Option nur der generierte Namen.

install [--use_snapshot [<snapshotname>]] 
        [--patch <patchverzeichnis>] ...
        version 
	- prüft vor der Installation, ob alles deinstalliert ist.
        - installiert die genannte Version. 
	   Falls ein <Versionsdir>/snapshots/basis existiert,
	   kann optional die dortige ebkus.conf und
	   db-Datenbanken für die Einrichtung verwendet werden.
	   Das sollte schneller gehen.
        - Problem: Was tun bei der Erstinstallation?
          Erstmal: --> Manuell, dann von snapshot
          - die default ebkus.conf nehmen
          - SERVER_NAME per Skript in 0.0.0.0 umwandeln, 
            damit Tests von entfernten Maschinen gehen. 
	  - Beim ersten Mal werden alle demos neu berechnet und in ebkus/sql
            abgespeichert, damit sind sie auch im basis-snapshot.
          - Bei der Zweitinstallation vom Snapshot sollte dann alles schneller 
            gehen.
uninstall [--all]
	-- deinstalliert ebkus, mit --all auch apache und mysql
           auf windows
	   Sollte prüfen, dass keine Start/Stop-Skripte bzw
	   Windowsdienste danach vorhanden sind oder gar laufen.

update [--configure] [--patch <patchverzeichnis>] version
        - stoppt alle Instanzen
        - updatet die installierte Version auf eine neuere Version.
          Dazu muss es im Verzeichnis der neueren Version ein Verzeichnis
          ebkus-update-<from-version>-to-<to-version>
          geben, wobei from-version <= basis-version der Basisversion der aktuellen Installation. 
          - head: Dateien in trunk, einschließlich der noch nicht eingecheckten.
            Es werden alle hineinkopiert, die nicht schon identisch in der Basisversion sind.
          - trunk: Dateien in trunk, die eingecheckt sind, also ohne nicht eingecheckte Äbderungen.
            Es werden alle hineinkopiert, die nicht schon identisch in der Basisversion sind.
          - *update*-to-<version>: Es wird ein Update-Distribution gesucht und alles daraus hineinkopiert.

        - --patch <pattern>: Für jede --patch-Option wird ein matchendes Verzeichnis gesucht, das zusätzlich
           hineinkopiert wird. Im Namen des Verzeichnisses muss 'patch' vorkommen und danach irgendwo der angegebene String.
           Für jedes Pattern muss genau ein Patch gefunden werden, sonst ist es ein Fehler.
           Wenn pattern mit '/' beginnt, wird nicht gematched, dann muss das ein existierender Pfad sein.

          genannten Version, immer von einem dist-Verzeichnis,
	  niemals von einem Snapshot. Soll Testen, ob ein
	  offizielles Update funktioniert.
        - funktioniert nur mit größerer oder gleicher version
        - alle Instanzen bleiben ungestartet

refresh 
        -- updatet head und trunk im Installationsverzeichnis
startc [instanz] [debug-level][--sql]
      -- startet die zuletzt verwendete Instanz mit Konsolen
      ausgabe. print statements werden auch ausgegeben.
      Zu erkennen anhand der Pseudodatei
      CURRENT__<hostname>__<Instanzname>
      Wird mit C-c abgebrochen.

log [-n <zahl>] [name]
      Letzten n (default 20) Zeilen der logdatei der zuletzt benutzten Instanz bzw name
      ausgeben. name=all von allen Instanzen. Das Ergebnis kann
      man greppen.
test [server] ...
     - testet alle installierte Server

demo_daten -n Anzahl --user default='test' --instanz default='test'


"""


import os
import platform
import sys
import subprocess
from tempfile import TemporaryFile
import traceback
import datetime
import glob
from shutil import rmtree, copy2
assert sys.version_info >= (2,6), "Es wird Python 2.6 oder höher benötigt"
from os.path import join, split, dirname, basename, exists, isdir, isfile, normpath, normcase, \
    abspath, realpath, splitext, getmtime, getsize

import argparse # , argcomplete
from ebkusctl import EBKuSControl, ServiceException, touch, create_directory, mycopytree, check_output

# print 'Python Version: ', sys.version

"""
Ist es möglich, eine Klasse für jeden Subparser zu machen?
"""

# Die folgenden Kommandos werden an ebkusctl weitergeleitet:
# (Vielleicht eines Tages richtig beide Skripte integrieren)
_ebkus_ctl_commands = ('start', 'stop', 'status', 'restart', 'test', 'log', 'message')

# Auf dieses Verzeichnis wird auf dem Zielrechner das 
# Verzeichnis test_installation des Entwicklungsrechners gemountet
_test_installation = {
    'alibih': '/home/data/dev/ebkus/test_installationen',
    'ebkusvm1': '/home/ebkus/test_installationen',
    'ebkusdvm': '/home/data/dev/ebkus/test_installationen',
    'ebkusvm10': '/home/data/dev/ebkus/test_installationen',
    'debianvm': '/home/data/dev/ebkus/test_installationen',
    'alibis': '/home/ebkus/test_installationen',
    'win10_ebkus': r'G:\test_installationen',
    'winxp_ebkus': r'G:\test_installationen',
}
# In diesem Verzeichnis wird auf dem Zielrechner ebkus installiert
# Sollte eigentlich überall ebkus_test heißen und neben test_installationen stehen.
_ebkus_installation = {
    'alibih': '/home/data/dev/ebkus/ebkus_test',
    'ebkusvm1': '/home/ebkus/ebkus_test',
    'ebkusdvm': '/home/ebkus/ebkus_test',
    'ebkusvm10': '/home/ebkus/ebkus_test',
    'debianvm': r'/home/atms/ebkus_installation',
    'alibis': '/home/ebkus/ebkus_test',
    'win10_ebkus': r'C:\ebkus_test',
    #'win10_ebkus': r'C:\ebkus_installation',
    'winxp_ebkus': r'C:\ebkus_installation',
}

#_trunk = '/home/data/dev/ebkus/ebkus-3/trunk'
_git_work = '/home/data/dev/ebkus/work/ebkus/'
# nicht der volle domain-Name wg. .ssh/config
#_svn_path = 'svn+ssh://ebkus@alibis/home/ebkus/svn_repos/ebkus/ebkus-3'
_git_path = '/home/data/dev/ebkus/work/ebkus' # Lokales git!!
_devel_hosts = ('alibih', 'albreed', 'ebkusvm10')

class EBKuSDevException(Exception):
    pass
class EBKuSDev(object):
    def __init__(self, argv, **kw):
        self.parser = self.make_parser()
        # Die einfachste Methode, an ebkusctl weiterzuleiten.
        # Kommandos von ebkusctl sind auch hier bekannt (sonst könnte
        # nicht korrekt geparst werden), aber die Argumente nicht.
        # Die stehen in rest.
        self.ns, self.rest = self.parser.parse_known_args()
        #print self.ns, self.rest
        if isfile(self.ebkus_conf):
            self.ectl = EBKuSControl(self.ebkus_home)
        else:
            # Nur install möglich
            self.ectl = None

    def make_parser(self):
        parser = argparse.ArgumentParser(description="Skript zur Unterstuetzung beim Entwickeln und Testen von EBKuS.\n"
                                         "Unbekannte Befehle werden an ebkusctl.py weitergegeben")
        parser.add_argument("--ebkus_installation", dest='ebkus_installation', 
                            help="EBKuS-Installationsverzeichnis. "
                            "Falls nicht angegeben, muss das Skript im Homeverzeichnis der Installation stehen ('ebkus'), "
                            "oder in der EBKuS-Testumgebung (sinnvoll nur fuer EBKuS-Programmierer)"
                            )
        parser.add_argument("--test_installationen", dest='test_installationen', 
                            help="Verzeichnis mit Testinstallationen auf dem Entwicklungsrechner")
        # parser.add_argument("--verbose", "-v", action='count', 
        #                     help="Verbosity (error, info, debug)")
        subparsers = parser.add_subparsers(dest='command', help='sub-command help')
        # Alle ebkusctl Kommandos anhängen
        for cmd in _ebkus_ctl_commands:
            subparsers.add_parser(cmd, help="ebkusctl.py {0}".format(cmd))
        def add_after_change_arguments(parser, change_type):
            """
            change_type: Install, Update, Revert
            """
            parser.add_argument('--configure', action='store_true', help="Nach dem {0} neu konfigurieren.".format(change_type))
            parser.add_argument('--restart', action='store_true', help="Nach dem {0} Instanzen starten.".format(change_type))
            parser.add_argument('--test', action='store_true', help="Nach dem {0} Instanzen testen (login-Seite ueber Apache aufrufen). Impliziert --restart".format(change_type))
            parser.add_argument('--patch', action='append', help="Nach dem {0} Patches einspielen. (Argument ist Pfad)".format(change_type))
        def add_common_arguments(parser, name):
            parser.add_argument("--verbose", "-v", action='count', 
                                help="Verbosity. v wiederholen um zu steigern. (error, info, debug, aber nicht umgesetzt)")

        uninstall = subparsers.add_parser('uninstall', help="Eine EBKuS-Installation deinstallieren.")
        uninstall.add_argument('--force', action='store_true', help="Verschaerft deinstallieren, zB wenn kein Basissnapshot da ist oder ebkus_home fehlt.")
        add_common_arguments(uninstall, 'Uninstall')

        install = subparsers.add_parser('install', help="Eine EBKuS-Installation aus einer Version erstmals erstellen. Die alte Installation wird deinstalliert.")
        install.add_argument('version', help="Die Version, auf die installiert werden soll.")
        install.add_argument('--dist_dir', help="Von diesem Verzeichnis installieren anstatt von dem Standarddistributionsverzeichnis der Version. Fuer Experimente! Sonst zu verwirrend.")
        install.add_argument('--setvar', action='append', dest='key_values', help="Konfigurationsvariablen fuer ebkus.conf setzen, zB server_name:0.0.0.0")
        install.add_argument('--use_snapshot', nargs='?', const='basis', default=None, help="Auf der Basis eine Snapshot installieren. Ohne explizites Argument wird basis genommen.")
        install.add_argument('--use_ebkus_conf', help="Auf der Basis einer vorhandenen ebkus.conf installieren.")
        add_after_change_arguments(install, 'Install')
        add_common_arguments(install, 'Install')


        setvars = subparsers.add_parser('setvars', help="ebkus.conf einer EBKuS-Installation veraendern.")
        setvars.add_argument('key_values', nargs='*', help="Konfigurationsvariable <variablenname>:<wert>, zB 'server_name:0.0.0.0'")
        setvars.add_argument('--list', '-l', nargs='?', default=None, const='*', help="Konfigurationsvariablen aus ebkus.conf ausgeben. Mit Argument nur die, in der String vorkommt")
        add_after_change_arguments(setvars, 'Setvars')
        add_common_arguments(setvars, 'Setvars')

        update = subparsers.add_parser('update', help="Die EBKuS-Testinstallation updaten")
        update.add_argument('version', nargs='?', 
                            help="Die Version, auf die updatet werden so. Beispiele: "  
                            "4.9.2 -- die Version 4.9.2, head -- das aktuelle Arbeitsverzeichnis, "
                            "trunk -- worauf HEAD im lokalen GIT gerade zeit, kann also auch beliebiger branch sein."
                            )
        update.add_argument('--export', action='store_true', 
                            help="Falls Version head oder trunk ist, wird im Versionsverzeichnis"
                            " die Distribution vor dem Update der Installation durch git aktualisiert."
                            " head: das aktuelle Arbeitsverzeichnis. trunk: worauf HEAD im lokalen git zeigt. (Verwirrend:-)"
                            " (Nur auf einem Entwicklungsrechner.)")
        update.add_argument('--regen', action='store_true', help="ebapigen.py in trunk regenerieren")
        update.add_argument('--ignore_compare_versions', action='store_true', help="Beim Update kleiner/größer-Vergleich von Versionen weglassen (ist nicht richtig implementiert)")
        add_after_change_arguments(update, 'Update')
        add_common_arguments(update, 'Update')

        refresh = subparsers.add_parser('refresh', help="head und trunk im Installationsverzeichnis updaten, so dass sie auch in entfernten Rechnern da ist")
        refresh.add_argument('--regen', action='store_true', help="ebapigen.py in trunk regenerieren")
        refresh.add_argument('--trunk', action='store_true', help="trunk updaten (mit git: HEAD nehmen, entsprechend 'git checkout <branch>' machen)")
        refresh.add_argument('--head', action='store_true', help="head updaten (Mit git: Arbeitsverzeichnis nehmen)")

        revert = subparsers.add_parser('revert', help="Eine EBKuS-Installation aus einem Snapshot wiederherstellen")
        revert.add_argument('--name', default='basis', help="Name zur Auswahl eines Snapshots.")
        add_after_change_arguments(revert, 'Revert')
        add_common_arguments(revert, 'Revert')

        snapshot = subparsers.add_parser('snapshot', help="Snapshot einer EBKuS-Installation erstellen")
        snapshot.add_argument('--snapshots_dir', help="Das Verzeichnis, in dem der Snapshot erstellt werden soll. " 
                              "Falls nicht angegeben, wird das Verzeichnis test_installationen/<version>/<host>/<generierter Name>.")
        snapshot.add_argument('--basis', default=None, const='basis', nargs='?',
                              help="Ohne Argument: Der erzeugte Snapshot wird als Basissnapshot ausgezeichnet. "
                              "Mit Argument: Es wird kein Snapshot erzeugt, stattdessen wird der durch den Namen ausgezeichnete als Basissnaphost definiert.")
        snapshot.add_argument('--name', help="Der Snapshot erhaelt den angegebenen Namen.")
        snapshot.add_argument('--list', '-l', action='store_true', help="Snapshots auflisten.")
        snapshot.add_argument('--delete', nargs='*', help="Snapshots loeschen.")
        add_common_arguments(snapshot, 'Snapshot')

        startc = subparsers.add_parser('startc', help="Einen EBKuS-Dienst auf der Konsole starten")
        startc.add_argument('instance', nargs='?', help="Instanz, die gestartet werden soll. Wird sofort zu Defaultinstanz gemacht.")
        startc.add_argument('--auto_test', action='store_true', help="User test ist automatisch angemeldet.")
        startc.add_argument('--auto_Admin', action='store_true', help="User Admin ist automatisch angemeldet.")
        startc.add_argument('--auto', help="Der User ist automatisch angemeldet.")
        startc.add_argument('--today', help="Von der Anwendung verwendetes Datum setzten (today()), zurücksetzen mit None")
        startc.add_argument('--debug', action='store_true', help="Log-level ist debug.")
        startc.add_argument('--sql', action='store_true', help="Sql requests ausgeben.")
        add_common_arguments(startc, 'Startc')

        interact = subparsers.add_parser('interact', help="Eine EBKuS-API in der python-Kommandozeile starten")
        interact.add_argument('instance', nargs='?', help="Instanz, die gestartet werden soll. Wird sofort zu Defaultinstanz gemacht.")
        #startc.add_argument('--debug', action='store_true', help="Log-level ist debug.")
        add_common_arguments(interact, 'Interact')

        demo_daten = subparsers.add_parser('demo_daten', help="Demodaten in eine Instanz einfuegen")
        demo_daten.add_argument('instance', nargs='?', default='test', help="Instanz, in die Demodaten eingefuegt werden sollen.")
        demo_daten.add_argument('--anzahl', '-n', default=1, type=int, help="Anzahl der Akten, die eingefuegt werden sollen.")
        demo_daten.add_argument('--einrichten', action='store_true', help="Instanz einrichten (Stellen, Mitarbeiter, etc.).")
        add_common_arguments(demo_daten, 'Demodaten')
        #add_after_change_arguments(revert, 'Revert')

        dry = subparsers.add_parser('dry', help="Zum Testen")
        dry.add_argument('snapshot_path', nargs='*', help="Das Verzeichnis des Snapshots, aus dem die Installation wiederhergestellt werden soll.")
        dry.add_argument('--use_snapshot', nargs='?', const='basis', default=None, help="Name oder Verzeichnis zur Auswahl eines Snapshots. Ohne Argument wird basis genommen.")
        #dry.add_argument('--use_snapshot', default='basis', help="Name oder Verzeichnis zur Auswahl eines Snapshots. Ohne Argument wird basis genommen.")
        info = subparsers.add_parser('info', help="Infos ueber die Testinstallation")
        add_common_arguments(info, 'Info')
        pytest = subparsers.add_parser('pytest', help="Ruft pytest mit dem Verzeichnis lib/ebkus/test auf")
        add_common_arguments(pytest, 'Pytest')
        pytest.add_argument('-k', '--expression', help="expression zur Auswahl von Testfunktionen (pytest -k <expre>). Wird durchgereicht")
        pytest.add_argument('-q', '--quiet', action='store_true', help="-q (quiet) an pytest durchreichen")
        pytest.add_argument('-s', '--stdout', action='store_true', help="-s (don't hide output) an pytest durchreichen")
        pytest.add_argument('-w', '--warn', action='store_true', help="don't suppress warnings")
        
        return parser

    ####### props START
    @property
    def host(self):
        # Funktioniert auf allen Platformen
        import socket
        name = socket.gethostname()
        return name.lower()
    @property
    def basis_version(self):
        """Die Basisversion der aktuellen Installation, zB '4.5' oder '4.6.2' oder 'trunk'. 
        Aus dieser Version ist ursprünglich installiert worden.
        Wird in Dateinamen wie 'INSTALLED__<host>__<version>' gespeichert.
         """
        names = os.listdir(self.test_installationen)
        versions = [name for name in names if name.startswith('INSTALLED__{0}'.format(self.host))]
        if len(versions) < 1:
            return None
            #raise EBKuSDevException("Keine installierte Version für {0}".format(self.host))
        return versions[0].split('__')[2]
    @basis_version.setter
    def basis_version(self, basis_version):
        """Die Basisversion der aktuellen Installation setzen. Falls basis_version None ist,
        evt vorhandene Datei löschen.
        """
        names = os.listdir(self.test_installationen)
        versions = [name for name in names if name.startswith('INSTALLED__{0}'.format(self.host))]
        for name in versions:
            os.unlink(join(self.test_installationen, name))
        if basis_version:
            touch(join(self.test_installationen, 'INSTALLED__{0}__{1}'.format(self.host, basis_version)))
    @property
    def aktuelle_version(self):
        """Die aktuelle Version der aktuellen Installation. 
        Unterscheidet sich von der Basisversion durch zwischenzeitliche Updates.
        """
        try:
            return self.ectl.get_ebkus_version(self.ebkus_home)
        except:
            return 'Unbekannt'
    @property
    def version_dir(self):
        """Verzeichnis der Version"""
        return join(self.test_installationen, self.basis_version)
    # def svn_source(self, version):
    #     """SVN-Pfad für eine Version liefern."""
    #     if version == 'head':
    #         # Arbeitsverzeichnis mit nicht-eingecheckten Änderungen
    #         return _trunk
    #     elif version == 'trunk':
    #         # Eingecheckt aktuell in SVN
    #         return _svn_path + '/trunk'
    #     else:
    #         assert version[0].isdigit()
    #         # Version mit Versionstag
    #         return _svn_path + '/tags/EBKuS-' + version
    # Wird nirgends verwendet:
    # def git_source(self, version):
    #     """TODO
    #     GIT-Commit für eine Version liefern."""
    #     if version == 'head':
    #         # Arbeitsverzeichnis mit nicht-eingecheckten Änderungen
    #         return _trunk
    #     elif version == 'trunk':
    #         # Eingecheckt aktuell in SVN
    #         return _svn_path + '/trunk'
    #     else:
    #         assert version[0].isdigit()
    #         # Version mit Versionstag
    #         return _svn_path + '/tags/EBKuS-' + version
    def dist_dir(self, version, trailing_slash=False):
        """Distributionsverzeichnis für eine gegebene Version. 
        version kann auch 'head' oder 'trunk' sein.
        """
        if version in ('head', 'trunk'):
            dir_name = version
        else:
            # zB 4.4.2
            assert version[0].isdigit()
            dir_name = 'EBKuS-' + version
        res = join(self.test_installationen, version, dir_name)
        if trailing_slash:
            res += '/'
        return res
            
    def update_dir(self, basis_version, to_version):
        """Updateverzeichnis für ein Update von basis_version auf to_version. 
        """
        dir_name = "ebkus-update-{0}-to-{1}".format(basis_version, to_version)
        return join(self.test_installationen, to_version, dir_name)
    @property
    def default_instance(self):
        names = os.listdir(self.test_installationen)
        versions = [name for name in names if name.startswith('DEFAULT__{0}'.format(self.host))]
        if len(versions) < 1:
            return None
            #raise EBKuSDevException("Keine Defaultinstanz für {0}".format(self.host))
        return versions[0].split('__')[2]
    @default_instance.setter
    def default_instance(self, instance):
        names = os.listdir(self.test_installationen)
        versions = [name for name in names if name.startswith('DEFAULT__{0}'.format(self.host))]
        for name in versions:
            os.unlink(join(self.test_installationen, name))
        if instance:
            touch(join(self.test_installationen, 'DEFAULT__{0}__{1}'.format(self.host, instance)))
    @property
    def test_installationen(self):
        res = self.ns.test_installationen
        if res:
            return res
        return _test_installation[self.host]
    @property
    def ebkus_installation(self):
        res = self.ns.ebkus_installation
        if res:
            return res
        return _ebkus_installation[self.host]
    @property
    def ebkus_home(self):
        return join(self.ebkus_installation, 'ebkus')
    @property
    def ebkus_lib(self):
        return join(self.ebkus_home, 'lib')
    @property
    def ebkus_conf(self):
        return join(self.ebkus_installation, 'ebkus', 'ebkus.conf')
    @property
    def snapshots_dir(self):
        snapshots_dir = join(self.test_installationen, self.basis_version, 'snapshots', self.host)
        if not isdir(snapshots_dir):
            create_directory(snapshots_dir)
        return snapshots_dir
    @property
    def basis_snapshot(self):
        """Den Basissnaphotpfad für die aktuelle Installation liefern"""
        return self.get_snapshot_by_name('basis')
    @basis_snapshot.setter
    def basis_snapshot(self, path):
        """path als Basissnaphot definierten durch Anhängen von '__basis'
        Sicherstellen, dass nur ein Snapshot den Suffix '__basis' hat.
        """
        if not path:
            return
        basis_snapshot = self.get_snapshot_by_name('basis')
        if basis_snapshot:
            name = basename(basis_snapshot)
            name = name[:-len('__basis')]
            os.rename(basis_snapshot, join(dirname(basis_snapshot), name))
        name = basename(path) + '__basis'
        os.rename(path, join(dirname(path), name))
    def get_snapshot_by_name(self, name, verbose=0):
        """Den Snapshotpfad für name finden. 
        Snapshots sind so aufgebaut: <datum_zeit>__<name>[__basis]
        Falls name mit einer Zahl beginnt, Snapshot nach Datum suchen.
        Sonst nur im Namensteil suchen.
        """
        snapshots_dir = self.snapshots_dir
        filenames = [fname for fname in os.listdir(snapshots_dir)]
        basis_candidates = []
        match_candidates = []
        start_candidates = []
        # Basis auswählen
        if name == 'basis':
            basis_candidates = [fname for fname in filenames if fname.endswith('__basis')]
        for fname in filenames:
            parts = parts = fname.split('__')
            for part in parts:
                if part.startswith(name):
                    start_candidates.append(fname)
        match_candidates = [fname for fname in filenames if name.lower() in fname.lower()]
        count = 0
        all_candidates = []
        for candidates in (basis_candidates, start_candidates, match_candidates):
            count += len(candidates)
            all_candidates += candidates
            if len(candidates) == 1:
                return join(snapshots_dir, candidates[0])
        if count < 1:
            return None
        msg = 'Name für snapshot nicht eindeutig: {}'.format(', '.join(all_candidates))
        raise EBKuSDevException(msg)
    ####### props END

    def execute(self):
        if not self.ectl and not self.ns.command in ('info', 'install', 'uninstall'):
            print 'Keine Installation vorhanden. Es sind nur die Kommandos install, uninstall oder info moeglich'
            return
        cmd = self.ns.command
        if cmd in _ebkus_ctl_commands:
            res = self.execute_ebkusctl()
            sys.exit(res)
        # Nochmal parsen. Jetzt müssen alle Argumente ohne Rest stimmen.
        self.ns = self.parser.parse_args()
        cmd = self.ns.command
        func = getattr(self, cmd, None)
        if func:
            return func()
    def execute_ebkusctl(self):
        # assert cmd in _ebkus_ctl_commands
        mydir = dirname(abspath(__file__))
        args = self.ectl.python_exe + [join(mydir, 'ebkusctl.py'),]
        args += sys.argv[1:]
        #print 'ARGV', sys.argv
        #print 'ARGS', args
        rc = subprocess.call(args)
        sys.exit(rc)

    def modify_ebkus_conf(self, path, new_path=None, **kw):
        """ebkus.conf modifizieren.
        kw ist ein dict mit ebkus.conf Variablen und neuen Werten.
        Werte in der ebkus.conf werden durch diese ersetzt, auch wenn sie
        vorher auskommentiert waren.
        Falls new_path, wird die modifizierte Datei dorthin geschrieben, ansonsten in place.
        """
        import re
        _var_regex = re.compile(r'[# \t]*(?P<var>[a-z][a-z_]*[a-z]*)[ ]*:')
        modified_lines = []
        modified_vars = set()
        with open(path, 'r') as f:
            for line in f:
                line = line.decode('latin1')
                var = None
                match = _var_regex.search(line)
                if match:
                    var = match.group('var')
                if var in kw:
                    new_line = "{0}: {1}\n".format(var, kw[var])
                    modified_lines.append(new_line)
                    modified_vars.add(var)
                    #print new_line
                else:
                    modified_lines.append(line)
        if new_path:
            path = new_path
        with open(path, 'w') as f:
            for line in modified_lines:
                f.write(line.encode('latin1'))
        return modified_vars
    def list_ebkus_conf(self, path, searchstr=None):
        """ebkus.conf auflisten. 
        Falls match, nur Zeilen, wo der String vorkommt, aber dann auch auskommentierte.
        """
        import re
        _var_regex = re.compile(r'[# \t]*(?P<var>[a-z][a-z_]*[a-z]*)[ ]*:')
        with open(path, 'r') as f:
            for line in f:
                line = line.decode('latin1').strip()
                if searchstr:
                    if searchstr in line:
                        print line
                else:
                    if line.startswith('#'):
                        continue
                    print line
    def setvars(self):
        #print 'KEY_VALUES', self.ns.key_values
        key_values = [x.split(':', 1) for x in self.ns.key_values]
        key_value_dict = dict([(k.strip(), v.strip()) for k,v in key_values])
        if key_value_dict:
            self.modify_ebkus_conf(self.ebkus_conf, **key_value_dict)
        list = self.ns.list
        if list:
            self.list_ebkus_conf(self.ebkus_conf, searchstr=list if list != '*' else None)
        self.after_change()
        if key_value_dict:
            print 'ebkus.conf modified'
    def _call_install_script(self, dist_dir, config=False):
        exe = [sys.executable]
        if platform.system() == 'Windows':
            exe.append('-u')
        args = exe + [join(dist_dir, 'install.py')]
        if config:
            args.append('--config')
        args.append(self.ebkus_installation)
        if config:
            # Variable nicht ausgeben
            rc = subprocess.call(args, stdout=open(os.devnull, 'wb'))
        else:
            rc = subprocess.call(args)
        if rc != 0:
            raise EBKuSDevException('install ging schief')
    def install(self):
        #print 'INSTALL'
        # if isdir(self.ebkus_home):
        #     print 'EBKuS-Home existiert noch. Bitte erst deinstallieren: {0}'.format(self.basis_version)
        #     return
        version = self.ns.version
        self.basis_version = version
        special_dist_dir = self.ns.dist_dir
        if special_dist_dir:
            assert isdir(special_dist_dir)
        use_snapshot = self.ns.use_snapshot
        ebkus_conf = self.ns.use_ebkus_conf
        if use_snapshot:
            snapshot_path = self.get_snapshot_by_name(use_snapshot)
            snapshot_home = join(snapshot_path, 'ebkus')
            mycopytree(snapshot_home, self.ebkus_installation)
            assert isdir(self.ebkus_home)
            self.ectl = EBKuSControl(self.ebkus_home)
            self.ectl.restore(snapshot_path, configure=True, restart=False, 
                              test=False, install_mode=True, verbose=0)
        else:
            if special_dist_dir:
                dist_dir = special_dist_dir
            else:
                dist_dir = self.dist_dir(version)
                if not isdir(dist_dir):
                    #self.svn_export_dist(version)
                    self.git_export_dist(version)
            assert isdir(dist_dir)
            if ebkus_conf:
                # mkdir self.ebkus_home
                create_directory(self.ebkus_home)
                # cp ebkus_conf self.ebkus_conf
                copy2(ebkus_conf, self.ebkus_conf)
            if self.ns.key_values:
                key_values = [x.split(':', 1) for x in self.ns.key_values]
                key_value_dict = dict([(k.strip(), v.strip()) for k,v in key_values])
                if key_value_dict:
                    if not isfile(self.ebkus_conf):
                        self._call_install_script(dist_dir, config=True)
                    self.modify_ebkus_conf(self.ebkus_conf, **key_value_dict)
                    #self.list_ebkus_conf(self.ebkus_conf)
            self._call_install_script(dist_dir, config=False)
        self.basis_version = version
        self.after_change()
        print 'Version {0}{1} installiert'.format(
            version, "(von Snapshot {0})".format(snapshot_path) if use_snapshot else '')
    def uninstall(self):
        print 'UNINSTALL'
        basis_version = self.basis_version
        force = self.ns.force
        if basis_version and self.ectl:
            # reguläre Deinstallation möglich
            basis_snapshot = self.basis_snapshot
            if not basis_snapshot and not force:
                print 'Kein Basissnapshot'
                return
            try:
                self.ectl.stop_instances()
            except ServiceException:
                # evt. nicht richtig installiert ?
                pass
            try:
                args = self.ectl.python_exe + [join(self.ebkus_home, 'uninstall.py'), '--force', self.ebkus_installation, 'ebkus']
                print 'ARGS'
                rc = subprocess.call(args)
                assert rc == 0
                self.basis_version = None
                print 'Deinstalliert: {0}'.format(basis_version)
                return
            except AssertionError:
                if not force:
                    print 'uninstall ging schief, evt. mit --force wiederholen'
        if force:
            print 'forced uninstall'
            # laufende Instanzen killen
            # Alle Dienste, die mit ebkus_ anfangen, deinstallieren
            # ebkus_home löschen
            if isdir(self.ebkus_home):
                rmtree(self.ebkus_home)
            # INSTALLED_ auf None setzen
            self.basis_version = None
    def snapshot(self):
        print 'SNAPSHOT:', self.ns.basis, self.ns.name
        basis = self.ns.basis
        name = self.ns.name
        snapshots_dir = self.snapshots_dir
        if self.ns.list:
            snapshots = os.listdir(snapshots_dir)
            snapshots.sort()
            if snapshots:
                print 'Snapshots:'
                for x in snapshots:
                    print ' ', x
            return
        if self.ns.delete:
            for name in self.ns.delete:
                snapshot_path = self.get_snapshot_by_name(name)
                if snapshot_path:
                    rmtree(snapshot_path)
                    print 'Deleted: ', snapshot_path
            return
        if basis and basis != 'basis':
            assert name is None, "--name <name1> und --basis <name2> geht nicht zusammen"
            # Snapshot als basis definieren und fertig
            name = basis
            snapshot_path = self.get_snapshot_by_name(name)
            if snapshot_path:
                self.basis_snapshot = snapshot_path
                print 'Als Basissnapshot definiert: {0}'.format(self.basis_snapshot)
            return
        snapshot_name = self.ectl.snapshot(snapshots_dir=snapshots_dir, restart=False, verbose=0)
        path = join(snapshots_dir, snapshot_name)
        if name:
            new_path = join(snapshots_dir, 
                            snapshot_name + '__' + name)
            #print 'rename', path, new_path
            os.rename(path, new_path)
            path = new_path
        basis_snapshot = self.basis_snapshot
        if not basis_snapshot:
            # Wenn noch kein Basissnapshot, den ersten zur Basis machen.
            basis = True
        if basis:
            self.basis_snapshot = path
        print 'Snapshot erstellt: {0}'.format(path if not basis else self.basis_snapshot)
    def revert(self):
        print 'REVERT:', self.ns.name
        name = self.ns.name
        if name:
            snapshot_path = self.get_snapshot_by_name(name)
        else:
            snapshot_path = self.basis_snapshot
        if snapshot_path:
            #print 'reverting', snapshot_path
            self.ectl.restore(snapshot_path, configure=self.ns.configure, restart=False, test=False)
        else:
            print 'Kein Snapshot gefunden: {0}'.format(name if name else 'basis')
            return
        self.after_change()
        if snapshot_path:
            print 'Revertiert nach: {0}'.format(snapshot_path)

    def after_change(self):
        patches = self.ns.patch
        verbose = self.ns.verbose
        if self.ectl is None and isfile(self.ebkus_conf):
            self.ectl = EBKuSControl(self.ebkus_home)
        if patches:
            for patch in patches:
                if isdir(patch):
                    self.ectl.update(patch, configure=False, no_snapshot=True, restart=False, test=False, compare_versions=False,verbose=0)
                    print '  Gepatched mit {0}'.format(patch)
        if self.ns.configure:
            self.ectl.configure_ebkus()
            self.ectl.restart('apache')
        if self.ns.restart or self.ns.test:
            self.ectl.restart_instances(verbose=verbose, test=False)
            if self.ns.test:
                self.ectl.test_instances(verbose=verbose)
        
    def regenerate_ebapigen(self, dist_dir):
        from_dir = dist_dir
        assert exists(from_dir)
        try:
            lib = join(dist_dir, 'lib')
            filename = ebapigen = join(from_dir, 'lib', 'ebkus', 'app', 'ebapigen.py')
            #print 'PATH', path
            sys.path.insert(0, lib)
            from ebkus.gen.genEb import generate_ebapi
            generate_ebapi(filename)
            print 'Regenerated: ', filename
        finally:
            if sys.path[0] == lib:
                del sys.path[0]

    def refresh(self):
        """head und trunk im Installationsverzeichnis updaten, so dass die
        auch in entfernten Rechnern aktuell zur Verfügung stehen.
        """
        if not self.host in _devel_hosts:
            print "refresh nur auf dem Entwicklungsrechner"
            return
        if  self.ns.regen:
            self.regenerate_ebapigen(_git_work)
        if self.ns.trunk:
            #self.svn_export_dist('trunk', force=True)
            self.git_export_dist('trunk', force=True)
        if self.ns.head:
            #self.svn_export_dist('head', force=True)
            self.git_export_dist('head', force=True)
        
    def update(self):
        """Die aktuell installierte Version auf version updaten.
        
        Voraussetzung dafür ist, dass im Verzeichnis der Zielversion ein ebkus-update-<basis_version>-to-<to_version> 
        liegt.
        """
        # find update dist '*to-4.5', trunk, head
        to_version = self.ns.version
        # muss nicht die aktuelle version sein, da bereits ein Update eingespielt worden sein kann
        basis_version = self.basis_version
        aktuelle_version = self.aktuelle_version
        # if self.host == 'alibis':
        #     from_lib_ebkus = join(self.dist_dir(to_version), 'lib', 'ebkus')
        #     print 'Auf alibis nur den Inhalt von {0} nach ebkus_installation/ebkus/testlib/ebkus schieben.'.format(from_lib_ebkus)
        print 'basis_version:', basis_version
        print 'aktuelle_version:', aktuelle_version
        print 'to_version:', to_version
        compare_versions = False if self.ns.ignore_compare_versions else True
        if  self.ns.regen and self.host in _devel_hosts:
            self.regenerate_ebapigen(_git_work)
        if to_version not in ('head', 'trunk') and to_version == basis_version:
            print '** Update zur selben Version nicht moeglich'
            return
        if to_version:
            update_dir = self.update_dir(basis_version, to_version)
            force = False
            if self.ns.export or (self.host in _devel_hosts and to_version in ('head', 'trunk')):
                assert self.host in _devel_hosts, "Export aus git nur auf Entwicklungsrechnern"
                force = True if to_version in ('head', 'trunk') else False
                # Distribution aus git updaten, wenn force==False nichts tun, wenn Verzeichnis schon da ist.
                print 'GIT_EXPORT:', to_version
                self.git_export_dist(to_version, force=force)
            # update_dir wird nur dann neu erzeugt, wenn sie noch gar nicht vorhanden war, oder nach einem export aus head oder trunk
            if not exists(self.dist_dir(basis_version)):
                self.git_export_dist(basis_version)
            if self.create_update_dir(basis_version, to_version, force=force):
                assert isdir(update_dir)
            else:
                # basis_version, to_version identisch, to_version ist head oder trunk
                # Direkt aus dist kopieren
                update_dir = self.dist_dir(to_version)
            print 'UPDATE_DIR',  update_dir
            # Wenn alle Voraussetzungen erfüllt sind ...
            self.ectl.update(update_dir, stop=True, configure=False, no_snapshot=True, 
                             restart=False, test=False, verbose=0, compare_versions=compare_versions)
            print 'Updated von {aktuelle_version} auf {to_version}. (Basis: {basis_version})'.format(**locals())
        #self.after_change()

    def create_update_dir(self, basis_version, to_version, force=False):
        """Das Verzeichnis 
        <test_installationen>/<to_version>/ebkus-update-<basis_version>-to-<to_version> erzeugen.

        Es enthält alle Dateien aus der <to_version>-Distribution, die nicht identisch in der
        <base_version>-Distribution schon enthalten sind.

        Falls force, evt. vorhandens Verzeichnis vorher löschen. Sonst nichts tun, falls schon vorhanden.
        """
        # to_version
        tree_new = self.dist_dir(to_version)
        print 'TREE_NEW', tree_new
        assert isdir(tree_new)
        tree_old = self.dist_dir(basis_version)
        assert isdir(tree_old)
        copy_to = self.update_dir(basis_version, to_version)
        if isdir(copy_to):
            if force:
                rmtree(copy_to, ignore_errors=True)
                print 'CREATE_UPDATE_DIR deleted:', copy_to
        if not isdir(copy_to):
            from tree_diff import TreeDiff
            TreeDiff(tree_new, tree_old, copy_to).tree_diff()
            if isdir(copy_to):
                print 'CREATE_UPDATE_DIR created:', copy_to
                return True
        return False

    # def svn_export_dist(self, version, force=False):
    #     """Das Distributionsverzeichnis von <version> aus dem SVN updaten.
    #     Falls force, vorher löschen. Sonst nichts tun, falls vorhanden.
    #     """
    #     assert self.host in _devel_hosts, "Export aus svn nur auf Entwicklungsrechnern"
    #     dist_dir = self.dist_dir(version)
    #     print 'SVN_EXPORT_DIST dist_dir', dist_dir
    #     if isdir(dist_dir) and force:
    #         rmtree(dist_dir, ignore_errors=True)
    #         print 'SVN_EXPORT_DIST deleted:', dist_dir
    #     if isdir(dist_dir):
    #         return
    #     # svn kann working directory exportieren! (siehe svn export --help)
    #     args = ['svn', 'export', '-q', self.svn_source(version), self.dist_dir(version)]
    #     rc = subprocess.call(args)
    #     if rc != 0:
    #         print 'SVN_EXPORT_DIST ERROR args', args
    #         raise EBKuSDevException('Export nach {0} ging schief.'.format(dist_dir))
    #     print 'SVN_EXPORT_DIST exported', args

    def git_export_dist(self, version, force=False):
        """
        Das Distributionsverzeichnis von <version> aus dem GIT updaten.
        Falls force und nicht head: vorher löschen. Sonst nichts tun, falls vorhanden.
        Falls force und head: rsync --delete. Sonst nichts tun, falls vorhanden.

        Die Bedeutung der speziellen Versionen 'head' und 'trunk' werden aus der svn-Zeit
        übernommen:
        - head ist der aktuelle Zustand des Arbeitsverzeichnisses, das noch nicht committed ist 
          (nicht zu verwechseln mit 'HEAD' in git!)
        - trunk ist die letzte eingecheckte Version. Ich verwende hier das git-'HEAD'. Damit kann man
          leicht verschieden Branches testen. Man muss lediglich vor dem update ein
          git checkout <branch> machen.
          Evt. wäre es hier sinnvoll, explizit mit Namen versehene Versionen vorzusehen. Da müsste aber
          in diesem Skript mehr umgebaut werden.
        """
        assert self.host in _devel_hosts, "Export aus git nur auf Entwicklungsrechnern"
        dist_dir = self.dist_dir(version)
        print 'GIT_EXPORT_DIST dist_dir', dist_dir
        if isdir(dist_dir) and force and version != 'head':
            rmtree(dist_dir, ignore_errors=True)
            print 'GIT_EXPORT_DIST deleted:', dist_dir
        if force and version == 'head':
            # aktuelle Arbeitsverzeichnis
            # rsync erzeugt das fehlende Verzeichnis
            args = ['rsync', '-a', '--exclude', '.git', '--delete', _git_work, self.dist_dir(version, trailing_slash=True)]
            rc = subprocess.call(args)
        else:
            # falls --force wurde gelöscht, sonst fertig, da vorhanden
            if isdir(dist_dir):
                return
            if version == 'trunk':
                commit_ish = 'HEAD'
            else:
                commit_ish = 'EBKuS-'+version
            # tar braucht das existierende Verzeichnis
            os.mkdir(dist_dir, 0755)
            args = "git archive --remote {} --format tar {} | tar xf - -C {}" \
                .format(_git_work, commit_ish, dist_dir) 
            rc = subprocess.call(args, shell=True)
        if rc != 0:
            print 'GIT_EXPORT_DIST ERROR args', args
            raise EBKuSDevException('Export nach {0} ging schief.'.format(dist_dir))
        print 'GIT_EXPORT_DIST exported', args

    def startc(self):
        default_instance = self.default_instance
        # Argument ändert default
        if self.ns.instance and self.ns.instance != self.default_instance:
            self.default_instance = self.ns.instance
        instance = self.default_instance
        if instance is None:
            print 'Keine Instanz'
            return
        instance_dir = join(self.ebkus_home, instance)
        print 'Auf der Konsole starten: ', instance, self.ectl.python_exe
        args = self.ectl.python_exe + [join(instance_dir, 'start.py'), '--console']
        if self.ns.debug:
            args.append('--debug')
        if self.ns.sql:
            args.append('--sql')
        if self.ns.auto:
            args.append('--auto_{}'.format(self.ns.auto))
        if self.ns.auto_test:
            args.append('--auto_test')
        if self.ns.auto_Admin:
            args.append('--auto_Admin')
        if self.ns.today:
            args.append('--today')
            args.append(self.ns.today)
        rc = subprocess.call(args)
        if rc != 0:
            raise EBKuSDevException('Start ging schief')

    def interact(self):
        default_instance = self.default_instance
        # Argument ändert default
        if self.ns.instance and self.ns.instance != self.default_instance:
            self.default_instance = self.ns.instance
        instance = self.default_instance
        if instance is None:
            print 'Keine Instanz'
            return
        instance_dir = join(self.ebkus_home, instance)
        print 'In der python-Kommandozeile starten: ', instance, self.ectl.python_exe
        copy2(join(self.ebkus_lib, 'ebkus', 'interaktiv.py'), instance_dir)
        os.chdir(instance_dir)
        args = self.ectl.python_exe + ['-i', 'interaktiv.py']
        rc = subprocess.call(args)
        if rc != 0:
            raise EBKuSDevException('Start ging schief')

    def demo_daten(self):
        print self.ns.instance
        print self.ns.anzahl
        instance = self.ns.instance
        anzahl = self.ns.anzahl
        assert instance in self.ectl.instances, "Testinstanz '{0}' ist nicht konfiguriert".format(instance)

        # Pfad setzen
        path = self.ebkus_lib
        if path and path != sys.path[0]:
            sys.path.insert(0, path)
        from ebkus.config import init
        config = init(instance, self.ebkus_conf)
        from ebkus.gen.demo_daten import DemoDaten
        dd = DemoDaten(config, verbose=self.ns.verbose)
        if self.ns.einrichten:
            dd.einrichten()
        for i in range(anzahl):
            dd.eine_akte(zumitid='test')
        
    def dry(self):
        print 'USE_SNAPSHOT', self.ns.use_snapshot
        #print 'Host: {self.host} Installed version: {self.basis_version} Defaultinstanz: {self.default_instance}'.format(self=self)
        #print 'test_installationen: {self.test_installationen} ebkus_installation: {self.ebkus_installation} ebkus_home: {self.ebkus_home}'.format(self=self)
        print 'DRY:', self.ns
    def info(self):
        def zeile(key, value):
            print "{0:>30}: {1}".format(key, value)
        def liste(key, values):
            if values:
                zeile(key, values[0])
                for x in values[1:]:
                    zeile('', x)
        def installed_services():
            """Liste der Namen aller als Dienst installierten EBKuS-Server.
            Namen beginnen immer mit 'ebkus_'"""
            if sys.platform == 'win32':
                res = subprocess.check_output(['sc', 'query', 'state=', 'all'])
                # b' muss davor, sonst Unicode-Error
                res_list = res.split(b'\r\n')
                instance_names = [line[20:] for line in res_list if line.startswith(b'SERVICE_NAME: ebkus_')]
                return instance_names
            elif sys.platform.startswith('linux'):
                res_list = glob.glob("/etc/init.d/ebkus_*")
                instance_names = [line[18:] for line in res_list]
                return instance_names

        version = self.basis_version
        zeile('Host', self.host)
        zeile('Installiert', self.basis_version)
        zeile('Aktuell', self.aktuelle_version)
        zeile('Defaultinstanz', self.default_instance)
        if version:
            liste('Snapshots', sorted(os.listdir(self.snapshots_dir)))
            updates = [basename(x) for x in glob.glob(self.test_installationen + '/*' + '/ebkus-update*')]
            liste('Updates', sorted(updates))
        # geht so nicht
        #check_output(['ps', 'aux', '|', 'grep', '-i', 'ebkus', '|', 'grep', 'start.py'])
        #zeile('Aktive EBKuS-Dienste', '') 
        #os.system("ps aux | grep -i ebkus | grep start.py")
        liste('Installierte EBKuS-Dienste', sorted(installed_services()))
        if self.ectl:
            self.ectl.show(config=self.ns.verbose>0)
    def pytest(self):
        args = ['pytest']
        v = self.ns.verbose
        if v:
            args.append('-' + 'v'*v)
        if self.ns.expression:
            args += ['-k', self.ns.expression]
        if self.ns.quiet:
            args.append('-q')
        if self.ns.stdout:
            args.append('-s')
        if not self.ns.warn:
            args.append('-p')
            args.append('no:warnings')
        args.append(join(self.ebkus_lib, 'ebkus', 'test'))
        if v:
            print args
        subprocess.call(args)

if __name__ == '__main__':
    ctl = EBKuSDev(sys.argv)
    ctl.execute()
    
