# coding: latin-1
"""
Routinen für die Erzeugung des amtlichen Datensatzes
für die Bundesjugendhilfestatistik

Die Funktion jghexport(jahr) gibt zwei Strings zurück:

- den abzugebenden Datensatz ohne Zeilenumbrüche
- den Kontrolldatensatz mit Zeilenumbrüchen und der zusätzlichen
  Angabe der Fallnummer und der Mitarbeiterkennung pro Fall

Die Funktion wird vom Webinterface (html/datenaustausch.py) benutzt, um
dem Anwender zu ermöglichen, den abzugebenen Datensatz und den
Kontrolldatensatz herunterzuladen.

Außerdem kann diese Datei als Skript verwendet werden, um die Datensätze
direkt als Dateien ohne Umweg über das Webinterface zu erzeugen.
Dazu ist diese Datei (jghexport.py) in das Verzeichnis der Instanz
zu kopieren, für die die Statistik erzeugt werden soll.

Aufruf:

python jghexport.py <jahr>

python sollte zum Aufruf desselben Interpreters führen, mit dem EBKuS
installiert wurde.
"""


if __name__ == '__main__':
    import sys
    try:
        import init
    except ImportError:
        print "Dieses Skript zuerst in das Verzeichnis der Instanz kopieren,"
        print "fuer die der JGH-Export durchgefuehrt werden soll."
        sys.exit(1)
        
from ebkus.config import config
from ebkus.app.ebapi import FallList, JugendhilfestatistikList, Jugendhilfestatistik2007List, \
    today, EE
from ebkus.app.ebupd import jgh_laufende_nummer_setzen
from ebkus.html.jghstatistik import get_ags_plz_ort_if_wohnort_ausserhalb


# Die SQL-Variante unten ist viel schneller, also das hier nur noch aus historischen Gründen...
def get_daten_fuer_uebersicht(berichtsjahr):
    """Basis sind alle Fälle, die zu irgendeinem Zeitpunkt im Berichtsjahr offen
    waren.

        Im Berichtsjahr 2018 gab es 2772 Fälle, davon 1150 Übernahmen aus 2017 und 1650 
        Neuaufnahmen.

                                       Übernahmen  Neuaufnahmen  Insgesamt ohne Fall
        Fälle im Berichtsjahr:

                                               mit zdA  ohne zdA  exportieren  zu klären  insgesamt ohne Fall (nur falls vorhanden)
                                                                   
        Bundesstatistik mit Eintrag 
        Ende der Hilfe/Beratung 2018              234     123         236         123         432      2        

        Bundesstatistik mit Eintrag
        Hilfe/Beratung dauert an für 2018           0     344         345           0         345      1
        
        Keine Bundesstatistik vorhanden             0      23           0          23          23

        Sonstige                                  123      23           0          23         146

        Summe                                    


                           Button        Problematische Fälle anzeigen

                                                                                        
                           Fallnummer    Beginn (Anmeldung) letzte Zuständigkeit    zdA   Bundesstatistik  fortdauernd
                                                                                          Berichtsjahr
                           123-2016HAU     02.08.2016              Tester          offen         2017           ja


        Abgeschlossene Fälle im Berichtsjahr:

                                                      
        Fordauernde Fälle am Ende des Berichtsjahrs: 

        

    """
    class data(object):
        def __init__(self, **kw):
            for k,v in kw.items():
                setattr(self, k, v)
    jahr = berichtsjahr
    # ohne Fall
    BS_ohne_fall_andauernd = Jugendhilfestatistik2007List(where='fall_id is NULL and ey is null and jahr = {jahr}'.format(**locals()))
    sum_BS_ohne_fall_andauernd = len(BS_ohne_fall_andauernd)
    BS_ohne_fall_beendet = Jugendhilfestatistik2007List(where='fall_id is NULL and ey = {jahr}'.format(**locals()))
    sum_BS_ohne_fall_beendet = len(BS_ohne_fall_beendet)
    BS_ohne_fall_beendet += JugendhilfestatistikList(where='fall_id is NULL and ey = {jahr}'.format(**locals()))
    sum_BS_ohne_fall_beendet = len(BS_ohne_fall_beendet)
    # faelle, die im Berichtsjahr irgendwann offen waren
    faelle = FallList(where = 'bgy<={jahr} and '
                      '(zday >= {jahr} or zday = 0)'.format(**locals()), 
                      cache_related=True)
    sum_faelle = len(faelle)
    sum_uebernahmen = len([f for f in faelle if f['bgy'] < jahr])
    sum_neuaufnahmen = sum_faelle - sum_uebernahmen

    BS_Berichtsjahr_beendet = [f for f in faelle if f['jgh'] and f['jgh__ey']==jahr]
    sum_BS_Berichtsjahr_beendet = len(BS_Berichtsjahr_beendet)
    BS_Berichtsjahr_beendet_mit_zda = [f for f in BS_Berichtsjahr_beendet if f['zday'] > 0]
    sum_BS_Berichtsjahr_beendet_mit_zda = len(BS_Berichtsjahr_beendet_mit_zda)
    BS_Berichtsjahr_andauernd = [f for f in faelle if f['jgh'] and f['jgh__ey']==None and f['jgh__hda__code']=='1' and f['jgh__jahr']==jahr]
    sum_BS_Berichtsjahr_andauernd = len(BS_Berichtsjahr_andauernd)
    BS_fehlt = [f for f in faelle if f['jgh']==None]
    sum_BS_fehlt = len(BS_fehlt)


    BS_sonstige = [f for f in faelle if f['jgh'] and ((f['jgh__ey']!=None and f['jgh__ey']!=jahr) or (f['jgh__ey']==None and f['jgh__jahr']!=jahr))]
    sum_BS_sonstige = len(BS_sonstige)

    BS_sonstige_zda = [f for f in BS_sonstige if f['zday'] > 0]
    sum_BS_sonstige_zda = len(BS_sonstige_zda)
    sum_BS_sonstige_ohne_zda = sum_BS_sonstige - sum_BS_sonstige_zda

    res = data(
        sum_BS_ohne_fall_andauernd=sum_BS_ohne_fall_andauernd,
        sum_BS_ohne_fall_beendet=sum_BS_ohne_fall_beendet,
        sum_BS_ohne_fall=sum_BS_ohne_fall_andauernd + sum_BS_ohne_fall_beendet,
        sum_faelle=sum_faelle,
        sum_uebernahmen=sum_uebernahmen,
        sum_neuaufnahmen=sum_neuaufnahmen,
        sum_BS_Berichtsjahr_beendet=sum_BS_Berichtsjahr_beendet,
        sum_BS_Berichtsjahr_beendet_mit_zda=sum_BS_Berichtsjahr_beendet_mit_zda,
        sum_BS_Berichtsjahr_andauernd=sum_BS_Berichtsjahr_andauernd,
        sum_BS_fehlt=sum_BS_fehlt,
        sum_BS_sonstige=sum_BS_sonstige,
        sum_BS_sonstige_zda=sum_BS_sonstige_zda,
        sum_BS_sonstige_ohne_zda=sum_BS_sonstige_ohne_zda,
        )

    assert (
        sum_BS_Berichtsjahr_beendet +
        sum_BS_Berichtsjahr_andauernd +
        sum_BS_fehlt +
        sum_BS_sonstige
        == 
        sum_faelle
        )
    print
    print 'Berichtsjahr: ', berichtsjahr
    print '======================'
    print
    for k,v in res.__dict__.items():
        print k,v

    return res

def get_daten_fuer_uebersicht_sql(berichtsjahr):
    """Basis sind alle Fälle, die zu irgendeinem Zeitpunkt im Berichtsjahr offen
    waren.

        Im Berichtsjahr 2018 gab es 2772 Fälle, davon 1150 Übernahmen aus 2017 und 1650 
        Neuaufnahmen.

                                       Übernahmen  Neuaufnahmen  Insgesamt ohne Fall
        Fälle im Berichtsjahr:

                                               mit zdA  ohne zdA  exportieren  zu klären  insgesamt ohne Fall (nur falls vorhanden)
                                                                   
        Bundesstatistik mit Eintrag 
        Ende der Hilfe/Beratung 2018              234     123         236         123         432      2        

        Bundesstatistik mit Eintrag
        Hilfe/Beratung dauert an für 2018           0     344         345           0         345      1
        
        Keine Bundesstatistik vorhanden             0      23           0          23          23

        Sonstige                                  123      23           0          23         146

        Summe                                    


                           Button        Problematische Fälle anzeigen

                                                                                        
                           Fallnummer    Beginn (Anmeldung) letzte Zuständigkeit    zdA   Bundesstatistik  fortdauernd
                                                                                          Berichtsjahr
                           123-2016HAU     02.08.2016              Tester          offen         2017           ja


        Abgeschlossene Fälle im Berichtsjahr:

                                                      
        Fordauernde Fälle am Ende des Berichtsjahrs: 

        

    """
    class Data(object):
        def __init__(self, **kw):
            for k,v in kw.items():
                setattr(self, k, v)
        def __str__(self):
            return str(self.__dict__)
    def get_count(result_dict_list, counters, **kw):
        for counter_name in kw.keys():
            setattr(counters, counter_name, 0)
        for row_dict in result_dict_list:
            mygroup = row_dict.get('mygroup')
            mycount = row_dict.get('mycount')
            for counter_name, group_name in kw.items():
                if mygroup == group_name:
                    setattr(counters, counter_name, mycount)
    jahr = berichtsjahr
    counters = Data()
    from ebkus.db.sql import getDBHandle
    # ohne Fall
    sql = """
SELECT count(j.id) as mycount, c.name as mygroup
FROM jghstat07 j
     INNER JOIN code c ON j.hda = c.id
WHERE j.fall_id is null and (j.jahr = {jahr} or j.ey = {jahr})
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_BS_ohne_fall_andauernd='ja', sum_BS_ohne_fall_beendet='nein')
    counters.sum_BS_ohne_fall = counters.sum_BS_ohne_fall_beendet + counters.sum_BS_ohne_fall_andauernd

    # Fälle
    sql = """
SELECT count(f.bgy) as mycount, IF(f.bgy<{jahr}, 'ueber', 'neu') as mygroup
FROM fall f
WHERE f.bgy <= {jahr} and (f.zday >= {jahr} or f.zday = 0)
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_uebernahmen='ueber',sum_neuaufnahmen='neu')
    counters.sum_faelle = counters.sum_uebernahmen + counters.sum_neuaufnahmen


    # Fälle beendet mit/ohne zdA

    sql = """
SELECT count(f.id) as mycount, IF(f.zday > 0, 'mit_zda', 'ohne_zda') as mygroup
FROM fall f INNER JOIN jghstat07 j ON f.id = j.fall_id
WHERE f.bgy <= {jahr} and (f.zday >= {jahr} or f.zday = 0) and j.ey = {jahr}
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_BS_Berichtsjahr_beendet_mit_zda='mit_zda',sum_BS_Berichtsjahr_beendet_ohne_zda='ohne_zda')
    counters.sum_BS_Berichtsjahr_beendet = counters.sum_BS_Berichtsjahr_beendet_mit_zda + counters.sum_BS_Berichtsjahr_beendet_ohne_zda

    # Fälle andauernd

    sql = """
SELECT count(f.id) as mycount, c.name as mygroup
FROM fall f INNER JOIN jghstat07 j ON f.id = j.fall_id
            INNER JOIN code c ON j.hda = c.id
WHERE f.bgy <= {jahr} and (f.zday >= {jahr} or f.zday = 0) and 
      j.ey IS NULL and c.name = 'ja' and j.jahr = {jahr}
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_BS_Berichtsjahr_andauernd='ja')

    # BS fehlt

    sql = """
SELECT count(f.id) as mycount, 'fehlt' as mygroup
FROM fall f LEFT OUTER JOIN jghstat07 j ON f.id = j.fall_id
WHERE f.bgy <= {jahr} and (f.zday >= {jahr} or f.zday = 0) and 
      j.id IS NULL
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_BS_fehlt='fehlt')

    # BS sonstige

    sql = """
SELECT count(f.id) as mycount, IF(f.zday > 0, 'mit_zda', 'ohne_zda') as mygroup
FROM fall f INNER JOIN jghstat07 j ON f.id = j.fall_id
            INNER JOIN code c ON j.hda = c.id
WHERE f.bgy <= {jahr} and (f.zday >= {jahr} or f.zday = 0) and 
      ((j.ey IS NOT NULL and j.ey != {jahr}) OR
      (j.ey IS NULL and j.jahr != {jahr}))
GROUP BY mygroup
""".format(jahr=jahr)
    res = getDBHandle().query(sql, dictionary=True)
    get_count(res, counters, sum_BS_sonstige_zda='mit_zda',sum_BS_sonstige_ohne_zda='ohne_zda')
    counters.sum_BS_sonstige = counters.sum_BS_sonstige_zda + counters.sum_BS_sonstige_ohne_zda

    # assert (
    #     counters.sum_BS_Berichtsjahr_beendet +
    #     counters.sum_BS_Berichtsjahr_andauernd +
    #     counters.sum_BS_fehlt +
    #     counters.sum_BS_sonstige
    #     == 
    #     counters.sum_faelle
    #     )
    # print
    # print 'Berichtsjahr: ', berichtsjahr
    # print '======================'
    # print
    # for k,v in counters.__dict__.items():
    #     print k,v
    return counters




def check_gef8a(jgh_list):
    """Diese Prüfung wird hier gemacht, weil im Jahre 2012 möglicherweise
Fälle bereits abgeschlossen wurden inklusive Bundesstatistik bevor dieses Update (EBKuS 4.5)
mit der neuen Frage bzgl. der Gefährdungseinschätzung installiert war."""
    fallnummern_ohne_gef8a = []
    for jgh in jgh_list:
        #print 'CHECK_GEF8A:', jgh.get('gef8a')
        if not jgh.get('gef8a'):
            fallnummern_ohne_gef8a.append(jgh.get('fall_fn'))
    if fallnummern_ohne_gef8a:
        raise EE("Für folgende Fallnummern fehlt Angabe "
"\"Einleitung der Hilfe auf Grund einer vorangegangenen "
"Gefährdungseinschätzung gemäß § 8a Absatz 1 SGB VIII\": %s"
% ', '.join(fallnummern_ohne_gef8a))


def jghexport(jahr, andauernd):
    """Erzeugt den Datensatz für die Jugendhilfestatistik.

    Es werden zwei Strings zurückgegeben, der eigentliche Datensatz
    und ein Kontrolldatensatz mit Zeilenumbrüchen und vorangesteller
    Fallnummer und Mitarbeiterkennung.
    """
    jgh_laufende_nummer_setzen()
    if jahr >= 2007:
        if jahr >= 2017:
            get_datensatz = get_datensatz_ab_2017
        elif jahr >= 2016:
            get_datensatz = get_datensatz_ab_2016
        else:
            get_datensatz = get_datensatz_ab_2007
        if andauernd:
            #jghl = Jugendhilfestatistik2007List(where = 'ey is NULL and bgy <= %s' % jahr, order = 'lnr')
            jghl = Jugendhilfestatistik2007List(where = 'ey is NULL and jahr = %s' % jahr, order = 'lnr')
        else:
            where = 'ey=%s' % jahr
            where_fall = " and fall.zday > 0"
            where_kein_fall = " and fall_id IS NULL"
            join = [('fall', 'jghstat07.fall_id=fall.id')]
            jghl = (Jugendhilfestatistik2007List(where=where+where_fall, join=join) +
                    # wg des Join sind Statistiken ohne Fall nicht exportiert worden!
                    # Spielt aber praktisch keine Rolle, weil zum Zeitpunkt des Exports
                    # normalerweise die Fälle nicht gelöscht sind. Es könnte aber
                    # Einzelfälle geben, wenn jemand ausdrücklich die Löschung verlangt.
                    #Jugendhilfestatistik2007List(where=where+where_kein_fall, join=join))
                    Jugendhilfestatistik2007List(where=where+where_kein_fall))
            jghl.sort('lnr')
        if jahr >= 2012:
            check_gef8a(jghl)
    else:
        jghl = JugendhilfestatistikList (where = 'ey =  %s' % jahr, order = 'lnr')
        get_datensatz = get_datensatz_bis_2006
    daten_saetze = []
    log_daten_saetze = []
    log_header = """Erzeugen der Datei mit den Datensätzen für die Jugendhilfestatistik für %s
%s
Erstellt am: %s
Datei: %s
Anzahl der Datensätze: %s

Datensätze (vorangestellt zusätzlich Fallnummer und Mitarbeiterkennung):

""" % (jahr,
       andauernd and "(andauernde Fälle)" or '',
       #time.strftime("%c", time.localtime(time.time())),
       today(),
       get_export_datei_name(jahr, False, andauernd),
       len(jghl))
    for record in jghl:
        daten_satz, log_daten_satz = get_datensatz(record)
        # Test neue Klasse mit Datensatzgenerierung
        if get_datensatz is get_datensatz_ab_2017:
            # TODO doppelter Check kann bald raus, wenn keine Probleme auftreten.
            daten_satz_new, log_daten_satz_new = JGHDatensatz(record).get_datensatz_ab_2017()
            assert daten_satz == daten_satz_new
            assert log_daten_satz == log_daten_satz_new
        daten_saetze.append(daten_satz)
        log_daten_saetze.append(log_daten_satz)
    return ''.join(daten_saetze), log_header + ''.join(log_daten_saetze)

def get_export_datei_name(jahr, log, andauernd):
    log = log and '_log' or ''
    t = today()
    if andauernd:
        date_and_type = "_am_%04d-%02d-%02d_andauernd" % (t.year, t.month, t.day)
    else:
        date_and_type = "_am_%04d-%02d-%02d_abgeschlossen" % (t.year, t.month, t.day)
    name = "jgh_%s%s%s.txt" % (jahr, date_and_type, log)
    return name

def get_datensatz_bis_2006(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz für einen Fall

    Zurückgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.
    """
    l = [
      check_code(r['rbz__code'], 1),
      check_code(r['kr__code'][1:3], 2),
      check_code(r['gm__code'], 3),
      check_code(r['gmt__code'], 3),
      "%05d" % r['lnr'],
      check_code(r['traeg__code'], 1),
      "%02d" % r['bgm'],
      "%d" % r['bgy'],
      "%02d" % r['em'],
      "%d" % r['ey'],
      check_code(r['bgr__code'], 1),
      check_code(r['gs__code'], 1),
      check_code(r['ag__code'], 1),
      check_code(r['fs__code'], 2),
      check_code(r['hke__code'], 1),
      check_gsa(r['gsa'], r['gsu__code']),
      check_code(r['zm__code'], 1),
      check_code(r['ba0__code'], 1, '0'),
      check_code(r['ba1__code'], 1, '0'),
      check_code(r['ba2__code'], 1, '0'),
      check_code(r['ba3__code'], 1, '0'),
      check_code(r['ba4__code'], 1, '0'),
      check_code(r['ba5__code'], 1, '0'),
      check_code(r['ba6__code'], 1, '0'),
      check_code(r['ba7__code'], 1, '0'),
      check_code(r['ba8__code'], 1, '0'),
      check_code(r['ba9__code'], 1, '0'),
      check_code(r['schw__code'], 1),
      check_code(r['fbe0__code'], 1, '0'),
      check_code(r['fbe1__code'], 1, '0'),
      check_code(r['fbe2__code'], 1, '0'),
      check_code(r['fbe3__code'], 1, '0'),
      ' '*28,
    ]
    daten_satz = ''.join(l)
    assert len(daten_satz) == 80
    log_daten_satz = "%-10s%-10s%s\r\n" % (r['fall_fn'], r['mit_id__ben'], daten_satz[:54])
    return daten_satz, log_daten_satz
    
def get_datensatz_ab_2007(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz für einen Fall

    Zurückgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.
    """
    gs = config.JGH_GEMEINDESCHLUESSEL
    if gs == 'blank':
        gs = '   '
    elif not gs:
        gs = '000'
    d = [' ']*210      # Datensatz
    _set(d, 1, 1, 'A')
    _set(d, 2, 3, int(r['land__code']), "%02d")
    #_set(d, 4, 4, '0') # Regierungsbezirk jetzt beim Kreis mit drin
    _set(d, 4, 6, int(r['kr__code']), "%03d")
    _set(d, 7, 9, gs) # Wird wohl nicht ausgewertet??
    _set(d, 10, 15, int(r['einrnr__code']), "%06d")
    _set(d, 16, 20, r['lnr'], '%05d')
    _set(d, 21, 22, r['bgm'], '%02d')
    _set(d, 23, 26, r['bgy'], '%04d')
    _set(d, 27, 27, '%1s' % r['zustw'])
    _set(d, 28, 29, r['hilf_art__code'])
    _set(d, 30, 31, r['hilf_ort__code'])
    _set(d, 32, 33, r['traeger__code'])
    _set(d, 34, 34, r['gs__code'])
    _set(d, 35, 36, r['gem'], '%02d')
    _set(d, 37, 40, r['gey'], '%04d')
    _set(d, 41, 112, ' '*(112-41+1)) # keine andere Kinder
    _set(d, 113, 114, r['aort_vor__code'])
    _set(d, 115, 115, r['sit_fam__code'])
    _set(d, 116, 116, r['ausl_her__code'], None, {'0': ' '})
    _set(d, 117, 117, r['vor_dt__code'], None, {'0': ' '})
    _set(d, 118, 118, r['wirt_sit__code'], None, {'0': ' '})
    _set(d, 119, 119, r['aip__code'])
    _set(d, 120, 120, r['ees__code'])
    _set(d, 121, 121, r['va52__code'])
    _set(d, 122, 122, r['rgu__code'])
    _set(d, 123, 123, r['hda__code'])
    _set(d, 124, 126, r['nbkakt'], '%03d', {None: ' '*3})
    _set(d, 127, 130, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 131, 132, r['gr1__code'])
    _set(d, 133, 134, r['gr2__code'], None, {None: ' '*2})
    _set(d, 135, 136, r['gr3__code'], None, {None: ' '*2})
    _set(d, 137, 138, r['em'], '%02d', {None: ' '*2})
    _set(d, 139, 142, r['ey'], '%04d', {None: ' '*4})
    _set(d, 143, 145, r['nbkges'], '%03d', {None: ' '*3})
    _set(d, 146, 146, r['lbk6m__code'], None, {None: ' '})
    _set(d, 147, 150, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 151, 152, r['grende__code'], None, {None: ' '*2})
    _set(d, 153, 154, r['aort_nac__code'], None, {None: ' '*2})
    _set(d, 155, 155, r['unh__code'], None, {None: ' '})
    ags = plz = ort = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(r)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        if ags:
            plz = ort = '' # plz, ort nur wenn Gemeindeschlüssel unbekannt ist.
    _set(d, 156, 163, ags, '%8s', {None: ' '*8}) # Gemeindeschlüssel für Orte außerhalb des Kreises
    _set(d, 164, 168, plz, '%5s', {None: ' '*5}) # plz ausserhalb
    _set(d, 169, 208, ort, '%-40s', {None: ' '*40}) # ort ausserhalb
    if r['jahr'] >= 2012:
        _set(d, 209, 209, r['gef8a__code'])
        _set(d, 210, 211, '\r\n') # neu: newline zwischen den Datensätzen
        datensatz = ''.join(d)
        assert len(datensatz) == 211
    else:
        _set(d, 209, 210, '\r\n') # neu: newline zwischen den Datensätzen
        datensatz = ''.join(d)
        assert len(datensatz) == 210
    if datensatz[122] == '1': # Hilfe dauert an
        assert datensatz[137:155] == ' '*18
    else:
        assert datensatz[123:126] == ' '*3
    log_datensatz = "%-14s%-10s%s" % (r['fall_fn'], r['mit_id__ben'], datensatz)
    return datensatz, log_datensatz

def get_datensatz_ab_2016(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz für einen Fall

    Zurückgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.

    Geändert ab 2016: 20-stellige Kennnummer von Position 21 bis 40 eingefügt.
                      Alle folgende Positionen verschieben sich daher um 20 Stellen.
                      Als Kennnummer tragen wir die Fallnummer ein. Dient nur zur Identifikation
                      eines Falles für evt. Rückfragen, was vorher die Laufende Nummer leistete,
                      die es allerdings auch noch immer gibt.
    """
    gs = config.JGH_GEMEINDESCHLUESSEL
    if gs == 'blank':
        gs = '   '
    elif not gs:
        gs = '000'
    d = [' ']*210      # Datensatz
    _set(d, 1, 1, 'A')
    _set(d, 2, 3, int(r['land__code']), "%02d")
    #_set(d, 4, 4, '0') # Regierungsbezirk jetzt beim Kreis mit drin
    _set(d, 4, 6, int(r['kr__code']), "%03d")
    _set(d, 7, 9, gs) # Wird wohl nicht ausgewertet??
    _set(d, 10, 15, int(r['einrnr__code']), "%06d")
    _set(d, 16, 20, r['lnr'], '%05d')
    _set(d, 21, 40, r['fall_fn'], '%-20s') # Neu 2016

    _set(d, 21+20, 22+20, r['bgm'], '%02d')
    _set(d, 23+20, 26+20, r['bgy'], '%04d')
    _set(d, 27+20, 27+20, '%1s' % r['zustw'])
    _set(d, 28+20, 29+20, r['hilf_art__code'])
    _set(d, 30+20, 31+20, r['hilf_ort__code'])
    _set(d, 32+20, 33+20, r['traeger__code'])
    _set(d, 34+20, 34+20, r['gs__code'])
    _set(d, 35+20, 36+20, r['gem'], '%02d')
    _set(d, 37+20, 40+20, r['gey'], '%04d')
    _set(d, 41+20, 112+20, ' '*(112-41+1)) # keine andere Kinder
    _set(d, 113+20, 114+20, r['aort_vor__code'])
    _set(d, 115+20, 115+20, r['sit_fam__code'])
    _set(d, 116+20, 116+20, r['ausl_her__code'], None, {'0': ' '})
    _set(d, 117+20, 117+20, r['vor_dt__code'], None, {'0': ' '})
    _set(d, 118+20, 118+20, r['wirt_sit__code'], None, {'0': ' '})
    _set(d, 119+20, 119+20, r['aip__code'])
    _set(d, 120+20, 120+20, r['ees__code'])
    _set(d, 121+20, 121+20, r['va52__code'])
    _set(d, 122+20, 122+20, r['rgu__code'])
    _set(d, 123+20, 123+20, r['hda__code'])
    _set(d, 124+20, 126+20, r['nbkakt'], '%03d', {None: ' '*3})
    _set(d, 127+20, 130+20, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 131+20, 132+20, r['gr1__code'])
    _set(d, 133+20, 134+20, r['gr2__code'], None, {None: ' '*2})
    _set(d, 135+20, 136+20, r['gr3__code'], None, {None: ' '*2})
    _set(d, 137+20, 138+20, r['em'], '%02d', {None: ' '*2})
    _set(d, 139+20, 142+20, r['ey'], '%04d', {None: ' '*4})
    _set(d, 143+20, 145+20, r['nbkges'], '%03d', {None: ' '*3})
    _set(d, 146+20, 146+20, r['lbk6m__code'], None, {None: ' '})
    _set(d, 147+20, 150+20, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 151+20, 152+20, r['grende__code'], None, {None: ' '*2})
    _set(d, 153+20, 154+20, r['aort_nac__code'], None, {None: ' '*2})
    _set(d, 155+20, 155+20, r['unh__code'], None, {None: ' '})
    ags = plz = ort = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(r)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        if ags:
            plz = ort = '' # plz, ort nur wenn Gemeindeschlüssel unbekannt ist.
    _set(d, 156+20, 163+20, ags, '%8s', {None: ' '*8}) # Gemeindeschlüssel für Orte außerhalb des Kreises
    _set(d, 164+20, 168+20, plz, '%5s', {None: ' '*5}) # plz ausserhalb
    _set(d, 169+20, 208+20, ort, '%-40s', {None: ' '*40}) # ort ausserhalb
    _set(d, 209+20, 209+20, r['gef8a__code'])
    _set(d, 210+20, 211+20, '\r\n') # neu: newline zwischen den Datensätzen
    datensatz = ''.join(d)
    assert len(datensatz) == 211+20
    if datensatz[122+20] == '1': # Hilfe dauert an
        assert datensatz[137+20:155+20] == ' '*18
    else:
        assert datensatz[123+20:126+20] == ' '*3
    log_datensatz = "%-14s%-10s%s" % (r['fall_fn'], r['mit_id__ben'], datensatz)
    return datensatz, log_datensatz

def get_datensatz_ab_2017(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz für einen Fall

    Zurückgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.

    Geändert ab 2017: Neues Item in Spalte 230 Inobhutnahme
    """
    gs = config.JGH_GEMEINDESCHLUESSEL
    if gs == 'blank':
        gs = '   '
    elif not gs:
        gs = '000'
    d = [' ']*210      # Datensatz
    _set(d, 1, 1, 'A')
    _set(d, 2, 3, int(r['land__code']), "%02d")
    #_set(d, 4, 4, '0') # Regierungsbezirk jetzt beim Kreis mit drin
    _set(d, 4, 6, int(r['kr__code']), "%03d")
    _set(d, 7, 9, gs) # Wird wohl nicht ausgewertet??
    _set(d, 10, 15, int(r['einrnr__code']), "%06d")
    _set(d, 16, 20, r['lnr'], '%05d')
    _set(d, 21, 40, r['fall_fn'], '%-20s') # Neu 2016

    _set(d, 21+20, 22+20, r['bgm'], '%02d')
    _set(d, 23+20, 26+20, r['bgy'], '%04d')
    _set(d, 27+20, 27+20, '%1s' % r['zustw'])
    _set(d, 28+20, 29+20, r['hilf_art__code'])
    _set(d, 30+20, 31+20, r['hilf_ort__code'])
    _set(d, 32+20, 33+20, r['traeger__code'])
    _set(d, 34+20, 34+20, r['gs__code'])
    _set(d, 35+20, 36+20, r['gem'], '%02d')
    _set(d, 37+20, 40+20, r['gey'], '%04d')
    _set(d, 41+20, 112+20, ' '*(112-41+1)) # keine andere Kinder
    _set(d, 113+20, 114+20, r['aort_vor__code'])
    _set(d, 115+20, 115+20, r['sit_fam__code'])
    _set(d, 116+20, 116+20, r['ausl_her__code'], None, {'0': ' '})
    _set(d, 117+20, 117+20, r['vor_dt__code'], None, {'0': ' '})
    _set(d, 118+20, 118+20, r['wirt_sit__code'], None, {'0': ' '})
    _set(d, 119+20, 119+20, r['aip__code'])
    _set(d, 120+20, 120+20, r['ees__code'])
    _set(d, 121+20, 121+20, r['va52__code'])
    _set(d, 122+20, 122+20, r['rgu__code'])
    _set(d, 123+20, 123+20, r['hda__code'])
    _set(d, 124+20, 126+20, r['nbkakt'], '%03d', {None: ' '*3})
    _set(d, 127+20, 130+20, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 131+20, 132+20, r['gr1__code'])
    _set(d, 133+20, 134+20, r['gr2__code'], None, {None: ' '*2})
    _set(d, 135+20, 136+20, r['gr3__code'], None, {None: ' '*2})
    _set(d, 137+20, 138+20, r['em'], '%02d', {None: ' '*2})
    _set(d, 139+20, 142+20, r['ey'], '%04d', {None: ' '*4})
    _set(d, 143+20, 145+20, r['nbkges'], '%03d', {None: ' '*3})
    _set(d, 146+20, 146+20, r['lbk6m__code'], None, {None: ' '})
    _set(d, 147+20, 150+20, ' '*4) # irrelevant für Erziehungsberatung
    _set(d, 151+20, 152+20, r['grende__code'], None, {None: ' '*2})
    _set(d, 153+20, 154+20, r['aort_nac__code'], None, {None: ' '*2})
    _set(d, 155+20, 155+20, r['unh__code'], None, {None: ' '})
    ags = plz = ort = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(r)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        if ags:
            plz = ort = '' # plz, ort nur wenn Gemeindeschlüssel unbekannt ist.
    _set(d, 156+20, 163+20, ags, '%8s', {None: ' '*8}) # Gemeindeschlüssel für Orte außerhalb des Kreises
    _set(d, 164+20, 168+20, plz, '%5s', {None: ' '*5}) # plz ausserhalb
    _set(d, 169+20, 208+20, ort, '%-40s', {None: ' '*40}) # ort ausserhalb
    _set(d, 209+20, 209+20, r['gef8a__code'])
    _set(d, 210+20, 210+20, r['inobh42__code'])
    _set(d, 211+20, 212+20, '\r\n') # neu: newline zwischen den Datensätzen
    datensatz = ''.join(d)
    assert len(datensatz) == 212+20
    if datensatz[122+20] == '1': # Hilfe dauert an
        assert datensatz[137+20:155+20] == ' '*18
    else:
        assert datensatz[123+20:126+20] == ' '*3
    log_datensatz = "%-14s%-10s%s" % (r['fall_fn'], r['mit_id__ben'], datensatz)
    return datensatz, log_datensatz

def _set(vector, von, bis, val, template=None, substitutions=None):
    res = None
    if substitutions:
        s = substitutions.get(val)
        if s != None:
            res = s
    if res == None:
        if template:
            res = template % val
        else:
            res = val
    #print 'VAL', val
    assert len(res) == (bis + 1 - von), 'Fehler beim Export der Bundesstatistik. Spalte %s-%s' % (von, bis)
    vector[von-1:bis] = list(res)

def check_code(str, length, leer_code = None):
    assert len(str) == length
    if not leer_code is None and str == leer_code:
        str = ' '*length
    return str
    
def check_gsa(gsa, gsu):
    """Falls das gsu-Feld 1 ist (Geschwisteranzahl unbekannt), wird das
    gsa Feld immer mit Leerzeichen gefüllt. Sonst wird die Zahl aus gsa
    übernommen. """
    if gsu == '1':
        return '  1'
    else:
        return "%02d " % gsa
        
class JGHDatensatz(object):
    """Aus einer Jugendhilfestatistik2007()-Instanz einen Datensatz 
    für die statistischen Landesämter generieren.

    Ersetzt die Funktion get_datensatz_ab_2017().
    Liefert mit der Methode get_datensatz_ab_2017() identische Ergebnisse,
    ist aber auf Dauer besser zu pflegen, da alle Informationen über den Datensatz in 
    der Tabelle datensatz_info konzentriert sind.
    Nur für die neueste Datensatzdefinition ab 2017.
    Name und wireb_nr aus ebkus/wir.eb/Anlage Dateiformat Statistikdaten 2019-02-14.pdf
    """
    # name und wireb_nr spielt für den traditionalle Export der Jugendhilfestatistik
    # keine Rolle.
    # name, wireb_nr, von, bis, field, format, substitution, value
    datensatz_info = (
        (None, None, 1, 1, None, None, None, 'A'),
        (None, None, 2, 3, 'land__code', '%02d', None, None),
        (None, None, 4, 6, 'kr__code', '%03d', None, None),
        (None, None, 7, 9, 'gs', None, None, None),
        (None, None, 10, 15, 'einrnr__code', '%06d', None, None),
        ('KennNr', 1, 16, 20, 'lnr', '%05d', None, None),
        (None, None, 21, 40, 'fall_fn', '%-20s', None, None),
        ('BeginnMonatHilfe', 5, 41, 42, 'bgm', '%02d', None, None),
        ('BeginnJahrHilfe', 6, 43, 46, 'bgy', '%04d', None, None),
        ('UebernahmeVonAnderemJugendamt', 7, 47, 47, 'zustw', '%1s', None, None),
        ('ArtHilfe', 10, 48, 49, 'hilf_art__code', None, None, None),
        ('OrtDurchfuehrung', 11, 50, 51, 'hilf_ort__code', None, None, None),
        ('TraegerEinrichtung', 12, 52, 53, 'traeger__code', None, None, None),
        ('Geschlecht', 13, 54, 54, 'gs__code', None, None, None),
        ('GeburtMonat', 14, 55, 56, 'gem', '%02d', None, None),
        ('GeburtJahr', 15, 57, 60, 'gey', '%04d', None, None),
        (None, None, 61, 132, None, None, None, '                                                                        '),
        ('AufentOrtVorHilfe', 17, 133, 134, 'aort_vor__code', None, None, None),
        ('SituationHerkunft', 18, 135, 135, 'sit_fam__code', None, None, None),
        ('AuslaendHerkunft', 19, 136, 136, 'ausl_her__code', None, {'0': ' '}, None),
        ('SpracheDeutsch', 20, 137, 137, 'vor_dt__code', None, {'0': ' '}, None),
        ('SozialSicherung', 21, 138, 138, 'wirt_sit__code', None, {'0': ' '}, None),
        ('Kontaktanregung', 22, 139, 139, 'aip__code', None, None, None),
        ('EntzugSorge', 23, 140, 140, 'ees__code', None, None, None),
        ('GerichtlAnordnung', 24, 141, 141, 'va52__code', None, None, None),
        ('RichtGenehm', 25, 142, 142, 'rgu__code', None, None, None),
        ('AndauerHilfeJahresende', 26, 143, 143, 'hda__code', None, None, None),
        ('ZahlKontakte', 27, 144, 146, 'nbkakt', '%03d', {None: '   '}, None),
        (None, None, 147, 150, None, None, None, '    '),
        ('HauptgrundHilfegewaehrung', 30, 151, 152, 'gr1__code', None, None, None),
        ('ZweiterGrund', 31, 153, 154, 'gr2__code', None, {None: '  '}, None),
        ('DritterGrund', 32, 155, 156, 'gr3__code', None, {None: '  '}, None),
        ('EndeMonatHilfe', 33, 157, 158, 'em', '%02d', {None: '  '}, None),
        ('EndeJahrHilfe', 34, 159, 162, 'ey', '%04d', {None: '    '}, None),
        ('ZahlKontakteEnde', 35, 163, 165, 'nbkges', '%03d', {None: '   '}, None),
        ('LetzterKontakt', 36, 166, 166, 'lbk6m__code', None, {None: ' '}, None),
        (None, None, 167, 170, None, None, None, '    '),
        ('GrundEndeHilfe', 39, 171, 172, 'grende__code', None, {None: '  '}, None),
        ('AnschlAufenthalt', 40, 173, 174, 'aort_nac__code', None, {None: '  '}, None),
        ('NachfolgendeHilfe', 41, 175, 175, 'unh__code', None, {None: ' '}, None),
        ('AGSWohnortdesBeratenen', 2, 176, 183, 'ags', '%8s', {None: '        '}, None),
        ('PLZWohnortdesBeratenen', 3, 184, 188, 'plz', '%5s', {None: '     '}, None),
        ('WohnortdesBeratenen', 4, 189, 228, 'ort', '%-40s', {None: '                                        '}, None),
        ('Gefaehrdungseinschaetzung', 8, 229, 229, 'gef8a__code', None, None, None),
        ('Schutzmassnahme', 9, 230, 230, 'inobh42__code', None, None, None),
        (None, None, 231, 232, None, None, None, '\r\n'),
        )
    def __init__(self, jgh):
        self.jgh = jgh
        gs = config.JGH_GEMEINDESCHLUESSEL
        if gs == 'blank':
            gs = '   '
        elif not gs:
            gs = '000'
        ags = plz = ort = ''
        ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(jgh)
        if ags_plz_ort:
            ags, plz, ort = ags_plz_ort
            if ags:
                plz = ort = '' # plz, ort nur wenn Gemeindeschlüssel unbekannt ist.
        self.ags_plz_ort_gs = dict(ags=ags, plz=plz, ort=ort, gs=gs)
    def get_datensatz_ab_2017(self):
        """Liefert exakt dasselbe Ergebnis wie die alte Funktion:
        datensatz, log_datensatz
        """
        last_bis = 0 # Intervalle prüfen
        res = []
        for name, wireb_nr, von, bis, field, format, substitution, value in self.datensatz_info:
            assert von == last_bis + 1
            last_bis = bis
            data = self.get_value(field, format, substitution, value)
            assert isinstance(data, str)
            assert len(data) == (bis - von + 1)
            res.append(data)
        datensatz = ''.join(res)
        assert len(datensatz) == 232
        if datensatz[142] == '1': # Hilfe dauert an
            assert datensatz[157:175] == ' '*18
        else:
            assert datensatz[143:146] == ' '*3
        log_datensatz = "%-14s%-10s%s" % (self.jgh['fall_fn'], self.jgh['mit_id__ben'], datensatz)
        return datensatz, log_datensatz
    def get_value(self, field, format, substitution, value):
        # Konstanter Wert, steht schon in datensatz_info 
        if value is not None:
            return value
        assert field is not None
        try:
            # Einige Werte in __init__() berechnet
            value = self.ags_plz_ort_gs[field]
        except KeyError:
            # Die große Masse direkt aus der jgh-Instanz
            value = self.jgh[field]
        if substitution:
            # Falls es ein Substitution gibt, die nehmen
            res = substitution.get(value)
            if res is not None:
                return res
        # Wir müssen jetzt einen Wert haben
        assert value is not None
        if format:
            # Formatieren, falls format definiert ist
            if format.endswith('d'):
                value = int(value)
            return format % value
        return value


def get_export_datei_daten(jahr, welche):
    """Liefert nach 'name' geordnete Liste von Bunch()-Objekten mit den
    Attributen path, path_log, name, name_log, datum, count.
    """
    import datetime
    import re
    import os
    from operator import itemgetter
    from os.path import join, getmtime, exists
    from ebkus.utils import Bunch
    assert welche in ('andauernd', 'abgeschlossen')
    regex_export_datei_name = re.compile(r"jgh_(\d{4})(_am_(\d\d\d\d-\d\d-\d\d)_(andauernd|abgeschlossen))?(_log)?.txt")
    export_dir = os.path.join(config.DOCUMENT_ROOT, 'daten', 'export')
    filenames = os.listdir(export_dir)
    dateien_dict = {} # <jahr>_<datum> --> bunch
    for filename in filenames:
        try:
           groups = regex_export_datei_name.match(filename).groups()
        except:
            # keine exportdatei, sollte aber nicht existieren
            continue
        berichtsjahr, exundtyp, datum, typ, log = groups
        if typ is None:
            # die alten Namen sind alle abgeschlossen
            typ = 'abgeschlossen'
        if int(berichtsjahr) != jahr:
            continue
        if welche != typ:
            continue
        log = log if log else ''
        # Alte Namen umbenennen
        if exundtyp is None:
            # keine Exportdatum und Typ im Dateiname, also alter Dateiname
            # für abgeschlossene Fälle. --> umbenennen
            path = join(export_dir, filename)
            timestamp = getmtime(path)
            datum = str(datetime.datetime.fromtimestamp(timestamp).date())
            typ = 'abgeschlossen'
            new_filename = "jgh_{jahr}_am_{datum}_{typ}{log}.txt".format(**locals())
            new_path = join(export_dir, new_filename)
            if not exists(new_path):
                os.rename(path, new_path)
            filename = new_filename
            typ = 'abgeschlossen'
        path = join(export_dir, filename)
        # index ist für Daten und Log gleich. Infos werden also im selben Bunchobjekt abgelegt.
        index = "{jahr}_{datum}_{typ}".format(**locals())
        bunch = dateien_dict.setdefault(index, Bunch())
        # Dateiname Log-Datei Exportdatum Anzahl der Datensätze
        bunch['name'+log] = filename
        bunch.datum = datum
        bunch['path'+log] = path
        if not log:
            bunch.count = len(open(path).readlines())
    dateien = dateien_dict.values()
    dateien.sort(key=itemgetter('name'))
    #for b in dateien: print b
    return dateien

if __name__ == '__main__':
    if len(sys.argv) < 2 or sys.argv[1] in ('-h', '-help', '--help'):
        print "Aufruf: python jghexport.py <jahr> [andauernd]"
        sys.exit(1)
    try:
        jahr = sys.argv[1]
        jahr = int(jahr)
        assert jahr > 1995 and jahr < 2500
    except:
        print "***Fehler: Kein gültiges Jahr: '%s'" % jahr
        sys.exit(1)
    try:
        andauernd = sys.argv[2]
        if andauernd.startswith('a'):
            andauernd = True
        else:
            print "***Fehler: Zweiter Parameter muss mit 'a' (fuer andauernd) beginnen"
            sys.exit(1)
    except IndexError:
        andauernd = False
    print
    print "Export der Jugendhilfestatistik fuer Instanz '%s'" % init.INSTANCE_NAME
    print "   (Pfad %s)" % init.INSTANCE_HOME
    print
    print "Jahr: %s  (%s)" % (jahr, andauernd and "andauernde Faelle" or "abgeschlossene Faelle" )
    print
    from ebkus.db.sql import opendb
    opendb()
    daten_saetze, log_daten_saetze = jghexport(jahr, andauernd)

    datei_name = get_export_datei_name(jahr, False, andauernd)
    f = open(datei_name, "w")
    f.write(daten_saetze)
    f.close()
    print "ausgegeben: %s" % datei_name 

    log_datei_name = get_export_datei_name(jahr, True, andauernd)
    f = open(log_datei_name, "w")
    f.write(log_daten_saetze)
    f.close()
    print "ausgegeben: %s" % log_datei_name
    print
        
