# coding: latin-1
"""Klasse fr Neumelde- und Abschlussdaten sowie Klienten pro Mitarbeiter.

Grund fr die Klasse besteht darin, dass die Definition von Fallbeginn ber
das Datum der ersten Leistung die Sache komplizierter macht und spezielle SQL-Abfragen
erforderlich wurden fr diesen Fall 
(siehe die Klasse _NeumeldungAbschlussFBLeistungsbeginn).
"""

import sys
# import logging
from ebkus.config import config
from ebkus.db.sql import execute
from ebkus.app.ebapi import JugendhilfestatistikList, Jugendhilfestatistik2007List, \
     Zustaendigkeit, ZustaendigkeitList, AkteList, BezugspersonList, Fall, FallList, GruppeList, \
     Tabelle, Code, Feld, Mitarbeiter, MitarbeiterList, MitarbeiterGruppeList, \
     AbfrageList, Leistung, LeistungList, \
     today, cc, check_int_not_empty, check_list, \
     check_str_not_empty, EBUpdateDataError, EE, getQuartal, get_rm_datum, Date
from ebkus.html.abfragedef import Query

def get_neumelde_abschluss_daten(jahr, stelle, fallbeginn, bundesstatistik=True):
    if fallbeginn == 'fb_lbg':
        cls = _NeumeldungAbschlussFBLeistungsbeginn
    else:
        cls = _NeumeldungAbschlussFBAnmeldung
    return cls(jahr, stelle, bundesstatistik).get_neumelde_abschluss_daten()
def get_neumelde_nach_region_daten(jahr, stelle, fallbeginn):
        if fallbeginn == 'fb_lbg':
            cls = _NeumeldungAbschlussFBLeistungsbeginn
        else:
            cls = _NeumeldungAbschlussFBAnmeldung
        return cls(jahr, stelle, bundesstatistik=False).get_neumelde_nach_region_daten()
def get_mitarbeiter_ergebnisse(jahr, mitarbeiter, fallbeginn):
        if fallbeginn == 'fb_lbg':
            cls = _NeumeldungAbschlussFBLeistungsbeginn
        else:
            cls = _NeumeldungAbschlussFBAnmeldung
        return cls(jahr, bundesstatistik=False).get_mitarbeiter_ergebnisse(mitarbeiter)


class _NeumeldungAbschluss(object):
    def get_neumelde_abschluss_daten(self):
        # logging.info('BEGIN: get_neumelde_abschluss_daten')
        stelle = self.stelle
        jahr = self.jahr
        JGHList = (jahr >= 2007 and Jugendhilfestatistik2007List or JugendhilfestatistikList)
        faelle = self._get_faelle()
        # logging.info('ENDE: _get_faelle')
        # bgy <= jahr, lbgy = jahr
        # neumeldungen = FallList(where = 'bgy = %s' % jahr
        # Vorjahr dazunehmen, da Anmeldung im Vorjahr, Leistungsbeginn in diesem Jahr sein kann
        neumeldungen = [f for f in faelle if f[self.bgy] == jahr]
        # logging.info('ENDE: neumeldungen')
        asdliste = [f for f in neumeldungen
                    if f['anmeldung'] and f['anmeldung'][0]['zm__code'] == '3']
        # logging.info('ENDE: asdliste')
        if self.bundesstatistik:
            # !!! TODO falsche Bezeichung
            zdaliste = JGHList(where = 'ey = %s' % jahr
                               + ' and stz = %d' % stelle['id'],
                               order = 'em' )
            # logging.info('ENDE: zdaliste')
            hauptfallliste = JGHList(where = 'ey = %s' % jahr
                                    + ' and gfall = %d'   % cc('gfall', '1')
                                    + ' and stz = %d' % stelle['id'],
                                    order = 'em' )
            # logging.info('ENDE: hauptfallliste')
            geschwliste = JGHList(where = 'ey = %s' % jahr
                                    + ' and gfall = %d' %  cc('gfall', '2')
                                    + ' and stz = %d' % stelle['id'],
                                    order = 'em' )
            # logging.info('ENDE: geschwliste')
        else:
            zdaliste_fall = FallList(where = 'zday = %s' % jahr
                                     + ' and akte_id__stzbg = %d' % stelle['id'],
                                     order = 'zdam' )
            # logging.info('ENDE: zdaliste_fall')
        # und Leistungsbeginn <= jahr
        # alle Flle, die in jahr irgendwann mal laufend waren
        laufendliste = faelle
        # und Leistungsbeginn < jahr
        # alle Flle, die zum Jahreswechsel zu jahr laufend waren
        uebernahmen_aus_vorjahr = len([f for f in faelle if f[self.bgy] < jahr])
        # lbgm oder bgm
        neul = [n[self.bgm] for n in neumeldungen]
        # lbgm oder bgm
        asdl = [f[self.bgm] for f in asdliste]
        if self.bundesstatistik:
            zdal = [z['em'] for z in zdaliste]
            hauptf = [z['em'] for z in hauptfallliste]
            geschw = [z['em'] for z in geschwliste]
            # logging.info('VOR: _count_mit_bundesstatistik')
            return self._count_mit_bundesstatistik(laufendliste,
                uebernahmen_aus_vorjahr, neul, asdl, zdal, hauptf, geschw)
        else:
            zdal = [z['zdam'] for z in zdaliste_fall]
            # logging.info('VOR: _count_ohne_bundesstatistik')
            return self._count_ohne_bundesstatistik(laufendliste,
                uebernahmen_aus_vorjahr, neul, asdl, zdal)
    def _count_mit_bundesstatistik(self, laufendliste, uebernahmen_aus_vorjahr, 
                                   neul, asdl, zdal, hauptf, geschw):
        # def get_jgh(fall):
        #     jghs = fall['jgh07_statistiken']
        #     if not jghs:
        #         jghs = fall['jgh_statistiken']
        #     if jghs:
        #         assert len(jghs) == 1
        #         return jghs[0]
        #     return None
                
        jahr = self.jahr
        quartal1_neu = quartal1_asd = quartal1_zda = quartal1_hauptf = quartal1_geschw = 0
        quartal2_neu = quartal2_asd = quartal2_zda = quartal2_hauptf = quartal2_geschw = 0
        quartal3_neu = quartal3_asd = quartal3_zda = quartal3_hauptf = quartal3_geschw = 0
        quartal4_neu = quartal4_asd = quartal4_zda = quartal4_hauptf = quartal4_geschw = 0
        i = 1
        monats_ergebnisse = [('12/%s' % (int(jahr)-1), uebernahmen_aus_vorjahr,'','','','','','',''),]
        while i < 13:
            # i steht fr den Monat in jahr
            neumeldezahl = neul.count(i)
            asdzahl = asdl.count(i)
            zdazahl = zdal.count(i)
            hauptfzahl = hauptf.count(i)
            geschwzahl = geschw.count(i)
            laufendzahl = 0
            davonkeinzdazahl = 0
            davonheutekeinzdazahl = 0
            # immer der erste des Folgemonats
            if i == 12:
                laufend_stichtag = Date(jahr+1, 1, 1)
            else:
                laufend_stichtag = Date(jahr, i+1, 1)
            # Es geht rasend schnell, wenn man diese Schleife weglsst.
            # Kann man das noch irgendwie anders berechnen?
            # Entscheidend ist, dass f['jgh'] im Cache ist, entweder als
            # cached field, oder ber inverses Feld: jgh07_statistiken_ids
            # Sollte jetzt durch cache_related gewhrleistet sein.
            for f in laufendliste:
                # lbg
                if (f.getDate(self.bg) < laufend_stichtag and
                    f.getDate('zda') >= laufend_stichtag): # Wenn zda Date 0 ist, ist es groesser als alle anderen (siehe ebapi.Date)
                    laufendzahl +=1
                    # logging.info('VOR: f[jgh]')
                    jgh = f['jgh']
                    #jgh = get_jgh(f)
                    # logging.info('NACH: f[jgh]')
                    if jgh and jgh['ey'] == jahr and jgh['em'] == i:
                        # laufender Fall am Ende des Monats, aber 
                        # Ende der Beratung fr diesen Monat bereits eingetragen
                        davonkeinzdazahl += 1
                        if f['aktuell']:
                            davonheutekeinzdazahl += 1
            if i <= 3:
                quartal1_neu += neumeldezahl
                quartal1_asd += asdzahl
                quartal1_zda += zdazahl
                quartal1_hauptf +=  hauptfzahl
                quartal1_geschw +=  geschwzahl
            elif i <= 6:
                quartal2_neu += neumeldezahl
                quartal2_asd += asdzahl
                quartal2_zda += zdazahl
                quartal2_hauptf += hauptfzahl
                quartal2_geschw += geschwzahl
            elif i <= 9:
                quartal3_neu += neumeldezahl
                quartal3_asd += asdzahl
                quartal3_zda += zdazahl
                quartal3_hauptf += hauptfzahl
                quartal3_geschw += geschwzahl
            elif i <= 12:
                quartal4_neu += neumeldezahl
                quartal4_asd += asdzahl
                quartal4_zda += zdazahl
                quartal4_hauptf += hauptfzahl
                quartal4_geschw += geschwzahl
            monats_ergebnisse.append((i, laufendzahl, neumeldezahl, asdzahl,
                                      hauptfzahl, geschwzahl, zdazahl, 
                                      #davonkeinzdazahl)) 
                                      davonkeinzdazahl, davonheutekeinzdazahl)) 
            i = i + 1
        quartals_ergebnisse = []
        quartals_ergebnisse.append((1, quartal1_neu, quartal1_asd, quartal1_hauptf,
                                    quartal1_geschw, quartal1_zda,))
        
        quartals_ergebnisse.append((2, quartal2_neu, quartal2_asd, quartal2_hauptf,
                                    quartal2_geschw, quartal2_zda,))
        quartals_ergebnisse.append((3, quartal3_neu, quartal3_asd, quartal3_hauptf,
                                    quartal3_geschw, quartal3_zda,))
        quartals_ergebnisse.append((4, quartal4_neu, quartal4_asd, quartal4_hauptf,
                                    quartal4_geschw, quartal4_zda,))
        gesamt_ergebnisse = (len(neul), len(asdl), len(hauptf), len(geschw), len(zdal))
        return monats_ergebnisse, quartals_ergebnisse, gesamt_ergebnisse

    def _count_ohne_bundesstatistik(self, laufendliste, uebernahmen_aus_vorjahr, 
                                   neul, asdl, zdal):
        jahr = self.jahr
        quartal1_neu = quartal1_asd = quartal1_zda = 0
        quartal2_neu = quartal2_asd = quartal2_zda = 0
        quartal3_neu = quartal3_asd = quartal3_zda = 0
        quartal4_neu = quartal4_asd = quartal4_zda = 0
        i = 1
        monats_ergebnisse = [('12/%s' % (int(jahr)-1), uebernahmen_aus_vorjahr,'','','',),]
        while i < 13:
            # i steht fr den Monat in jahr
            neumeldezahl = neul.count(i)
            asdzahl = asdl.count(i)
            zdazahl = zdal.count(i)
            laufendzahl = 0
            # immer der erste des Folgemonats
            if i == 12:
                laufend_stichtag = Date(jahr+1, 1, 1)
            else:
                laufend_stichtag = Date(jahr, i+1, 1)
            for f in laufendliste:
                if (f.getDate(self.bg) < laufend_stichtag and
                    f.getDate('zda') >= laufend_stichtag):
                    laufendzahl +=1
            if i <= 3:
                quartal1_neu += neumeldezahl
                quartal1_asd += asdzahl
                quartal1_zda += zdazahl
            elif i <= 6:
                quartal2_neu += neumeldezahl
                quartal2_asd += asdzahl
                quartal2_zda += zdazahl
            elif i <= 9:
                quartal3_neu += neumeldezahl
                quartal3_asd += asdzahl
                quartal3_zda += zdazahl
            elif i <= 12:
                quartal4_neu += neumeldezahl
                quartal4_asd += asdzahl
                quartal4_zda += zdazahl
            monats_ergebnisse.append((i, laufendzahl, neumeldezahl, asdzahl,
                                      zdazahl, 
                                      )) 
            i = i + 1
        quartals_ergebnisse = []
        quartals_ergebnisse.append((1, quartal1_neu, quartal1_asd, 
                                    quartal1_zda,))
        quartals_ergebnisse.append((2, quartal2_neu, quartal2_asd, 
                                    quartal2_zda,))
        quartals_ergebnisse.append((3, quartal3_neu, quartal3_asd, 
                                    quartal3_zda,))
        quartals_ergebnisse.append((4, quartal4_neu, quartal4_asd, 
                                    quartal4_zda,))
        gesamt_ergebnisse = (len(neul), len(asdl), 
                             len(zdal))
        return monats_ergebnisse, quartals_ergebnisse, gesamt_ergebnisse

    def get_neumelde_nach_region_daten(self):
        """Konfigurationsvariable:
        neumeldungen_nach_region: name1;name2;name3
        """
        abfrage_namen = [name.strip() for name in config.NEUMELDUNGEN_NACH_REGION.split(';') if name.strip()]
        queries = []
        for n in abfrage_namen:
            abfragen = AbfrageList(where="name = '%s'" % n)
            if abfragen:
                queries.append((n, Query(abfragen[0])))
        if not queries:
            return None,None,None,None
        jahr = self.jahr
        stelle = self.stelle
        #JGHList = (jahr >= 2007 and Jugendhilfestatistik2007List or JugendhilfestatistikList)
        faelle = self._get_faelle()
        neumeldungen = [f for f in faelle if f[self.bgy] == jahr]
        neul = [n[self.bgm] for n in neumeldungen]
        # analog fr Regionen
        # fr jede Region eine Liste, jeder Eintrag entspricht einem Monat, in der die Neumeldung
        # aus dieser Region erfolgte.
        # Liste (name, monatsliste)
        regiolisten = []
        for name, query in queries:
            rl = [f[self.bgm] for f in neumeldungen if query.test(f['akte'])]
            regiolisten.append(rl)
        n = len(regiolisten) + 1 # Neumeldungen, fr jede Region eine weitere Spalte
        quartale = [([i]+[0]*n) for i in range(1,5)] # zustzlich Quartalsnummer als erstes Element
        monate =   [([i]+[0]*n) for i in range(1,13)] # zustzlich Monatsnummer als erstes Element
        for monat in monate:
            # i steht fr den Monat in jahr
            i = monat[0]
            #print 'MONAT', i, 'QUARTAL', getQuartal(i)
            #print quartale
            quartal = quartale[getQuartal(i)-1] # getQuartal: {1..12} --> {1..4}
            monat[1] = neul.count(i)
            quartal[1] += monat[1]
            for j,rl in enumerate(regiolisten):
                monat[j+2] = rl.count(i)
                quartal[j+2] += monat[j+2]
        gesamt = (len(neul),) + tuple([len(rl) for rl in regiolisten])
        return tuple(abfrage_namen), monate, quartale, gesamt
    def get_mitarbeiter_ergebnisse(self, mitarbeiter):
        a = b = c = d = e = f = 0
        jahr = self.jahr
        mitarbeiter_ergebnisse = []
        for m in mitarbeiter:
            # Methode: 1. fr jedes Element der Zustndigkeitslist das Leistungsbeginnjahr fr den Fall hinzufgen.
            #          2. die Liste dann damit filtern
            # lbgy < jahr, zustzlch, wird kleiner
            zustaendigkeiten = self._get_zustaendigkeiten(m['id'])
            # uebernahmel = ZustaendigkeitList(where = '(ey = 0 or ey >= %s) and bgy < %s' % (jahr, jahr)
            #                              + ' and bgy > 1980 and mit_id = %d ' %m['id'])
            uebernahmel = [z for z in zustaendigkeiten if z[self.bgy] < jahr]


            # bgy <= jahr, lbgy = jahr, weniger scharf, kann grer werden
            # neuel = ZustaendigkeitList(where = 'bgy = %s' % jahr
            #                            + ' and mit_id = %d ' %m['id'])
            neuel = [z for z in zustaendigkeiten if z[self.bgy] == jahr]

            # das kann bleiben, beides knnte im Vorjahr liegen, zhlen tut aber Leistungsbeginn dieses Jahr
            davon_fallbeginn = [z for z in neuel
                                if z['fall'].getDate('bg') == z.getDate('bg')]

            # lbgy <= jahr, sonst zhlts nicht als laufend, kann kleiner werden, wenn Leistungsbeginn erst im nchsten Jahr
            # laufendl = ZustaendigkeitList(where = '(ey = 0 or ey > %s) and bgy <= %s' % (jahr, jahr)
            #                              + ' and bgy > 1980 and mit_id = %d ' %m['id'])
            laufendl = [z for z in zustaendigkeiten if z['ey'] == 0 or z['ey'] > jahr]
            abgeschl = ZustaendigkeitList(where = 'ey = %s' % jahr
                                           + ' and mit_id = %d ' %m['id'])
            # langjhrige Version:
            # davon_zda = [z for z in abgeschl
            #              if z['fall'].getDate('zda') == z.getDate('e')]

            # In EBKuS ist es mglich, Zustndigkeiten einzutragen, die am selben Tag enden, an
            # dem sie begonnen haben. Insbesondere also auch fr den Tag des Fallabschlusses (zdA).
            # Es gengt daher nicht, das Datum des Zustndigkeitsende zu nehmen, um den zuletzt Zustndigen
            # zu ermitteln, denn das muss nicht eindeutig sein. Wir nehmen also alle Zustndigkeiten fr
            # einen Fall, die am Tag des zdA enden und definieren die mit der hchsten id als die letzte Zustndigkeit.
            # Damit ist genau eine letzte Zustndigkeit gegeben.
            # Die Attributfunktion zuletzt_zustaendig (siehe ebapi.py) garantiert, dass es fr jeden Fall genau einen
            # zuletzt Zustndigen gibt:
            davon_zda = [z for z in abgeschl
                         if z['fall']['zuletzt_zustaendig'] == z]
            mitarbeiter_ergebnisse.append((m['na'], len(neuel),len(davon_fallbeginn),
                                           len(laufendl), len(abgeschl), len(davon_zda), len(uebernahmel)))
            if uebernahmel or neuel or laufendl or abgeschl:
                a = a + len(neuel)
                b = b + len(davon_fallbeginn)
                c = c + len(laufendl)
                d = d + len(abgeschl)
                e = e + len(davon_zda)
                f = f + len(uebernahmel)
        gesamt = a,b,c,d,e,f
        return mitarbeiter_ergebnisse, gesamt
class _NeumeldungAbschlussFBAnmeldung(_NeumeldungAbschluss):
    def __init__(self, jahr, stelle=None, bundesstatistik=True):
        self.jahr = jahr
        self.stelle = stelle
        self.bg, self.bgy, self.bgm, self.bgd = 'bg', 'bgy', 'bgm', 'bgd'
        self.bundesstatistik = bundesstatistik
    def _get_faelle(self):
        """Alle Flle, die irgendwann in jahr laufend waren"""
        jahr = self.jahr
        faelle = FallList(where = 'bgy <= %s' % jahr
                          + ' and akte_id__stzbg = %d' % self.stelle['id']
                          + ' and (zday = 0 or zday >= %s)' % jahr,
                          order = 'bgy, bgm, bgd',
                          cache_related=True)
        return faelle
    def _get_zustaendigkeiten(self, mit_id):
        """Alle Zustndigkeiten, die irgendwann in jahr laufend waren"""
        jahr = self.jahr
        zustaendigkeiten = ZustaendigkeitList(where = '(ey = 0 or ey >= %s) and bgy <= %s' % (jahr, jahr)
                                              + ' and bgy > 1980 and mit_id = %d ' % mit_id)
        return zustaendigkeiten
class _NeumeldungAbschlussFBLeistungsbeginn(_NeumeldungAbschluss):
    def __init__(self, jahr, stelle=None, bundesstatistik=True):
        self.jahr = jahr
        self.stelle = stelle
        self.bg, self.bgy, self.bgm, self.bgd = 'lbg', 'lbgy', 'lbgm', 'lbgd'
        self.bundesstatistik = bundesstatistik
    def _get_zustaendigkeiten(self, mit_id):
        """Alle Zustndigkeiten, die irgendwann in jahr laufend waren, werden versehen mit (lbgy, lbgm, lbgd).
        lbg unterscheidet sich von bg nur dann, wenn bg vor dem Leistungsbeginn des Falles liegt.
        Wenn auch das Ende der Zustndigkeit vor Leistungsbeginn liegt, wird sie nicht gezhlt. Ansonsten
        wird es durch das Datum der ersten Leistung ersetzt.
        """
        #print 'LBG'
        jahr = self.jahr
        sql = """
SELECT z.id, z.bgy, z.bgm, z.bgd, z.ey, z.em, z.ed, l.bgy, l.bgm, l.bgd, f.id, l.id
FROM zustaendigkeit z, fall f, leistung l
WHERE (z.ey = 0 or z.ey >= %s) AND z.bgy <= %s AND z.mit_id = %s AND 
      z.fall_id = f.id AND f.id = l.fall_id
ORDER BY z.id, l.bgy, l.bgm, l.bgd""" % (jahr, jahr, mit_id)
        res = execute(sql)
        zustaendigkeiten = []
        curr = None
        for zust_id, zbgy, zbgm, zbgd, zey, zem, zed, lbgy, lbgm, lbgd, fid, lid in res:
            #print 'CURR', curr, 'DATA', zust_id, zbgy, zbgm, zbgd, zey, zem, zed, lbgy, lbgm, lbgd, fid, lid
            #assert Zustaendigkeit(zust_id)['fall'] == Fall(fid)
            #assert Leistung(lid)['fall'] == Fall(fid)
            if not zust_id == curr:
                #print 'Erste Leistung'
                # erste Leistung
                if (zey, zem, zed) >= (lbgy, lbgm, lbgd) or zey == 0:
                    zust = Zustaendigkeit(zust_id)
                    # Zustndigkeit endet nach Leistungsbeginn.
                    if (zbgy, zbgm, zbgd) < (lbgy, lbgm, lbgd):
                        # Es muss sich um die erste Zustndigkeit des Falles handeln,
                        # die zum Zeitpunkt der ersten Leistung bestand.
                        #print 'zbg < lbg, ersetzen'
                        zust['lbgy'] = lbgy
                        zust['lbgm'] = lbgm
                        zust['lbgd'] = lbgd
                    elif (zbgy, zbgm, zbgd) == (lbgy, lbgm, lbgd):
                        # Datum der ersten Leistung und der Zustndigkeit ist identisch
                        #print 'zbg == lbg, kopieren'
                        zust['lbgy'] = zbgy
                        zust['lbgm'] = zbgm
                        zust['lbgd'] = zbgd
                    elif (zbgy, zbgm, zbgd) > (lbgy, lbgm, lbgd):
                        # Zustndigkeit begann nach der ersten Leistung, kann also nicht die erste gewesen sein.
                        #print 'zbg > lbg, kopieren'
                        zust['lbgy'] = zbgy
                        zust['lbgm'] = zbgm
                        zust['lbgd'] = zbgd
                    zustaendigkeiten.append(zust)
                else:
                    # Zustndigkeit die vor Leistungsbeginn endet, entfllt.
                    #print 'ze < lbg, entfllt'
                    pass
                curr = zust_id
        return zustaendigkeiten
    def _get_faelle(self):
        """Alle Flle, die in irgendwann in jahr laufend waren, versehen mit 
        Leistungsbeginndatum (lbgy, lbgm, lbgd). Das ist die Grundgesamtheit 
        fr spezielle Teilmengen, die im Programm gebildet werden.
        """
        jahr = self.jahr
        sql = """SELECT fall.id, leistung.bgy, leistung.bgm, leistung.bgd
FROM fall, leistung, akte
WHERE fall.id = leistung.fall_id AND fall.bgy <= %s AND (fall.zday = 0 OR fall.zday >= %s) AND
      fall.akte_id = akte.id AND akte.stzbg = %s
ORDER BY fall.id, leistung.bgy, leistung.bgm, leistung.bgd""" % (jahr, jahr, self.stelle['id'])
        res = execute(sql)
        faelle = []
        curr = None
        for fall_id, lbgy, lbgm, lbgd in res: 
            if not fall_id == curr:
                # erste Leistung
                fall = Fall(fall_id)
                fall['lbgy'] = lbgy
                fall['lbgm'] = lbgm
                fall['lbgd'] = lbgd
                faelle.append(fall)
                curr = fall_id
        from ebkus.db.dbapp import cache_related
        cache_related(objekte=faelle)
        return faelle
