# coding: latin-1

"""PDF-Generator fr die Benutzung durch andere Module.
"""

import os
import re
from cStringIO import StringIO
from functools import partial
import reportlab
assert reportlab.Version >= '3.0', "Zu niedrige Reportlabversion: {}".format(reportlab.Version)
from reportlab.platypus import SimpleDocTemplate, Paragraph, PageBreak, Spacer, Flowable
from reportlab.platypus.tables import Table, TableStyle
from reportlab.lib import colors
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import inch, cm, mm


class MySimpleDocTemplate(SimpleDocTemplate):
    def afterFlowable(self, flowable):
        """Jedes Flowable prfen, ob die Headerinformation gewechselt hat.
        Wenn ja, das header-Attribut von self setzen, das in self.header()
        abgefragt wird.
        """
        #raise Exception('ADASDASD')
        #print flowable, getattr(flowable, 'header_right', 'None')
        try:
            self.header_right = flowable.header_right
        except:
            pass
    

class PDF(object):
    """Klasse zur Erzeugung eines PDF-Dokuments mit reportlab.platypus
    
    Zur Verwendung bedarf es noch zweier Unterklassen:
    - CustomPDF (Unterklasse von PDF) mit den zu verwendenden Layoutparametern
    - TestDokument (Unterklasse von CustomPDF) mit einer add_content()-Methode,
      die die eigentlichen Inhalte hinzufgt.

    Anwendung:
    doc = TestDokument()
    pdf_byte_string = doc.create_pdf()
    open("TestDokument.pdf", 'wb').write(pdf_byte_string)
    """
    def __init__(self):
        self.flowables = []
        self.stylesheet = getSampleStyleSheet()
        self.set_paragraph_styles()
        self.set_table_styles()
    def create_pdf(self, path=None):
        """Erzeugt PDF als Bytestring, der dann in eine Datei 
        geschrieben werden kann oder in einer HTTP-Response 
        an den Browser geschickt werden kann.
        Falls path angegeben wird, steht das Ergebnis in der benannten Datei.
        Zurckgegeben wird dann der Pfad (also path).
        """
        if path:
            storage = path
        else:
            storage = StringIO()
        self.doc = MySimpleDocTemplate(storage, 
                                     pagesize=self.pagesize,
                                     leftMargin=self.lm,
                                     rightMargin=self.rm,
                                     topMargin=self.tm,
                                     bottomMargin=self.bm,
                                     showBoundary=self.sb
                                     )
        self.add_content()
        self.build()
        if path:
            return path
        else:
            pdf_value = storage.getvalue()
            storage.close()
            return pdf_value
    def set_paragraph_styles(self):
        """Benutzerdefinierte ParagraphStyles in die Standardstylesheet
        einfgen und als Attribut (mit gleichem Namen) des Dokuments
        zur Verfgung stellen
        """
        for name, parent, params in self.extra_paragraph_styles:
            if name not in self.stylesheet:
                self.stylesheet.add(
                    ParagraphStyle(name=name,
                                   parent=self.stylesheet[parent],
                                   **params))
            setattr(self, name, self.stylesheet[name])
    def set_table_styles(self):
        """Benutzerdefinierte TableStyles als Attribute des Dokuments
        zur Verfgung stellen.
        """
        for name, style in self.table_styles:
            setattr(self, name, style)
    def get_table_style(self, name):
        return TableStyle(getattr(self, name).getCommands())
    def addp(self, text, stylename, **kw):
        """Hilfsmethode um vereinfacht Abstze mit Style hinzuzufgen."""
        p = Paragraph(text, self.stylesheet[stylename], **kw)
        self.flowables.append(p)
    def addf(self, flowable):
        """Hilfsmethode um Flowables hinzuzufgen."""
        self.flowables.append(flowable)
    def addt(self, data, headerstyle=None, datastyle=None, colWidths=None, 
             extratablestyles=[], encoding=None):
        """Hilfsmethode um Standardtabelle hinzuzufgen."""
        hstyle = headerstyle if headerstyle else self.HeaderCell
        dstyle = datastyle if datastyle else self.DataCell
        if encoding:
            def decode_(s):
                if not isinstance(s, unicode):
                    return s.decode(encoding)
                return s
            data = self.do_for_elements([(decode_, (0,0), (-1,-1))], data)
        data = self.do_for_elements([(self.para(hstyle), (0,0), (-1, 0))], data)
        data = self.do_for_elements([(self.para(dstyle), (0,1), (-1, -1))], data)
        table = Table(data, colWidths=colWidths)
        #print self.StandardTable.getCommands()
        table_style = self.get_table_style('StandardTable')
        for style in extratablestyles:
            table_style.add(*style)
        table.setStyle(table_style)
        self.flowables.append(table)
    def addpropt(self, data, table_style):
        """Gerade Spalten sind Attributname, ungerade Spalten sind Attributwerte."""
        dstyle = self.DataCell
        table_data = self.wrap_data_with_style([(1,0,1,-1, dstyle),
                                                (3,0,3,-1, dstyle),
                                                (5,0,5,-1, dstyle),
                                                ], data)
        table = Table(table_data)
        table.setStyle(getattr(self, table_style))
        self.flowables.append(table)
    def addpropt1(self, data, colWidths=None):
        """Erste Spalte sind Attributnamen, zweite Spalte sind Attributwerte.
        Erste Zeile als berschrift formatieren, nicht Teil des Grids.
        """
        hstyle = self.HeaderCell
        dstyle = self.DataCell
        data = self.do_for_elements([(self.para(hstyle), (0,0), (-1, 0))], data)
        data = self.do_for_elements([(self.para(dstyle), (1,1), (-1, -1))], data)
        style = self.get_table_style('PropertyTable1')
        table = Table(data, colWidths=colWidths)
        table.setStyle(style)
        self.flowables.append(table)
        
    def myFirstPage(self, canvas, doc):
        """Seitenkonstanten fr die erste Seite des Dokuments"""
        canvas.saveState()
        if self.header_on_first_page:
            self.header(canvas, doc)
        self.footer(canvas, doc)
        canvas.restoreState()
    def myLaterPages(self, canvas, doc):
        """Seitenkonstanten fr alle folgenden Seiten des Dokuments"""
        canvas.saveState()
        self.header(canvas, doc)
        self.footer(canvas, doc)
        canvas.restoreState()
    def build(self):
        self.doc.build(self.flowables, 
                       onFirstPage=self.myFirstPage,
                       onLaterPages=self.myLaterPages,
                       )
    def footer(self, canvas, doc):
        """Seitenfu schreiben bestehend aus Line und darunter mittig die Seitenzahl"""
        canvas.setFont(self.footer_font, self.footer_fontsize)
        if self.bottomline:
            canvas.line(self.lm, self.bm, self.pw - self.rm, self.bm)
        if self.pagenumber:
            canvas.drawCentredString(self.pw/2.0, self.bm - 5*mm, "- %s -" % doc.page)
    def header(self, canvas, doc):
        """Seitenkopf schreiben bestehend aus Line und darber Text links-, mitte- und rechtsbndig"""
        canvas.setFont(self.header_font, self.header_fontsize)
        x = 2
        if self.topline:
            canvas.line(self.lm, self.ph - self.tm, self.pw - self.rm, self.ph - self.tm)
        if self.header_left:
            canvas.drawString(self.lm, self.ph - self.tm + x*mm,  self.header_left)
        if self.header_center:
            canvas.drawCentredString(self.pw/2, self.ph - self.tm + x*mm, self.header_center)
        if self.header_right:
            try: header = self.doc.header_right
            except: header = self.header_right
            canvas.drawRightString(self.pw - self.rm, self.ph - self.tm + x*mm,  header)
    def wrap_data_with_style(self, specs, data):
        """Selektiv die Zellen einer Datentabelle in einem ParagraphStyle wrappen.

        Beispiel:
        self.wrap_data_with_style(specs=[(0,0,-1, 0, self.HeaderCell),
                                         (0,1,-1,-1, self.DataCell)],
                                   data=[('Name', 'Geburtsdatum', 'Geschlecht'),
                                         ('Thomas', '20.01.1998', 'mnnlich')])
        
        data:  Die Daten fr eine Tabelle reprsentiert als Liste von Zeilen mit gleicher Lnge.
        """
        lx = len(data[0]) # col
        ly = len(data)    # row
        # Kopie
        data_new = [[x for x in y] for y in data]
        def wrap_one_rectangle(x0, y0, x1, y1, style):
            """
            x0, y0: Koordinaten der linken oberen Ecke. (0,0) ist die linke obere Ecke.
            x1, y1: Koordinaten der rechten unteren Ecke, ausgedrckt relativ zu rechten 
                    unteren Ecke in negativen Zahlen. (-1,-1) ist die rechte untere Ecke
                    von data.
            Alle Zellen in diesem Rechteck werden mit style versehen.
            Alle brigen Zelleninhalte werden unverndert gelassen.
            """
            x1abs = (x1 if x1 >= 0 else (lx+x1))
            y1abs = (y1 if y1 >= 0 else (ly+y1))
            # print 'STYLE: ', style.name
            # print 'dimension     {}  {}'.format(lx, ly)
            # print 'links   oben: {}, {}'.format(x0, y0)
            # print 'rechts unten: {}, {} ({}, {})'.format(x1abs, y1abs, x1, y1)
            w = ' '
            for yi in range(ly):
                for xi in range(lx):
                    content = data[yi][xi]
                    if (x0 <= xi <=  x1abs and
                        y0 <= yi <= y1abs and
                        content != ''):
                        # Content vorhanden und im Rechteck: wrappen
                        data_new[yi][xi] = Paragraph(content, style)
                        w = 'w'
                    else:
                        # kein Content oder auerhalb des Rechtecks: nichts machen
                        w = ' '
                    #print "xi: {} yi: {} {} content: {}".format(xi, yi, w, data[yi][xi].encode('utf8')) 
        for x0, y0, x1, y1, style in specs:
            wrap_one_rectangle(x0, y0, x1, y1, style)
        return data_new
    def do_for_elements(self, specs, data):
        """Selektiv eine Funktion auf Zellen einer Datentabelle anwenden.

        specs ist eine Liste von Tupeln:
          (function, (x0, y0), (x1, y1))
        Falls eine Datenzelle im spezifierten Rechteck liegt, wird der Inhalt der Zelle x
        ersetzt durch function(x).

        data:  Die Daten fr eine Tabelle reprsentiert als Liste von Zeilen mit gleicher Lnge.
        """
        # Dimension der Datentabelle
        lx = len(data[0]) # col, Zahl der Spalten
        ly = len(data)    # row, Zahl der Zeilen
        # Kopie
        data_new = [[x for x in y] for y in data]
        def apply_one_spec(spec):
            """
            x0, y0: Koordinaten der linken oberen Ecke. (0,0) ist die linke obere Ecke.
            x1, y1: Koordinaten der rechten unteren Ecke, entweder absolut oder relativ 
                    zur rechten unteren Ecke. 
                    Die rechte untere Ecke hat dann z.B. die Koordinaten 
                    - absolut: (6, 4) falls lx=7 und ly=5 bzw.
                    - relativ (-1, -1) unabhngig von der Dimension der Datentabelle.
            Alle Zellen in diesem Rechteck werden durch func(x) ersetzt.
            Alle brigen Zelleninhalte werden unverndert gelassen.
            """
            func, (x0, y0), (x1, y1) = spec
            # relative Werte von hinten in absolute Werte von vorne umwandeln
            # zB: x=-2 wird dann x=5 wenn lx=7
            # index relativ von hinten:     -7 -6 -5 -4 -3 -2 -1
            # datenzeile (lx=7):            [1, 2, 3, 4, 5, 6, 7]
            # index absolut von von vorne:   0  1  2  3  4  5  6
            x1abs = (x1 if x1 >= 0 else (lx+x1))
            y1abs = (y1 if y1 >= 0 else (ly+y1))
            w = ' ' # nur fr debug
            for yi in range(ly):
                for xi in range(lx):
                    if (x0 <= xi <=  x1abs and
                        y0 <= yi <= y1abs):
                        # im Rechteck: Funktion anwenden
                        content = data[yi][xi]
                        data_new[yi][xi] = func(content)
                        w = 'w'
                    else:
                        # auerhalb des Rechtecks: nichts machen
                        w = ' '
                    #print "xi: {} yi: {} {} content: {}".format(xi, yi, w, data[yi][xi].encode('utf8')) 
        for spec in specs:
            apply_one_spec(spec)
        return data_new
    def para(self, parastyle):
        """Liefert Funktion, die einen String mit einem ParagraphStyle versieht."""
        def para(s):
            if s:
                return Paragraph(s, parastyle)
            return s
        return para


class CustomPDF(PDF):
    """Layoutparameter fr die Oberklasse PDF

    Knnen auch in einer Unterklasse berschrieben werden.
    """
    # Margins
    lm = 20*mm
    rm = 10*mm
    tm = 20*mm
    bm = 20*mm
    # Show boundary around frame
    sb = False
    # Page sizes
    pagesize = A4
    ph = A4[1] # A4 Hhe in point
    pw = A4[0] # A4 Breite in point
    topline = True # explizite Linie auf dem topMargin
    bottomline = True # explizite Linie auf dem bottomMargin
    # topline = False # explizite Linie auf dem topMargin
    # bottomline = False # explizite Linie auf dem bottomMargin
    header_on_first_page = False
    # Diese Eintrge kann man im Laufe des Auffllens verndern
    header_left = 'Beraternotizen'
    header_center = 'Fallnummer 34-2019A'
    header_right = '11-03-2020'

    # header_left = ''
    # header_center = ''
    # header_right = ''

    header_font = 'Helvetica-Oblique'
    header_fontsize = 11
    footer_font = 'Helvetica-Oblique'
    footer_fontsize = 10
    pagenumber = True
    extra_paragraph_styles = [
        ('DataCell', 'BodyText', dict(fontName = 'Helvetica',
                                    fontSize=10,
                                    leading=11)),
        ('DataCellRoman', 'Normal', dict(fontName = 'Times-Roman',
                                     fontSize=10,
                                     leading=11)),
        ('DataCellRomanBoldGreen', 'Normal', dict(fontName = 'Times-Bold',
                                                textColor='green',
                                                fontSize=10,
                                                leading=11)),

        #('HeaderCell', 'Normal', dict(fontName = 'Helvetica-Oblique',
        ('HeaderCell', 'Normal', dict(fontName = 'Helvetica-Bold',
                                      fontSize=10,
                                      leading=11)),
        ('MyHeading4', 'Heading4', dict(fontName = 'Helvetica-BoldOblique',
                                        fontSize=10,
                                        leading=11)),
        ('Normal', None, None), # schon in stylesheet drin
        ]
    table_styles = [
        ('StandardTable', 
         TableStyle([('GRID', (0,0), (-1,-1), 0.25, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ])),
        ('PropertyTablex1', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (-1,0), 'Helvetica-Oblique', 10, 11),
                     ])),
        ('PropertyTablex2', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-Oblique', 10, 11),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('FONT', (2,0), (2,-1), 'Helvetica-Oblique', 10, 11),
                     ('ALIGN', (2,0), (2,-1), 'RIGHT'),
                     ])),
        ('PropertyTablex3', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-Oblique', 10, 11),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('RIGHTPADDING', (0,0), (0,-1), 6),
                     ('FONT', (2,0), (2,-1), 'Helvetica-Oblique', 10, 11),
                     ('ALIGN', (2,0), (2,-1), 'RIGHT'),
                     ('RIGHTPADDING', (2,0), (2,-1), 6),
                     ('FONT', (4,0), (4,-1), 'Helvetica-Oblique', 10, 11),
                     ('ALIGN', (4,0), (4,-1), 'RIGHT'),
                     ('RIGHTPADDING', (4,0), (4,-1), 6),
                     ])),
        ('PropertyTable1', 
         TableStyle([('GRID', (0,1), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('VALIGN', (0,0), (0,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (0,0), (0,-1), 6),
                     ])),
        ('PropertyTable2', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('VALIGN', (0,0), (0,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (0,0), (0,-1), 6),
                     ('FONT', (2,0), (2,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (2,0), (2,-1), 'RIGHT'),
                     ('VALIGN', (2,0), (2,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (2,0), (2,-1), 6),
                     ])),
        ('PropertyTable3', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('VALIGN', (0,0), (0,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (0,0), (0,-1), 6),
                     ('FONT', (2,0), (2,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (2,0), (2,-1), 'RIGHT'),
                     ('VALIGN', (2,0), (2,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (2,0), (2,-1), 6),
                     ('FONT', (4,0), (4,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (4,0), (4,-1), 'RIGHT'),
                     ('VALIGN', (4,0), (4,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (4,0), (4,-1), 6),
                     ])),
        ('PropertyTable4', 
         TableStyle([#('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ('FONT', (0,0), (0,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (0,0), (0,-1), 'RIGHT'),
                     ('VALIGN', (0,0), (0,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (0,0), (0,-1), 6),
                     ('FONT', (2,0), (2,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (2,0), (2,-1), 'RIGHT'),
                     ('VALIGN', (2,0), (2,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (2,0), (2,-1), 6),
                     ('FONT', (4,0), (4,-1), 'Helvetica-BoldOblique', 7, 8),
                     ('ALIGN', (4,0), (4,-1), 'RIGHT'),
                     ('VALIGN', (4,0), (4,-1), 'MIDDLE'),
                     ('RIGHTPADDING', (4,0), (4,-1), 6),
                     ])),
        ('BKontTable', 
         TableStyle([#('BOX', (0,0), (-1,-1), 0.5, colors.black),
                     ('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     #('FONT', (0,0), (-1,0), 'Helvetica-Bold' ),
                     #('FONTSIZE', (0,0), (-1,0), 8),
                     #('ALIGN', (0,0), (-1,-1), 'LEFT'),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ])),
        ('bkont2', 
         TableStyle([('BOX', (0,0), (-1,-1), 0.5, colors.black),
                     ('GRID', (0,0), (-1,-1), 0.5, colors.black),
                     ('FONT', (0,0), (-1,0), 'Helvetica-Bold' ),
                     ('FONTSIZE', (0,0), (-1,0), 8),
                     #('ALIGN', (0,0), (-1,-1), 'LEFT'),
                     ('VALIGN', (0,0), (-1,-1), 'TOP'),
                     ('LEFTPADDING', (0,0), (-1,-1), 3),
                     ('RIGHTPADDING', (0,0), (-1,-1), 3),
                     ('BOTTOMPADDING', (0,0), (-1,-1), 3),
                     ('TOPPADDING', (0,0), (-1,-1), 3),
                     ])),
        ]
    def get_unicode_dict_for_ebkus_object(self, ebkus_objekt=None, 
                                          prio={}, missing={},
                                          fields=(
            'gs__name', 'fs__name', 'aufbew__name', 'verw__name', 'nobed__name',
            'na', 'vn', 'gb', 'plz', 'ort', 'str', 'hsnr', 'tl1', 'tl2', 'mail',
            'ber', 'geschw', 'no')):
        """Es gab Probleme mit zB "{gs__name}".format(**akte).
        Ich habe es nicht ganz genau analyisert, vielleicht wg Keywords und unicode.
        Hier eine ganz einfache dict mit str als Keys und Unicode als Werte.
        Fr reportlab braucht es unicode. Nebenher None durch ' ' ersetzen.
        Dann kann auch das erste Zeichen eines Strings genommen werden.

        prio: falls prio ein Wert fr ein Feld hat, wird dieser auf alle Flle genommen
        missing: falls beim ebkus_object fr ein Feld eine Exception geworfen wird oder
        das Ergebnis None ist, wird falls vorhanden der Wert aus missing genommen.
        
        Falls missing auch keinen Wert hat, ist der Wert u' '.

        return: dict mit unicode-Wert fr jeden key in fields
        """
        result_dict = {}
        def process_value(v):
            if isinstance(v, unicode):
                return v
            if v is None:
                res = ' ' # damit res[0] keine Exception wirft
            else:
                res = str(v) # Zahlen etc
            # alle ebkus-Werte kommen als latin1 aus der Datenbank
            return res.decode('latin1', 'replace')

        for f in fields:
            if f in prio:
                res = prio[f]
            else:
                try:
                    res = ebkus_objekt[f]
                    assert res is not None
                except:
                    # Fehler werden nicht erkannt!!
                    res = missing.get(f)
            result_dict[f] = process_value(res)
            assert result_dict[f] is not None
        return result_dict
    def formatzelle(self, templates, ebkus_objekt=None, 
                    prio={}, missing={}, formatlabels=False):
        """Eine Liste von templates mit Werten aus ebkus_objekt fllen.

        Diese kleine Funktion lst zwei Problem:
        1. "{fall__akte__na}".format(**fall) funktioniert nicht,
        weil der **-Operator nicht die erweiterte Feldsyntax kennt.
        2. Fr Reportlab wird unicode bentigt.
        
        templates: Liste von strings, die formatiert werden sollen
        ebkus_objekt: dafr sollen die Felder dieses Objekts eingesetzt werden,
        auch die erweiterten.
        prio: dict mit Werten, die vor ebkus_objekt genommen werden
        missing: dict mit Werten, die genommen werden ebkus_object-Wert 
        None ist oder ganz fehlt.
        """
        # Feldname aus dem Template holen
        m = re.compile(r'{([_a-zA-Z][_a-zA-Z0-9]*)[^}]*}')
        fields_in_templates = []
        for t in templates:
            fields_in_templates += m.findall(t)
        ueo = self.get_unicode_dict_for_ebkus_object(ebkus_objekt=ebkus_objekt, 
                                                     prio=prio,
                                                     missing=missing,
                                                     fields=fields_in_templates)
        res = []
        for t in templates:
            if isinstance(t, unicode):
                tu = t
            else:
                s = str(t)
                tu = s.decode('latin1')
            # template is jetzt unicode, das ueo-Objekt liefert nur unicode
            # also ist das Ergebnis unicode
            # print ' TU'
            # print 'UEO', ueo
            res.append(tu.format(**ueo))
        if formatlabels:
            res1 = []
            for s in res:
                if ':' in s:
                    label, rest = s.split(':',1)
                    s = u"<i><font size=8>{label}:</font></i>{rest}".format(**locals())
                res1.append(s)
            res = res1
        return res


        
class TestDokument(CustomPDF):
    """Erzeugt PDF-Dokument mit 
    - den Inhalten aus der hier definierten add_content()-Methode,
      die von der Oberklasse aufgerufen wird,
    - den Layoutparametern der Oberklasse CustomPDF und
    - mit den Prozeduren der Oberklasse PDF, die reportlab.platypus verwendet.
    """
    def add_content(self):
        self.flowables.append(Paragraph(u"Eische: Mein Ttdkment", self.stylesheet['Title']))
        self.flowables.append(Spacer(1, 20))
        self.flowables.append((Paragraph(u"Erschtes Chapiddell", self.stylesheet['Heading3'])))
        self.flowables.append((Paragraph('Und so geht es weiter...'*30, self.stylesheet['BodyText'])))
        self.flowables.append((Paragraph(u"woodet Ckabetil", self.stylesheet['Heading3'])))
        self.flowables.append((Paragraph('Und so geht es weiter im 2. Kapitel...'*30, self.stylesheet['BodyText'])))
        self.flowables.append((Paragraph('Tabelle', self.stylesheet['Heading4'])))
        self._add_table()
        def bogus(n):
            for i in range(n):
                bogustext = ("Paragraph number %s. " % i) *20
                p = Paragraph(bogustext, self.Normal)
                self.flowables.append(p)
        for i in range(3):
            bogus(i)
            self.flowables.append(Spacer(1, 20))
            self._add_table()
            self.flowables.append(Spacer(1, 20))
        self.flowables.append(PageBreak())
        self.flowables.append(Paragraph(u"Eische: Mein Ttdkment", self.stylesheet['Heading1']))
        self.flowables.append(Spacer(1, 20))
        self._add_table()
        self.flowables.append(Spacer(1, 40))
        self.flowables.append((Paragraph('Und <b>Schluss!</b>', self.stylesheet['BodyText'])))
    def _add_table(self):
        headers = [('Datum', 'Mitarbeiter', 'Klienten', 'Art',
                   'Teilnehmer', 'Dauer in\nMinuten', 'BS', 'Notiz')]
        data = [('20.12.2011', 'Otto Kurz, Marianne Austerlitz, Sebastion Rikschaw',
                 u'Fritz Klein, May Gro, Stephan Lindemann', 
                 'telefonischer Kontakt (mit Beratungs- charakter)',
                 'Kind/ Jugendlicher, Geschwister', '110 / 110', '2', 
                 u'Keine Fragen weiter, aber man <b><font color="red">'
                 u'knnte hier</font> auch</b>  was viel lngeres <i>schreiben</i>'),
                ('20.12.2011', 'Otto Kurz, Marianne Austerlitz, Sebastion Rikschaw',
                 u'Fritz Klein, May Gro, Stephan Lindemann', 
                 'telefonischer Kontakt (mit Beratungs- charakter)',
                 'Kind/ Jugendlicher, Geschwister', '110 / 110', '2', 
                 'Keine Fragen weiter'),
                ]
        # HeaderCell etc. in der Klasse CustomPDF als Style definiert
        table_data = self.wrap_data_with_style([(0,0,-1, 0, self.HeaderCell),
                                                (0,1,-1,-1, self.DataCell),
                                                (1,2,-2,-1, self.DataCellRoman),
                                                (1,2,1,2, self.DataCellRomanBoldGreen),
                                                ],
                                               headers + data)

        table = Table(table_data)
        table.setStyle(self.BKontTable)
        self.flowables.append(table)


# Beratungskontakte als PDF ausgeben

from zipfile import ZipFile, ZIP_DEFLATED
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, get_akte_path, mk_akte_dir, today, cc, DokumentList
#, Akte, Dokument, Gruppendokument, Code, GruppendokumentList, DokumentList, Gruppe, today, cc, get_akte_path, get_gruppe_path
from ebkus.app import Request
from ebkus.html.viewdokument import get_permissions, AkteDokumentePDF, asciify

class KlientenakteKomplettAlsZIP(object):
    """Die komplette Klientenakte als ZIP-Datei zum Download bereitstellen.

    Das ZIP-Archiv besteht aus der generierten PDF-Klientenakte und enthlt
    zustzlich die vom Benutzer hochgeladenen Dokumente, die nicht in der
    generierten PDF-Datei enthalten sind.
    """
    def __init__(self, akte, mitarbeiter):
        self.akte = akte
        self.mitarbeiter = mitarbeiter
        self.akte_dir = get_akte_path(akte['id'])
        if not os.path.isdir(self.akte_dir):
            mk_akte_dir(akte['id'])
            assert os.path.isdir(self.akte_dir)
        name_geb = "{vorname}_{nachname}_{geb}".format(
            vorname=asciify(akte['vn']), nachname=asciify(akte['na']), geb=akte['gb'])
        self.filename_akte_pdf = "Klientenakte_{name_geb}.pdf".format(name_geb=name_geb)
        t = today()
        self.filename_akte_zip = "{jahr}-{monat:02}-{tag:02}_Klientenakte_{name_geb}.zip".format(
            jahr=t.year, monat=t.month, tag=t.day, name_geb=name_geb)
        self.path_akte_pdf = os.path.join(self.akte_dir, self.filename_akte_pdf)
        self.path_akte_zip = os.path.join(self.akte_dir, self.filename_akte_zip)
    def create_zip(self):
        self.create_klienten_akte()
        zip = ZipFile(self.path_akte_zip, 'w', ZIP_DEFLATED)
        zip.write(self.path_akte_pdf, self.filename_akte_pdf)
        for path, name in self.get_dateien():
            zip.write(path, name)
        zip.close()
        os.unlink(self.path_akte_pdf)
        return self.path_akte_zip
    def get_dateien(self):
        """Pfade der Dateien, die in das Archiv sollen:
        - die Klientenakte als PDF
        - die Nicht-Textdateien bei Klientendokumente
          (die Textdateien sind schon in der Klientenakte)
        Beraternotizen sind nicht Teil der Akte!

        Namen: das Archiv heit:
        {yyyy-mm-dd}_Klientenakte_{vorname}_{nachname}_{gb}.zip
        Klientenakte.pdf heit:
        Klientenakte_{vorname}_{nachname}_{gb}.pdf
        """
        where_art = " art != {}".format(cc('dokart', 'bnotiz'))
        # keine Beraternotizen, keine Textdokumente
        dokumente = DokumentList(
            where='fall.akte_id = %s and %s and mtyp != %s' % (self.akte['id'], where_art, cc('mimetyp','txt')), 
            join=[('fall', 'fall.id = fall_id'),],
            order='vy,vm,vd')
        dateien = [(os.path.join(self.akte_dir, dok['fname']),
                    "{name}_{id}{ext}".format(name=asciify(dok['art__name']), 
                                              id=dok['id'],
                                              ext=os.path.splitext(dok['fname'])[1]
                                              ))
                   for dok in dokumente]
        return dateien
    def create_klienten_akte(self):
        """Klientenakte in das Datenverzeichnis der Akte schreiben als
        temporrer File.
        """
        akte_pdf = KlientenaktePDF(self.akte, self.mitarbeiter)
        datei = akte_pdf.create_pdf()
        file = open(self.path_akte_pdf, 'wb')
        file.write(datei)
        file.close()
        # RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        # RESPONSE.setHeader('filename', '%s_%d.pdf' % ('akte', akte['id']))
        # RESPONSE.setBody(datei)    
        
class KlientenaktePDF(CustomPDF):
    """Den Inhalt der Klientenkarte als PDF ausgeben.

    Einbezogen werden auch die Klientendokumente in Textform (Endung .txt).
    Andere, vom Benutzer hochgeladene Klientendokumente werden nicht in
    die PDF-Datei einbezogen.
    """
    # Margins
    lm = 20*mm
    rm = 15*mm
    tm = 20*mm
    bm = 20*mm
    pagesize = A4
    topline = True # explizite Linie auf dem topMargin
    bottomline = True # explizite Linie auf dem bottomMargin
    header_on_first_page = False
    # Diese Eintrge kann man im Laufe des Auffllens verndern
    header_left = 'Klientenakte vom Datum'
    header_center = 'Name und Geburtsdatum'
    header_right = 'Stamm- und Falldaten'

    # header_left = ''
    # header_center = ''
    # header_right = ''

    header_font = 'Helvetica-Oblique'
    header_fontsize = 9
    footer_font = 'Helvetica-Oblique'
    footer_fontsize = 9
    pagenumber = True
    def __init__(self, akte, mitarbeiter):
        CustomPDF.__init__(self)
        self.akte = akte
        self.mitarbeiter = mitarbeiter
        self.faelle = akte['faelle']
        self.faelle.sort('bgy', 'bgm', 'bgd')
    def get_file_name(self):
        akte = self.akte
        name_geb = "{vorname}_{nachname}_{geb}".format(
            vorname=asciify(akte['vn']), nachname=asciify(akte['na']), geb=akte['gb'])
        return "Klientenakte_{name_geb}.pdf".format(name_geb=name_geb)
        
    def add_content(self):
        akte = self.akte
        name_geb = self.formatzelle(
            ('{name} geb. {gb}',),
            ebkus_objekt=akte)[0]
        stand = u"(Stand vom {})".format(today())
        titel = u"Klientenakte von {name_geb}".format(name_geb=name_geb)
        untertitel = stand

        self.header_left = u"Klientenakte {stand}".format(stand=stand)
        self.header_center = self.formatzelle(
            ('{name} geb. {gb}',),
            ebkus_objekt=akte)[0]
        # das ndert sich, wird von afterFlowable() aufgegriffen
        self.doc.header_right = u"Stamm- und Falldaten"
        self.header_right = u"Stamm- und Falldaten"

        self.addp(titel, "Heading1")
        self.addp(untertitel, "Heading3")
        self.addp("Stamm- und Falldaten", "Heading2")
        from ebkus.html.aktenvorblatt import AktenvorblattPDF
        av = AktenvorblattPDF(akte, noheader=True)
        av.add_content()
        self.flowables += av.flowables
        pg = PageBreak()
        # wird von afterFlowable() ausgewertet
        pg.header_right = u"Beratungskontakte"
        self.addf(pg)
        self.addp("Beratungskontakte", "Heading2")
        from ebkus.html.beratungskontakt import AkteBeratungskontaktPDF
        bk = AkteBeratungskontaktPDF(akte, self.mitarbeiter, standalone=False)
        bk.add_content()
        self.flowables += bk.flowables
        pg = PageBreak()
        pg.header_right = u"Dokumente"
        self.addf(pg)
        #self.addp("Dokumente", "Heading2")
        from ebkus.html.viewdokument import AkteDokumentePDF
        dok = AkteDokumentePDF(akte, self.mitarbeiter, beraternotizen=False)
        if dok.dokumente:
            dok.add_content()
        self.flowables += dok.flowables
        # Beraternotizen gehren wohl nicht in die Klientenakte
        # pg = PageBreak()
        # pg.header_right = u"Beraternotizen"
        # self.addf(pg)
        # #self.addp("Beraternotizen", "Heading2")
        # dok = AkteDokumentePDF(akte, self.mitarbeiter, beraternotizen=True)
        # dok.add_content()
        # self.flowables += dok.flowables
        #self.addf(PageBreak())


class akte2pdf(Request.Request):
    permissions = get_permissions()

    def processForm(self, REQUEST, RESPONSE):
        akte_id = self.form.get('akid')
        if not akte_id:
            self.last_error_message = "Keine ID fr die Akte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        akte = Akte(akte_id)
        akte_pdf = KlientenaktePDF(akte, self.mitarbeiter)
        name = akte_pdf.get_file_name()
        datei = akte_pdf.create_pdf()
        RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        #RESPONSE.setHeader('filename', '%s_%d.pdf' % ('akte', akte['id']))
        RESPONSE.setHeader('content-disposition', 'filename="{name}"'.format(name=name))
        RESPONSE.setBody(datei)    

class akte2zip(Request.Request):
    permissions = get_permissions()

    def processForm(self, REQUEST, RESPONSE):
        akte_id = self.form.get('akid')
        if not akte_id:
            self.last_error_message = "Keine ID fr die Akte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        akte = Akte(akte_id)
        akte_zip = KlientenakteKomplettAlsZIP(akte, self.mitarbeiter)
        path_akte_zip = akte_zip.create_zip()
        RESPONSE.setHeader('content-type', '%s' % 'application/zip')
        #RESPONSE.setHeader('filename', os.path.basename(path_akte_zip))
        RESPONSE.setHeader('content-disposition', 'filename="{name}"'.format(name=os.path.basename(path_akte_zip)))
        RESPONSE.setBody(open(path_akte_zip, 'rb').read())
        os.unlink(path_akte_zip)


def main():
    doc = TestDokument()
    pdf_byte_string = doc.create_pdf()
    open("TestDokument.pdf", 'wb').write(pdf_byte_string)

def main_and_show():
    doc = TestDokument()
    res = doc.create_pdf()
    open('tt.pdf', 'wb').write(res)
    #os.system('evince tt.pdf')
    os.system('evince tt.pdf &')



if __name__ == '__main__':
    #main_and_show()
    main()

