# coding: latin-1
"""Generic database adapter for MySQL. Implements only the minimum
of functionality needed in sql.py"""

import logging
import time
import re
from ebkus.config import config

import MySQLdb
from _mysql_exceptions import MySQLError as DatabaseError

ebkustype_conv = { MySQLdb.FIELD_TYPE.TINY: int,
            MySQLdb.FIELD_TYPE.SHORT: int,
            MySQLdb.FIELD_TYPE.LONG: int,
            MySQLdb.FIELD_TYPE.FLOAT: float,
            MySQLdb.FIELD_TYPE.DOUBLE: float,
            MySQLdb.FIELD_TYPE.LONGLONG: long,
            MySQLdb.FIELD_TYPE.INT24: int,
            MySQLdb.FIELD_TYPE.YEAR: int }

MySQLdb.type_conv = ebkustype_conv

def connect(host='', user='', passwd='', db=''):
    dbname = db
    #print 'DBADAPTER CONNECT:  host=%s db=%s user=%s pw=%s' % (host, dbname, user, passwd)
    try:
        db = MySQLdb.connect(host=host, user=user, passwd=passwd, db=dbname)
        logging.debug("Datenbankverbindung hergestellt: host=%s db=%s user=%s pw=*****" %
                     (host, dbname, user))
        return DBAdapter(db)
    except Exception, e:
        logging.exception("Datenbankfehler: Verbindung konnte nicht hergestellt werden")
        raise

# wird im Modul sql an- und ausgeschaltet
# TODO in config integrieren
debug = 0
class DBAdapter:
    def __init__(self, dbhandle):
        self.dbhandle = dbhandle
        self._mysqldb_needs_unicode = (MySQLdb.version_info > (1,2,1))
    def selectdb(self, db = ''):
        self.dbhandle.connect(db = config.DATABASE_NAME)

    def query(self, query, dictionary=False):
        #print query
        query = query.strip()
        try:
            from ebkus.app import protocol
            protocol.write_sql_protocol(sql=query)
        except:
            # hat fr die Funktionalitt keine Bedeutung
            logging.exception("Probleme beim SQL-Protokollieren")
        if dictionary:
            cursor = self.dbhandle.cursor(MySQLdb.cursors.DictCursor)
        else:
            cursor = self.dbhandle.cursor()
        is_select = (query[:6].lower() == 'select' or
                     query[:4].lower() == 'show')
        try:
                            
            #print 'DBAdapter', query
            logging.debug("Datenbankzugriff: \n%s", shorten(query))
            # for MySQLdb 1.2.2 MODIFE
            if self._mysqldb_needs_unicode and not isinstance(query, unicode):
                query = query.decode('latin1')
            if debug:
                query_start_time = time.time()
            e_res = cursor.execute(query)
            #logging.debug("Resultat (execute): %s", e_res)
            n_rows = 1
            if is_select:
                res = cursor.fetchall()
                n_rows = len(res)
                #logging.debug("Resultat (fetchall):\n%s", f_res)
                # schner, aber overhead fr jeden Request auch ohne debugging
                #logging.debug("Resultat (fetchall):\n%s", '\n'.join([str(e) for e in f_res]))
            else:
                res = e_res # ist ein int
            if debug:
                tdiff = (time.time() - query_start_time)*1000 # ms
                print "[%s rows %5.0f ms] %s" % (n_rows,tdiff,shorten(query))
            return res
                
        except Exception, e:
            logging.debug("Fehler beim Datenbankzugriff:", exc_info=True)
            raise

    def listtables(self):
        cursor = self.dbhandle.cursor()
        cursor.execute('SHOW tables')
        tables = map(lambda x: x[0], cursor.fetchall())
        return tables

    def listfields(self, table):
        cursor = self.dbhandle.cursor()
        cursor.execute('SHOW fields from %s' % table)
        return cursor.fetchall()

    def close(self):
        self.dbhandle.close()
        self.dbhandle = None


_match_in_clause_data = re.compile(r"\([0-9]+,[0-9,]*,[0-9]+\)", re.MULTILINE)
def shorten(query):
    """Lange IN Clause krzen fr Debugoutput"""
    matches = re.findall(_match_in_clause_data, query)
    res = query
    for match in matches:
        res = res.replace(match, match[:20]+' ... '+match[-20:])
    return res
