# coding: latin-1

class DBCache:

    def __init__(self):
        self.data = {}
        # Die Menge der Fälle, die mit cache_related() vollständig gecached wurden.
        # Wird nur von cache_related() verwendet.
        self.faelle = set()
        self.mitarbeiter_loaded = False
        self.code_loaded = False
    def clear(self):
        self.data.clear()
        self.faelle.clear()
        self.mitarbeiter_loaded = False
        self.code_loaded = False
    def clear_if_gt(self, no_items):
        if len(self.data) > no_items:
            self.clear()
            
    def undo_cached_fields(self):
        for k,d in self.data.items():
            klass = k[0]
            obj = klass()
            obj.data = d
            obj.undo_cached_fields()
            
    def get(self, object, id = None):
        id = id or object.data.get(object.primarykey)
        cache = self.data
        if id:
            data = cache.get((object.__class__, id))
            if data:
                object.data = data
                #print '_cache.get HIT'
                return object
            else:
                #print '_cache.get MISS'
                return None
        otherkeys = object.otherkeys
        data = object.data
        for i in range(len(otherkeys)):
            try:
                values = tuple(map(lambda x,data=data: data[x], otherkeys[i]))
                object.data = cache[(object.__class__, i, values)]
                return object
            except: pass
        return None
        
    def cache(self, object, update_inverses = None):
        """object in den Cache einfügen.
        update_inverses wird nur von insert() mit True
        aufgerufen.
        """
        id = object.data.get(object.primarykey)
        cache = self.data
        klass = object.__class__
        data = object.data
        otherkeys = object.otherkeys
        if id:
            cache[(klass, id)] = data
            if update_inverses:
                # zB für Fall:           akte_id (Akte, faelle)
                # zB für Zustaendigkeit: fall_id (Fall, zustaendigkeiten)
                for field, (klass, inversefield) in object.foreignfieldtypes.items():
                    fk = object.data.get(field)
                    if inversefield and fk and self.is_cached(klass, fk):
                        inverselist = klass(fk).data.get(inversefield + '_ids')
                        if not inverselist is None:
                            inverselist.append(id)
        for i in range(len(otherkeys)):
            try:
                values = tuple(map(lambda x, data=data: data[x], otherkeys[i]))
                cache[(klass, i, values)] = data
            except:
              #print 'DBCache-Exception'
                pass
                
    def uncache(self, object, update_inverses = None):
        """
        update_inverses wird nur von delete() mit True
        aufgerufen.
        """
        id = object.data.get(object.primarykey)
        cache = self.data
        klass = object.__class__
        data = object.data
        otherkeys = object.otherkeys
        if id:
            try:
                del cache[(klass, id)]
            except:
                pass # wenn nichts zu del'en gibt, ists auch nicht schlimme (hoffentlich :-)
            if update_inverses:
                for field, (klass, inversefield) in object.foreignfieldtypes.items():
                    fk = object.data.get(field)
                    if inversefield and fk:
                        inverselist = klass(fk).data.get(inversefield + '_ids')
                        if not inverselist is None:
                        ##jh: wirft sonst Exception beim delete von n-n, removeteiln
                            try:
                                inverselist.remove(id)
                            except: pass
        for i in range(len(otherkeys)):
            try:
                values = tuple(map(lambda x,data=data: data[x], otherkeys[i]))
                del cache[(klass, i, values)]
            except: pass
            
    def is_cached(self, klass, id):
        return self.data.get((klass, id))
        
    def is_on(self): return 1
    def on(self): pass
    def off(self):
        self.clear()
        self.__class__ = DummyDBCache
    def printe(self): print 'Real'

    def cache_related(self, objekte=None, beginn_jahr=None, ende_jahr=None, add_fields=[]):
        """Alle O
        objekte: Liste von Instanzen der Klassen Akte, Fall, Zustaendigkeit, 
        Fachstatistik, Jugendhilfestatistik2007,  Jugendhilfestatistik
        Alle Elemente der Liste müssen derselben Klasse angehören.
        Die Beziehungen zwischen diesen Objekten werden vollständig geladen, 
        fehlende Objekte werden nachgeladen.
        So ist jede Akte, in der das übergebene Objekt sich befindet, vollständig geladen
        (bis auf Einrichtung, Bezugsperson, Leistung und Beratungskontakt, die in der 
        Regel nur von der Klientenkarte benötigt werden, wo einzelnes Nachladen aus der 
        Datenbank für die Performance keine Rolle spielt).

        Falls objekte None ist, kann auch ein Beginnjahr und ein Endejahr angegeben werden.
        Es werden dann alle Fälle genommen mit beginn_jahr <= bgy <= ende_jahr.
        Wenn nichts angegeben wird, werden alle Fälle genommen. Das dauert etwas länger, aber vor
        allem wird dafür auch mehr Speicher benötigt.

        Es werden alle Objekte hinzugeladen, die für diese Fälle relevant
        sind, so dass keine weiteren Datenbankzugriffe nötig sind um Übersichten zu erstellen.
        Es werden die inversen Relationen zwischen Objekten dieser Klassen in den Cache eingetragen.
        Das sind Objekte folgender Klassen:
        Akte
        Fall (alle übrigen Fälle der Akte, auch wenn sie nicht in faelle enthalten sind. 
             Wichtig, um aktuellen Fall und letzte Zuständigkeit zu bestimmen)
        Zustaendigkeit
        Mitarbeiter
        JugendhilfestatistikList
        Jugendhilfestatistik2007List
        FachstatistikList

        add_fields: List von Feldnamen. Für jeden Feldname muss es eine Methode geben,
        die das Feld zu jedem Fall in self.faelle hinzufuegt.
        Bisher nur letzter_beratungskontakt_datum, damit man Fälle schnell finden kann,
        wo der letzte Kontakt schon länger zurückliegt. Bisher wurde für jeden Fall einzeln
        die Beratungskontakte aus der Datenbank geholt, was bei vielen Fällen lange dauerte.

        Falls objekte zur Klasse Fua_BS gibt es eine Sonderbehandlung, da diese nicht zu einer 
        Akte gehören ("fallunabhängig"). Es wird lediglich die Inversen mitarbeiterfuaktivitaeten
        befüllt. 
        """
        import logging
        from operator import itemgetter
        from ebkus.app.ebapi import Akte, AkteList, Fall, FallList, Jugendhilfestatistik2007, \
            Jugendhilfestatistik2007List, Jugendhilfestatistik, JugendhilfestatistikList, \
            Fachstatistik, FachstatistikList, Zustaendigkeit, ZustaendigkeitList, MitarbeiterList, \
            Anmeldung, AnmeldungList, CodeList, Fua_BS, Mitarbeiterfua_bsList
        if not self.code_loaded:
            CodeList(where='')
            self.code_loaded = True
        if not self.mitarbeiter_loaded:
            MitarbeiterList(where='')
            self.mitarbeiter_loaded = True
        if objekte is None:
            where_list = []
            if beginn_jahr:
                where_list.append('bgy >= {}'.format(beginn_jahr))
            if ende_jahr:
                where_list.append('bgy <= {}'.format(ende_jahr))
            where = ' and '.join(where_list)
            faelle = FallList(where=where)
        elif not objekte:
            # In diesem Fall macht die Methode nichts
            faelle = []
        elif objekte[0].__class__ == Fall:
            faelle = objekte # denn genau mit der Falliste fängt alles and
        elif objekte[0].__class__ == Akte:
            akte_ids = set()
            for akte in objekte:
                akte_ids.add(akte['id'])
            #akte_ids_string = ','.join(str(id) for id in akte_ids)
            faelle = FallList(where_in=('akte_id', akte_ids))
        elif objekte[0].__class__ in (Jugendhilfestatistik2007, Jugendhilfestatistik, 
                                      Fachstatistik, Zustaendigkeit, Anmeldung):
            # alle haben fall_id
            fall_ids = set()
            for obj in objekte:
                fall_id = obj['fall_id']
                if fall_id not in self.faelle:
                    fall_ids.add(fall_id)
            faelle = FallList(where_in=('id', fall_ids))
        elif objekte[0].__class__ == Fua_BS:
            # hier kommt es nur darauf an, die Inverse 'mitarbeiterfuaktivitaeten'
            # zu befüllen.
            # ids für die, wo die inverse fehlt
            fua_ids = set()
            for fua in objekte:
                if 'mitarbeiterfuaktivitaeten_ids' not in fua:
                    fua_ids.add(fua['id'])
                    fua['mitarbeiterfuaktivitaeten_ids'] = []
            mflist = Mitarbeiterfua_bsList(where_in=('fua_bs_id', fua_ids))
            for mf in mflist:
                self.data[(Fua_BS,mf['fua_bs_id'])]['mitarbeiterfuaktivitaeten_ids'].append(mf['id'])
            return
        # Ab hier gibt es eine geladene Fallliste
        getter = itemgetter('id', 'akte_id')
        akte_ids = set()
        fall_ids = set()
        for f in faelle:
            id, akte_id = getter(f)
            if id not in self.faelle: # laden nicht wiederholen falls der Fall schon gecached ist
                fall_ids.add(id)
                akte_ids.add(akte_id)
        # Ab hier muss die Liste von Fällen geladen sein, für die die Vervollständigung 
        # des Caches vorgenommen wird.
        # Es sind alles Fälle, deren Ids nicht als geladen bekannt sind, also nicht
        # in self.faelle enthalten sind.
        # Es müssen aber nicht alle Fälle einer Akte darin sein, denn die fehlenden werden
        # nachgeladen.
        # Die Ids dieser Fallliste stehen in fall_ids.
        # Die dazugehörigen Aktenids stehen in akte_ids. Diese Akten müssen nicht geladen sein,
        # sie werden im folgenden nachgeladen.
        if len(fall_ids) == 0:
            # logging.info("Cache musste nicht befüllt werden. Fallliste ist leer.")
            return
        assert len(akte_ids) > 0, 'Fälle, aber keine Akten. Darf nicht sein.'
        nfall_ids_orig = len(fall_ids)
        nakte_ids = len(akte_ids)
        # Akten laden, um fehlende Fälle einer Akte zu finden
        AkteList(where_in=('id', akte_ids))
        # Diese Fälle haben wir schon, nur die fehlenden holen
        for f in FallList(where_in=('akte_id', akte_ids), where_not_in=('id', fall_ids)):
            fall_ids.add(f['id'])
        nfall_ids = len(fall_ids)
        nzust_ids = len(ZustaendigkeitList(where_in=('fall_id', fall_ids)))
        njgh2007_ids = len(Jugendhilfestatistik2007List(where_in=('fall_id', fall_ids)))
        njgh_ids = len(JugendhilfestatistikList(where_in=('fall_id', fall_ids)))
        nfstat_ids = len(FachstatistikList(where_in=('fall_id', fall_ids)))
        nanm_ids = len(AnmeldungList(where_in=('fall_id', fall_ids)))
        #print 'nfall_ids_orig', nfall_ids_orig, 'nfall_ids', nfall_ids, 'nanm_ids', nanm_ids, 'njgh2007_ids', njgh2007_ids,'njgh_ids', njgh_ids, 'nfstat_ids', nfstat_ids
        # Inversen anlegen und befüllen
        for x in self.data:
            klass, id = x[0], x[1]
            if klass == Fall and id in fall_ids:
                falldaten = self.data[(klass,id)]
                akte_id = falldaten['akte_id']
                inverse_ids = self.data[(Akte,akte_id)].setdefault('faelle_ids', [])
                if id not in inverse_ids:
                    inverse_ids.append(id)
                # kann nur hier initialisiert werden, da unten nur noch vorkommenden Fälle auftauchen
                falldaten['zustaendigkeiten_ids'] = []
                falldaten['jgh07_statistiken_ids'] = []
                falldaten['jgh_statistiken_ids'] = []
                falldaten['fachstatistiken_ids'] = []
                falldaten['anmeldung_ids'] = []
        for x in self.data:
            klass, id = x[0], x[1]
            def insert_inverse(inverse_klass, inverse_name):
                if klass == inverse_klass:
                    fall_id = self.data[(klass,id)]['fall_id']
                    if fall_id in fall_ids:
                        # Gilt nur für unsere abgeschlossene Menge der Fälle!
                        # Nur die sind oben initialisiert und nur für die haben 
                        # wir die in Beziehung stehenden Objekte geladen.
                        inverse_ids = self.data[(Fall,fall_id)][inverse_name]
                        if fall_id not in inverse_ids:
                            inverse_ids.append(id)
            insert_inverse(Zustaendigkeit, 'zustaendigkeiten_ids')
            insert_inverse(Jugendhilfestatistik2007, 'jgh07_statistiken_ids')
            insert_inverse(Jugendhilfestatistik, 'jgh_statistiken_ids')
            insert_inverse(Fachstatistik, 'fachstatistiken_ids')
            insert_inverse(Anmeldung, 'anmeldung_ids')
        self.faelle.update(fall_ids)
        for field in add_fields:
            getattr(self, '_add_field_' + field)()
        logging.info("Cache befüllt für {} Fälle. Cachegröße insgesamt: {}".format(len(fall_ids), len(self.data)))

    def cache_beratungskontakte(self, akten):
        """Beratungskontakte cachen für alle Akten in akten.
        Nur aufrufen nach cache_related() für die betreffenden Akten und Fälle.
        """
        from ebkus.app.ebapi import Fall, FallberatungskontaktList, Beratungskontakt, BeratungskontaktList, \
            MitarbeiterberatungskontaktList
        faelle = []
        for akte in akten:
            faelle += akte['faelle']
        if not faelle:
            return
        # Es muss gelten: wenn ein Fall einer Akte in faelle ist, sind es auch
        # alle anderen Fälle derselben Akte.
        fall_ids = set(f['id'] for f in faelle)
        fall_ids_string = ','.join(str(id) for id in fall_ids)
        fblist = FallberatungskontaktList(where='fall_id IN ({})'.format(fall_ids_string))
        bklist = BeratungskontaktList(
            where='fallberatungskontakt.fall_id in ({})'.format(fall_ids_string),
            join=[('fallberatungskontakt',
                   'fallberatungskontakt.bkont_id=beratungskontakt.id')])
        for fall in faelle:
            fall['fallberatungskontakte_ids'] = []
        for fb in fblist:
            self.data[(Fall, fb['fall_id'])]['fallberatungskontakte_ids'].append(fb['id'])
        bkont_ids = [bkont['id'] for bkont in bklist]
        bkont_ids_string = ','.join(str(id) for id in bkont_ids)
        # Für bkont['faelle'] reicht das nicht, da derselbe Beratungskontakt noch in
        # anderen Fällen involviert sein kann.
        # Also  FallberatungskontaktList mit bkont_id aus bklist und fall nicht in faelle
        if bkont_ids:
            fblist_other = FallberatungskontaktList(
                where='fall_id NOT IN ({}) AND bkont_id in ({})'.format(fall_ids_string, bkont_ids_string))
            mblist = MitarbeiterberatungskontaktList(where='bkont_id in ({})'.format(bkont_ids_string))
        else:
            fblist_other = FallberatungskontaktList()
            mblist = MitarbeiterberatungskontaktList()
        for bkont in bklist:
            bkont['fallberatungskontakte_ids'] = []
            bkont['mitarbeiterberatungskontakte_ids'] = []
        for fb in fblist + fblist_other:
            self.data[(Beratungskontakt, fb['bkont_id'])]['fallberatungskontakte_ids'].append(fb['id'])
        for mb in mblist:
            self.data[(Beratungskontakt, mb['bkont_id'])]['mitarbeiterberatungskontakte_ids'].append(mb['id'])

    def _add_field_letzter_beratungskontakt_datum(self):
        """Zu jedem Fall in self.faelle ein Feld 'letzter_beratungskontakt_datum'
        hinzufügen, dass das Datum des letzten Beratungskontakt im Format dd.mm.yyyy
        enthält oder None, falls es keine Beratungskontakte gegeben hat.
        """
        from ebkus.app.ebapi import Fall
        from ebkus.db.sql import getDBHandle
        fall_ids_string = ','.join(str(id) for id in self.faelle)
        # fall_id, datum als string yyyymmdd zum Vergleich
        sql = """
SELECT f.id,
       CONCAT(LPAD(b.ky, 4, '0'),LPAD(b.km, 2, '0'),LPAD(b.kd, 2, '0'))
FROM fall f
     INNER JOIN fallberatungskontakt fb ON fb.fall_id = f.id
     INNER JOIN beratungskontakt b ON fb.bkont_id = b.id
WHERE f.id in ({})
ORDER BY fall_id, b.ky, b.km, b.kd, b.kh, b.kmin
    """.format(fall_ids_string)
        res = getDBHandle().query(sql)
        f2d = {} # fall_id --> yyyymmdd
        for fall_id, lk in res:
            if fall_id in f2d:
                v = f2d[fall_id]
                f2d[fall_id] = max(lk, v)
            else:
                f2d[fall_id] = lk
        for x in self.data:
            klass, fall_id = x[0], x[1]
            if klass == Fall:
                if fall_id in f2d:
                    datum = f2d[fall_id]
                    datum_deutsch = "{}.{}.{}".format(datum[6:8],datum[4:6],datum[:4])
                else:
                    datum_deutsch = None
                self.data[x]['letzter_beratungskontakt_datum'] = datum_deutsch


    
class DummyDBCache:

    def __init__(self): pass
    def clear(self): pass
    def clear_if_gt(self, no_items): pass
    def undo_cached_fields(self): pass
    def get(self, object, id = None): pass
    def cache(self, object, update_inverses = None): pass
    def uncache(self, object, update_inverses = None): pass
    def is_cached(self, klass, id): pass
    def is_on(self): return None
    def on(self):
        self.__class__ = DBCache
    def off(self): pass
    def printe(self): print 'DUmmy'
    def cache_related(self, **kw): pass
    def cache_beratungskontakte(self, akten): pass
    
