# coding: latin-1

import sys
from random import random

import logging
def log(s):
    logging.info(s)
    print s

def create_demo_daten(config, open_database=True, **kw):
    dd = DemoDaten(config, open_database=open_database, **kw)
    dd.create()

from ebkus.gen import test_input, test_daten

class DemoDaten(test_input.TestInput):
    seed = 101
    heute = None
    von_datum = None
    bis_datum = None
    n_akten = 500
    n_altdaten = 100
    log = log
    verbose = 1
    open_database = True
    def __init__(self, config, **kw):
        self.config = config
        for k, v in kw.items():
            setattr(self, k, v)
        from ebkus.app.ebapi import today
        if self.heute is None:
            self.heute = today()
        else:
            self.set_today(self.heute)
        assert self.heute == today()
        if not self.bis_datum:
            self.bis_datum = self.heute
        if not self.von_datum:
            self.von_datum = self.bis_datum.add_month(-24)
        self.td = test_daten.TestDaten(
                            instance_name=self.config.INSTANCE_NAME,
                            seed=self.seed, 
                            #seed=102, 
                            von_jahr=self.von_datum,
                            bis_jahr=self.bis_datum,
                            heute=self.heute,
                            log=self.log,
                            open_database=self.open_database,
                            )
        self.interact = test_daten.InteractEbupd(verbose=self.verbose)
    def create(self):
        user = None # fr demo_daten nur ebupd-API verwenden, login nicht ntig
        # Stellen, Einrichtungsnummern, Kreis, Land, Mitarbeiter, Fachstatistikkonfiguration 
        self.einrichten()
        # Akten anlegen fr alle Stellen und Mitarbeiter
        n_akte = n_abschluss = n_waufn = n_fall = 0
        def eintraege_und_evt_abschluss(akte_id, fall_id, beginn):
            """Es werden Eintrge in die Akte gemachte (Beratungskontakte, Leistungen, 
            Fachstatistik, Bundesstatistik), falls beginn mehr als 2 Monate vor self.bis_datum liegt.
            Mit einer gewissen Wahrscheinlichkeit wird ein Abschluss des Falles vorgenommen (zdA).
            """
            # Durch drei Ziehungen wird erreicht, dass insgesamt mehr 
            # Flle abgeschlossen werden (ca. 75%)
            # Abschluss eines Falles um so wahrscheinlicher je lter er ist.
            ende1 = self.td.choose_date(self.von_datum, self.bis_datum)
            ende2 = self.td.choose_date(self.von_datum, self.bis_datum)
            ende3 = self.td.choose_date(self.von_datum, self.bis_datum)
            for ende in (ende1, ende2, ende3, None):
                if ende > beginn.add_month(2):
                    break
            if ende:
                # Eintrge zusammen mit dem Abschluss
                self.eintraege(akte_id, fall_id, beginn, ende)
                self.abschluss(akte_id, fall_id, beginn, ende)
            else:
                # Nur Eintrge, falls beginn frh genug
                beginn_plus_2 = beginn.add_month(2)
                if beginn_plus_2 < self.bis_datum:
                    eintrags_datum = self.td.choose_date(beginn_plus_2, self.bis_datum)
                    self.eintraege(akte_id, fall_id, beginn, eintrags_datum)
            return ende
        for i in range(self.n_akten):
            if i % 10 == 0:
                print i,
                sys.stdout.flush()
            beginn = self.td.choose_date(self.von_datum, self.bis_datum)
            akte_id, fall_id = self.neuaufnahme(beginn)
            n_akte += 1
            n_fall += 1
            ende = eintraege_und_evt_abschluss(akte_id, fall_id, beginn)
            if ende:
                n_abschluss += 1
                # self.waufnahme(akte_id, datum)
                # 75% aller Flle werden wieder aufgenommen, wenn die fruehest mgliche
                # Wiederaufnahme innerhalb der simulierten Periode liegt.
                fruehestens = ende.add_month(self.td.config.WIEDERAUFNAHMEFRIST)
                if fruehestens < self.bis_datum and random() < .75:
                    wauf_beginn = self.td.choose_date(fruehestens, self.bis_datum)
                    # print 'ENDE:', ende
                    # print 'WIEDERAUFNAHMEFRIST', self.td.config.WIEDERAUFNAHMEFRIST
                    # print 'WAUF_BEGINN', wauf_beginn
                    wauf_fall_id = self.waufnahme(akte_id, wauf_beginn)
                    assert wauf_fall_id != fall_id
                    n_waufn += 1
                    n_fall += 1
                    # Abschluss von wiederaufgenommenen Fllen nach dem selben Schema
                    wauf_ende = eintraege_und_evt_abschluss(akte_id, wauf_fall_id, wauf_beginn)
                    if wauf_ende:
                        n_abschluss += 1
        if i >= 10:
            print
        # print
        # print '        Akten: ', n_akte, ' Abschluesse: ', n_abschluss, ' Wiederaufnahmen: ', n_waufn
        # Altdaten erzeugen, fallen aus dem Schema, werden hier direkt eingefgt.
        self.td.altdaten_einfuegen_und_beispiel_datei_erzeugen(self.n_altdaten)
        self.log('Erzeugt:  Akten: %s  Abschluesse: %s  Wiederaufnahmen: %s  Altdaten: %s' % (n_akte, n_abschluss, n_waufn, self.n_altdaten))

