# coding: latin-1

"""Module fr Akte und Fall."""

import string,time,sys
import logging

from ebkus.db.sql import SQL
from ebkus.db.dbapp import DBAppError
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, Gruppe, FallList, GruppeList, Altdaten, Anmeldung, \
     Zustaendigkeit, today, cc, check_list, check_code, check_int_not_empty, \
     check_str_not_empty, check_date, EE
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.akte_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

class _akte(Request.Request, akte_share):
    """Gemeinsame Methode zur Anzeige der Aktenformulare.
    """
    def _process(self,
                 title,
                 file,
                 akte,
                 anmeldung,
                 formname,
                 hidden,
                 force_strkat=None,
                 ):
        notiz = h.FieldsetInputTable(
            legend='Notiz',
            daten=[[h.TextareaItem(label='Notiz',
                                   label_width='14%',
                                   name='no',
                                   value=akte['no'],
                                   class_='textareanormal100prz',
                                   tip="Freies Feld fr Notizen",
                                   )],
                   ]
            )
        klientendaten = self.get_klientendaten(akte)
        anschrift = self.get_anschrift(akte, force_strkat)
        falldaten = leistung = altdaten = None
        if file in ('akteeinf', 'waufneinf'):
            falldaten = h.FieldsetInputTable(
                legend='Falldaten',
                daten=[[h.SelectItem(label='Bearbeiter',
                                     name='zumitid',
                                     tip='Fallbearbeiter auswhlen',
                                     options=self.for_mitarbeiter(sel=self.mitarbeiter['id'])),
                        h.DatumItem(label='Anmeldedatum',
                                    name='zubg',
                                    tip='Anmeldedatum',
                                    year=today().year,
                                    month=today().month),
                        ]],
                )
            leistung = h.FieldsetInputTable(
                legend='Leistung',
                daten=[[h.SelectItem(label='Mitarbeiter',
                                     name='lemitid',
                                     tip='Mitarbeiter auswhlen, der die Leistung erbracht hat',
                                     options=self.for_mitarbeiter(sel=self.mitarbeiter['id'])),
                        h.SelectItem(label='Leistung',
                                     name='le',
                                     tip='Art der erbrachten Leistung',
                                     class_='listbox250',
                                     options=self.for_kat('fsle', akte['fs'])),
                        ],[h.DummyItem(),
                           h.DatumItem(label='Am',
                                       name='lebg',
                                       tip='Datum der Leistung oder des Beginns der Leistung',
                                       date=today()),
                           ]],
                )

        if file=='akteeinf' and \
               SQL("select count(*) from altdaten").execute()[0][0] > 0:
            # Altdaten vorhanden
            altdaten = h.FieldsetInputTable(
                legend='Nach Altdaten suchen',
                tip="Bitte zuerst nach Altdaten suchen, dann erst weitere Daten eingeben",
                button=h.Button(value='Altdaten',
                                name='xxx',
                                onClick="go_to_url('altlist')",
                                tip="Suche in den Altdaten aus frheren EDV-Systemen",
                                ),
                )
        if config.ANMELDUNGSDATEN_OBLIGATORISCH and file in ('akteeinf', 'waufneinf'):
            anmeldungf = self.get_anmeldekontakt(anmeldung)
        else:
            anmeldungf = None
        res = h.FormPage(
            title=title,
            name=formname,action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                          (not file=='akteeinf') and
                           ('Klientenkarte', 'klkarte?akid=%s' % akte['id'])
                           or None,
                           ),
            hidden=(("akid", akte['id']),
                    ("file", file),
                    ("strid", ""), # wird nur von strkat mit javascript gesetzt
                    ) + hidden,
            rows=(h.Pair(left=klientendaten,
                         right=(altdaten, anschrift),
                         ),
                  notiz,
                  anmeldungf,
                  falldaten,
                  leistung,
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class akteneu(_akte):
    """Neue Fallakte anlegen (Tabellen: Akte, Fall, Zustndigkeit, Leistung)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        alt_ids = check_list(self.form, 'uebern', 'Fehler in Altdaten', [])
        if len(alt_ids) > 1:
            raise EE("Bitte nur einen Eintrag zur bernahme markieren!")
        if alt_ids:
            alt = Altdaten(alt_ids[0])
        else:
            alt = {}
        gs_alt = alt.get('geschlecht')
        if gs_alt:
            gs = (gs_alt=='w' and cc('gs', '2')) or (gs_alt=='m' and cc('gs', '1'))
        else:
            gs = ' '
        if alt:
            no = "Alte Fallnummer: %(fallnummer)s, Jahr: %(jahr)s, Frherer Mitarbeiter: %(mitarbeiter)s" % alt
        else:
            no = ''
        str = alt.get('strasse', '')
        if str:
            for end in ('trasse', 'trae'):
                if str.endswith(end):
                    i = str.index(end)
                    str = str[:i] + 'tr.'
        akte = Akte()
        akte.init(
            id=Akte().getNewId(),
            fs=cc('fsfs', '999'),
            vn=alt.get('vorname', ''),
            na=alt.get('name', ''),
            gb=alt.get('geburtsdatum', ''),
            ort=alt.get('ort', ''),
            plz=alt.get('plz', ''),
            str=str,
            hsnr=alt.get('hausnummer', '').upper(),
            gs=gs,
            tl1=alt.get('telefon1', ''),
            tl2=alt.get('telefon2', ''),
            no=no,
            aufbew=cc('aufbew', '1'),
            stzbg=self.stelle['id'],
            lage=(config.STRASSENKATALOG and  cc('lage', '0') or
                  cc('lage', '1')),
            )
        hidden = (('stzbg', akte['stzbg']),
                  )
        anm = None
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm = Anmeldung()
            anm.init(
                id=Anmeldung().getNewId(),
                zm=cc('fszm', '999'),
                )
            # Nachname und Telefon des Klienten
            # im Formular anbieten.
            anm['von'] = akte['na']
            anm['mtl'] = akte['tl1']
            if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
                anm['anmprobleme'] = None
                anm['qualij'] = ' '
                anm['bkm'] = ' '
                anm['bkv'] = ' '
            hidden += (('anmid', anm['id']),)
        
        return self._process(
            title='Neue Akte anlegen',
            file='akteeinf',
            akte=akte,
            anmeldung=anm,
            formname='akteform',
            hidden=hidden,
            )

class waufnneu(_akte):
    """Wiederaufnahme einer vorhandener  Fallakte
    (Tabellen: Akte, Fall, Zustndigkeit, Leistung)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        akid = self.form.get('akid')
        if not akid:
            raise EE("Keine ID fuer die Akte erhalten")
        akte = Akte(akid)
        hidden=(('fallid', Fall().getNewId()),
                ('status', cc('stand', 'l')),
                )
        anm = None
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm = Anmeldung()
            anm.init(
                id=Anmeldung().getNewId(),
                zm=cc('fszm', '999'),
                )
            # Nachname und Telefon des Klienten
            # im Formular anbieten.
            anm['von'] = akte['na']
            anm['mtl'] = akte['tl1']
            hidden += (('anmid', anm['id']),)
        return self._process(
                 title="Wiederaufnahme des Klienten",
                 file='waufneinf',
                 akte=akte,
                 anmeldung=anm,
                 formname='akteform',
                 hidden=hidden,
                 )
        
class updakte(_akte):
    """Akte ndern (Tabelle Akte)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        strkat = self.form.get('strkat')
        if strkat == '0':
            force_strkat = False
        elif strkat == '1':
            force_strkat = True
        else:
            force_strkat = None
        if self.form.has_key('akid'):
            akid = self.form.get('akid')
            akte = Akte(akid)
        else:
            self.last_error_message = "Keine ID fuer die Akte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        return self._process(
            title='Akte aktualisieren',
            file='updakte',
            akte=akte,
            anmeldung=None,
            formname='akteform',
            hidden=(),
            force_strkat=force_strkat,
            )

class updfall(_akte):
    """Fall updaten (Tabelle: Fall)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fuer den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        zustaendigkeiten_list = fall['zustaendigkeiten']
        beginndatum = h.FieldsetInputTable(
            legend='Beginndatum',
            daten=[[h.DatumItem(label='Fallbeginn',
                                name='bg',
                                tip='Fallbeginn',
                                date=fall.getDate('bg')),
            ]],
            )
        res = h.FormPage(
            title='Beginndatum ndern',
            name='akteform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'updfall'),
                    ),
            rows=(beginndatum,
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()
        
class zda(_akte):
    """Fallakte abschliessen (Tabellen: Fall und Zustndigkeit)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fuer den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        zustaendigkeiten_list = fall['zustaendigkeiten']
        zustaendigkeiten_list.sort('bgd', 'bgm', 'bgy')
        aktuell_zustaendig = fall['zustaendig']
        falldaten = h.FieldsetInputTable(
            legend='Falldaten',
            daten=[[h.DatumItem(label='Beginndatum',
                                name='bg',
                                tip='Fallbeginn',
                                date=fall.getDate('bg')),
                    h.DatumItem(label='Abschlussdatum',
                                name='zda',
                                tip='Fallabschlussdatum',
                                date=today()),
            ]],
            )
        res = h.FormPage(
            title='Abschlussdatum eintragen',
            name='akteform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'zdaeinf'),
                    ("aktuellzustid", aktuell_zustaendig['id']),
                    ("aktuellmitid", aktuell_zustaendig['mit_id__id']),
                    ),
            rows=(falldaten,
                  self.get_bisherige_zustaendigkeit(
            aktuell_zustaendig,
            legend='Bisherige Zustndigkeit wird ausgetragen'),
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()
        
class zdar(_akte):
    """Fallabschluss rckgngig machen und letzte Zustaendigkeit wiederherstellen
    (Tabellen: Fall und Zustndigkeit)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        fallid = self.form.get('fallid')
        if not fallid:
            raise EE("Keine ID fuer den Fall erhalten")
        fall = Fall(fallid)
        zuletzt_zustaendig = fall['zuletzt_zustaendig']
        res = h.FormPage(
            title='Abschlussdatum rckgngig machen',
            name='zdarform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'zdareinf'),
                    ),
            rows=(self.get_bisherige_zustaendigkeit(
            zuletzt_zustaendig,
            legend='Letzte Zustndigkeit wird wiederhergestellt',
            ),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class _zust(_akte):
    def _process(self,
                 title,
                 file,
                 fall,
                 zustaendigkeit, # Form schon fertig, da unterschiedlich upd/neu
                 hidden,
                 ):
        aktuell_zustaendig = fall['zustaendig']
        zustaendigkeiten_list = fall['zustaendigkeiten']
        zustaendigkeiten_list.sort('bgy', 'bgm', 'bgd')
        if file == 'zusteinf':
            bisherige_zustaendigkeit = self.get_bisherige_zustaendigkeit(
                aktuell_zustaendig,
                legend='Bisherige Zustndigkeit wird ausgetragen',
                )
        elif file == 'updzust':
            bisherige_zustaendigkeit = None
        res = h.FormPage(
            title=title,
            name='zustform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=hidden,
            rows=(self.get_klientendaten_kurz(fall),
                  zustaendigkeit,
                  bisherige_zustaendigkeit,
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class zustneu(_zust):
    """Neue Zustndigkeit eintragen. (Tabelle: Zustndigkeit.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fr den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        aktuell_zustaendig = fall['zustaendig']
        neue_zust = Zustaendigkeit()
        neue_zust.init(
            id=Zustaendigkeit().getNewId(),
            mit_id=self.mitarbeiter['id'],
            fall_id=fall['id'],
            )
        neue_zust.setDate('bg', today())
        zustaendigkeit = h.FieldsetInputTable(
            legend="Neue Zustndigkeit eintragen",
            daten=[[h.SelectItem(label='Bearbeiter',
                                 name='mitid',
                                 tip='Fallbearbeiter auswhlen',
                                 options=self.for_mitarbeiter(neue_zust['mit_id'])),
                    h.DatumItem(label='Beginn',
                                name='bg',
                                tip='Datum des Zustndigkeitbeginns',
                                date=neue_zust.getDate('bg')),
                    # keine Endedataum
                    ]],
            )
        return self._process(
            title="Neue Zust&auml;ndigkeit eintragen",
            file='zusteinf',
            fall=fall,
            zustaendigkeit=zustaendigkeit,
            hidden=(('zustid', neue_zust['id']),
                    ("aktuellzustid", aktuell_zustaendig['id']),
                    ('file', 'zusteinf'),
                    ('fallid', fall['id']),
                    ),
            )
        
class updzust(_zust):
    """Neue Zustndigkeit eintragen. (Tabelle: Zustndigkeit.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('zustid'):
            id  = self.form.get('zustid')
        else:
            self.last_error_message = "Keine ID fr die Zustndigkeit erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        zust = Zustaendigkeit(id)
        fall = Fall(zust['fall_id'])
        zustaendigkeit = h.FieldsetInputTable(
            legend="Zustndigkeit bearbeiten",
            daten=[[h.SelectItem(label='Bearbeiter',
                                 name='mitid',
                                 tip='Fallbearbeiter auswhlen',
                                 options=self.for_mitarbeiter(zust['mit_id'])),
                    h.DatumItem(label='Beginn',
                                name='bg',
                                tip='Datum des Zustndigkeitbeginns',
                                date=zust.getDate('bg')),
                    h.DatumItem(label='Ende',
                                name='e',
                                tip='Datum des Zustndigkeitendes',
                                date=zust.getDate('e')),
                    ]],
            )

        return self._process(
            title="Zust&auml;ndigkeit bearbeiten",
            file='updzust',
            fall=fall,
            zustaendigkeit=zustaendigkeit,
            hidden=(('zustid', zust['id']),
                    ('file', 'updzust'),
                    ),
            )

class rmaktenf(Request.Request, akte_share):
    """Einzelnen fehlerhaften Fall oder fehlerhafte Akte lschen.
    Ein Fall wird durch seine Fallnummer und das Anmeldedatum identifiziert.
    Dadurch wird ein irrtmliches Lschen weitgehend verhindert. Es soll nicht so
    leicht sein.

    Wenn eine ganze fehlerhafte Akte gelscht werden soll, so wird diese ebenfalls 
    ber die Fallnummer und das Anmeldedatum eines Falles in der Akte identifiziert.
    Die anderen Flle werden vor der Lschung angezeigt, falls vorhanden.

    Wenn nur ein Fall vorhanden ist, wird in jedem Fall die ganze Akte inklusive 
    Stammdaten gelscht.

    Statistik wird immer mitgelscht.
    Es knnen offene Flle bzw. Akten mit einem offenen Fall gelscht werden.
    Die Lschfrist spielt keine Rolle.
    """
    permissions = Request.ADMIN_PERM

    def get(self, REQUEST, RESPONSE):
        fn = check_str_not_empty(self.form, 'fn', 'Keine Fallnummer', '')
        anmeldedatum = check_date(self.form, 'bg', "Fehler im Anmeldedatum", (0,0,0))
        ganze_akte = self.form.get('ganze_akte')
        auswahl_akte = h.FieldsetFormInputTable(
            name="rmfallanzeigen",
            action="rmaktenf",
            method="post",
            legend='Fallnummer und Anmeldedatum des zu lschenden Falls bzw. eines Falles '
            'in der zu lschenden Akte',
            daten=[[h.TextItem(label='Fallnummer',
                               name='fn',
                               value=fn,
                               tip='Fallnummer des zu lschenden Falls bzw. eines Falles '
                               'in der zu lschenden Akte'
                               ),
                    h.DatumItem(label='Anmeldedatum',
                                date=anmeldedatum,
                                name='bg',
                                ),
                    h.CheckItem(label="Ganze Akte lschen",
                                name="ganze_akte",
                                value="1",
                                checked=ganze_akte,
                                tip="Ankreuzen, um die ganze Akte zu lschen, "
                                "zu der dieser Fall gehrt."),
                    ],
                   [h.Dummy(n_col=6)],
                   ],
            button=h.Button(value="Lschen",
                            name='op',
                            tip="Fall mit der angegebenen Nummer endgltig lschen",
                            type='submit',
                            n_col=4,
                            )
            )
        res = h.Page(
            title= 'Einzelnen Fall bzw. einzelne Akte lschen',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  auswahl_akte,
                  ),
            )
        return res.display()

    def post(self, REQUEST, RESPONSE):
        """
        2 Flle
        1. Fallauswahl mit Fallnummer und Anmeldedatum
        --> zurck Angebot mit Lschen der Akte oder des Falles
        2. fall_id und ok: Die Lschung des Falls durchfhren.
           akte_id und ok: Die Lschung der Akte durchfhren.
        --> Besttigung der Lschung mit Fallnummer bzw. ganze Akte mit Fallnummern. 
        """
        ok = self.form.get('ok')                    # ok zum endgltigen Lschen
        if ok:
            return self._einzelfall_oder_akte_loeschen()
        else:
            return self._einzelfall_oder_akte_loeschangebot()
    def _einzelfall_oder_akte_loeschen(self):
        from ebkus.app.ebupd import remove_faelle, remove_akten
        fall_id = self.form.get('fallid')           # Diesen Fall endgltig lschen
        akte_id = self.form.get('akid')             # Diese Akte endgltig lschen
        msg = self.form.get('msg')
        if not (fall_id or akte_id):
            raise EE('Nichts zu lschen')
        if fall_id:
            remove_faelle([int(fall_id)], statistik_auch=True, aktuell_auch=True)
        elif akte_id:
            remove_akten([int(akte_id)], statistik_auch=True, aktuell_auch=True)
        return h.Meldung(legend=msg,
                         zeilen=[msg + " gelscht"],
                         weiter='rmaktenf').display()
    def _einzelfall_oder_akte_loeschangebot(self):
        fn = check_str_not_empty(self.form, 'fn', 'Keine Fallnummer')
        anmeldedatum = check_date(self.form, 'bg', "Kein Anmeldedatum")
        if anmeldedatum.year == 0:
            EE('Kein Anmeldedatum')
        try:
            fall = Fall(fn=fn, bgy=anmeldedatum.year, 
                        bgm=anmeldedatum.month, bgd=anmeldedatum.day)
        except DBAppError:
            raise EE("Kein Fall gefunden mit Fallnumer {fn} "
                     "und Anmeldedatum {anmeldedatum}".format(**locals()))
        ganze_akte = self.form.get('ganze_akte')
        akte = fall['akte']
        faelle = akte['faelle']
        einzelfall = len(faelle) == 1
        faelle.sort('bgy', 'bgm', 'bgd')
        fn_str = ', '.join(f['fn'] for f in faelle)
        assert fall in faelle, "Fall nicht in Fllen der Akte :-("
        if not ganze_akte and len(faelle) > 1 and fall != akte['letzter_fall']:
            raise EE(
                "Der zu lschende Fall muss der neueste Fall aus der Akte sein. "
                "Die Akte besteht aus den Fllen {}. ".format(fn_str))
        msg_statistik = "Evt. vorhandene Fach- bzw. Bundesjugendhilfestatistik wird ebenfalls gelscht."
        if ganze_akte or einzelfall:
            zeilen = []
            legend = 'Ganze Akte lschen'
            if einzelfall and not ganze_akte:
                zeilen.append("Da die Akte nur den zu lschenden Fall enthlt, "
                "wird die gesamte Akte gelscht. ")
            zeilen.append(msg_statistik)
            if einzelfall:
                msg = "Ganze Akte bestehend aus dem Fall {}".format(fn_str)
            else:
                msg = "Ganze Akte bestehend aus den Fllen {}".format(fn_str)
            zeilen.append(msg + " endgltig lschen?")
            hidden = (('akid', akte['id']),
                      ('ok', '1'),
                      ('msg', msg),
                      )
        else:
            # Einen Fall lschen
            msg = 'Fall {}'.format(fn)
            legend = msg + ' lschen'
            zeilen = [msg_statistik, msg + " endgltig lschen?"]
            hidden = (('fallid', fall['id']),
                      ('ok', '1'),
                      ('msg', msg),
                      )
        return h.SubmitOrBack(legend=legend,
                              action='rmaktenf',
                              method='post',
                              hidden=hidden,
                              zeilen=zeilen,
                              ).display()
        
        



    def processForm(self, REQUEST, RESPONSE):
        gruppe = self.form.get('gruppe')            # Gruppe zum lschen anbieten, sonst Akte
        fn = self.form.get('fn')                    # Diesen Fall zum Lschen prfen
        gn = self.form.get('gn')                    # Diese Gruppe zum Lschen prfen
        if fn and gn:
            raise EE("Es kann entweder ein Fall oder eine Gruppe gelscht werden, nicht beides zugleich")
        statistik_auch = (self.form.get('statistik_auch') == '1')
        ok = self.form.get('ok')                    # ok zum endgltigen Lschen
        fall_id = self.form.get('fallid')           # Diesen Fall endgltig lschen
        gruppe_id = self.form.get('gruppeid')       # Diese Gruppe endgltig lschen
        if fn and not ok and not fall_id:
            faelle = FallList(where="fn = '%s'" % fn)
            if len(faelle) < 1:
                raise EE("Kein Fall mit dieser Fallnummer gefunden.")
            elif len(faelle) > 1:
                # darf nicht passieren
                raise EE("Mehrere passende Flle gefunden.")
            else:
                fall = faelle[0]
            if not fall['aktuell']:
                raise EE("Fall <em>%s</em> ist abgeschlossen. Es knnen nur aktuelle Flle gelscht werden."
                         % fn)
            akte = fall['akte']
            statistik_vorhanden = bool(fall['fachstatistiken'] or
                                       fall['jgh_statistiken'] or
                                       fall['jgh07_statistiken'])
            stattext = ''
            if statistik_vorhanden:
                if statistik_auch:
                    stattext = "(Die zugehrige Bundes- und Fachstatistik wird ebenfalls gelscht.)"
                else:
                    stattext = "(Die zugehrige Bundes- und Fachstatistik bleibt erhalten.)"
            if len(akte['faelle']) == 1:
                letzter_fall = True
                legend = 'Akte endgltig lschen'
                zeilen=("Der Fall <em>%(fn)s</em> ist der einzige in der Akte <em>%(name)s</em>." % fall,
                        "Da eine Akte immer mindestens einen Fall haben muss, wird die "
                        "Akte inklusive aller Stammdaten ebenfalls gelscht.",
                        stattext,
                        "Soll der Fall <em>%(fn)s</em> mitsamt der gesamten Akte <em>%(name)s</em> "
                        "endgltig gelscht werden?" % fall,
                        )
            else:
                letzter_fall = False
                legend = 'Fall endgltig lschen'
                zeilen=("Soll der Fall <em>%(fn)s (%(name)s)</em> endgltig gelscht werden?" % fall,
                        stattext,
                        )
            hidden = (('fallid', fall['id']),
                      ('statistik_auch', statistik_auch and '1' or ''),
                      ('ok', '1'),
                )
            return h.SubmitOrBack(
                legend=legend,
                action='rmaktenf',
                method='post',
                hidden=hidden,
                zeilen=zeilen,
                ).display()
        elif gn and not ok and not gruppe_id:
            gruppen = GruppeList(where="gn = '%s'" % gn)
            if len(gruppen) < 1:
                raise EE("Kein Gruppe mit dieser Gruppennummer gefunden.")
            elif len(gruppen) > 1:
                # darf nicht passieren
                raise EE("Mehrere passende Gruppen gefunden.")
            else:
                gruppe = gruppen[0]
            if gruppe['ey']:
                raise EE(("Gruppe <em>%s</em> hat eine Endedatum, ist also abgeschlossen. " +
                         "Es knnen nur laufende Gruppen gelscht werden.")
                         % gn)
            if gruppe['faelle'] or gruppe['bezugspersonen']:
                raise EE(("Die Gruppe <em>%s</em> hat Teilnehmer. " +
                         "Es knnen nur Gruppen ohne Teilnehmer gelscht werden.")
                         % gn)
            legend = 'Gruppe endgltig lschen'
            zeilen=("Soll die Gruppe <em>%(gn)s (%(name)s)</em> endgltig gelscht werden?" % gruppe,
                        )
            hidden = (('gruppeid', gruppe['id']),
                      ('ok', '1'),
                )
            return h.SubmitOrBack(
                legend=legend,
                action='rmaktenf',
                method='post',
                hidden=hidden,
                zeilen=zeilen,
                ).display()
        elif fall_id and ok == '1':
            fall = Fall(fall_id)
            akte = fall['akte']
            akten = [akte]
            name = akte['name']
            fn = fall['fn']
            from ebkus.app.ebupd import remove_fall, remove_akte
            if len(akte['faelle']) == 1:
                ganze_akte_loeschen = True
                remove_akte(akte,
                            statistik_auch=statistik_auch,
                            aktuell_auch=True)
                legend = "Ganze Akte gelscht"
            else:
                ganze_akte_loeschen = False
                remove_fall(fall,
                            statistik_auch=statistik_auch,
                            aktuell_auch=True)
                legend = "Fall gelscht"
            geloeschter_fall = h.FieldsetDataTable(
                legend=legend,
                headers=('Fallnummer', 'Name'),
                daten=[[h.String(string=fn),
                        h.String(string=name),
                        ]],
                )
            res = h.Page(
                title=legend,
                breadcrumbs = (('Aministratorhauptmen', 'menu'),
                               ('Einzelnen Fall lschen', None),
                               ),
                rows=(self.get_hauptmenu(),
                      geloeschter_fall,
                      ),
                )
            return res.display()
        elif gruppe_id and ok == '1':
            gruppe = Gruppe(gruppe_id)
            gn = gruppe['gn']
            name = gruppe['name']
            from ebkus.app.ebupd import remove_fall, remove_akte, remove_gruppe
            remove_gruppe(gruppe)
            legend = "Gruppe gelscht"
            geloeschte_gruppe = h.FieldsetDataTable(
                legend=legend,
                headers=('Gruppennummer', 'Name'),
                daten=[[h.String(string=gn),
                        h.String(string=name),
                        ]],
                )
            res = h.Page(
                title=legend,
                breadcrumbs = (('Aministratorhauptmen', 'menu'),
                               ('Einzelne Gruppe lschen', None),
                               ),
                rows=(self.get_hauptmenu(),
                      geloeschte_gruppe,
                      ),
                )
            return res.display()
            
        auswahl_akte = h.FieldsetFormInputTable(
            name="rmfallanzeigen",
            action="rmaktenf",
            method="post",
            legend='Fallnummer des zu lschenden Falls',
            daten=[[h.TextItem(label='Fallnummer',
                               name='fn',
                               value='',
                               tip='Fallnummer des zu lschenden Falls bzw. eines Falles '
                               'in der zu lschenden Akte'
                               ),
                    h.CheckItem(label="Statistik auch lschen",
                                name="statistik_auch",
                                value="1",
                                checked=True,
                                tip="Ankreuzen, um auch die zugehrige Bundes- und Fachstatistik zu lschen",
                                ),
                    h.CheckItem(label="Ganze Akte lschen",
                                name="ganze_akte",
                                value="1",
                                checked=False,
                                tip="Ankreuzen, um die ganze Akte zu lschen, "
                                "zu der dieser Fall gehrt."),
                    ],
                   [h.Dummy(n_col=4)],
                   ],
            button=h.Button(value="Lschen",
                            name='op',
                            tip="Fall mit der angegebenen Nummer endgltig lschen",
                            type='submit',
                            n_col=4,
                            ),
            )
        auswahl_gruppe = h.FieldsetFormInputTable(
            name="rmgruppeanzeigen",
            action="rmaktenf",
            method="post",
            legend='Gruppennummer der zu lschenden Gruppe',
            daten=[[h.TextItem(label='Gruppennummer',
                               name='gn',
                               value='',
                               tip='Gruppennummer der zu lschenden Gruppe'
                               ),
                    h.DummyItem(label_width="50%", class_="textbox280"), # breites Dummy aus optischen Grnden
                    ],
                   [h.Dummy(n_col=4)],
                   ],
            button=h.Button(value="Lschen",
                            name='op',
                            tip="Gruppe mit der angegebenen Nummer endgltig lschen",
                            type='submit',
                            n_col=4,
                            ),
            )
        res = h.Page(
            title= 'Einzelne Gruppe lschen' if gruppe else 'Einzelnen Fall lschen',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  auswahl_gruppe if gruppe else auswahl_akte,
                  ),
            )
        return res.display()


class rmakten(Request.Request, akte_share):
    """Abfrageformular zum Lschen von Akten."""
    permissions = Request.ADMIN_PERM
    def get(self, REQUEST, RESPONSE):
        """
        Formular fr die Auswahl der Lschkandidaten.
        Individuelle Auswahl ist nicht ntig, notfalls kann
        man dieselbe Fallnummer von bis angeben, um die Auswahl
        auf einen einzelnen Fall zu reduzieren.
        Fallabschlussjahr: von



        Es gibt wohl unterschiedliche Anforderungen. Wie kann man das
        flexibel halten?


        Lschfristen in der Konfiguration getrennt fr Beratung, Therapie und Missbrauch.
        

        Einzelfall lschen. Fallnummer, Anmeldedatum, Hkchen Lschfrist ignorieren.

        Ansonsten: nur Jahr des Fallabschlusses und Aufbewahrungskategorie.
        Dann kann man den kompletten Jahrgang lschen.

        Einzelfall lschbar unter Umgehung der Lschfrist, wie bei Lschen wg. Fehler.
        Ansonsten nur Flle einer bestimmte Kategorie fr einen bestimmten Jahrgang anbieten.
        

        Neumeldungen nur nach Bundesstatistik? Auch fr den Fallbeginn?
        """

    def processForm(self, REQUEST, RESPONSE):
        #print self.form
        alter = self.form.get('alter')
        if alter == None:
            alter = config.LOESCHFRIST
        else:
            alter = check_int_not_empty(
            self.form, 'alter', 'Fehler im Alter in Monaten')
            if alter < config.LOESCHFRIST:
                raise EE('Lschfrist betrgt mindestens %s Monate' % config.LOESCHFRIST)
        aufbew = self.form.get('aufbew')
        if aufbew == None:
            aufbew = cc('aufbew', '1')
        else:
            aufbew = check_code(self.form, 'aufbew', 'aufbew',
                                'Fehler in Aufbewahrungskategorie')
        op = self.form.get('op')
        rm = check_list(self.form, 'rmak', 'Fehler in markierten Akten', '')
        if op == 'Endgltig lschen' and rm:
            hidden = [('rmak', r) for r in rm] + [('op', 'loeschen_confirmed')]
            return h.SubmitOrBack(
                legend='Endgltig lschen',
                action='rmakten2',
                method='post',
                hidden=hidden,
                zeilen=("Sollen die markierten Akten endgltig gelscht werden?",
                        )
                ).display()
        elif op == 'Alle markieren':
            alle_markieren = True
        else:
            alle_markieren = False
        alter_und_kategorie = h.FieldsetFormInputTable(
            name="rmaktenanzeigen",
            action="rmakten",
            method="post",
            hidden = (),
            legend='Alter und Kategorie der zu lschenden Akten whlen',
            daten=[[h.SelectItem(label='Aufbewahrungskategorie',
                                 name='aufbew',
                                 options=self.for_kat('aufbew',
                                                      sel=aufbew or cc('aufbew', '1')),
                                 ),
                    h.TextItem(label='Monate seit Abschluss des letzten Falls',
                               name='alter',
                               value=alter or '',
                               class_='textboxsmall',
                               maxlength=3,
                               tip='Mindesanzahl der Monate seit Abschluss des letzten Falls'
                               ),
                    ],
                   [h.Dummy(n_col=4)],
                   [h.Button(value="Anzeigen",
                             name='op',
                             tip="Akten der gewhlten Kategorie und des gewhlten Alters anzeigen",
                             type='submit',
                             n_col=4,
                             ),
                    ],
                   ],
            )
        if alter and aufbew:
            rmdatum = today().add_month(-(alter+1)) # damit immer mindestens die Anzahl Monate
                                                    # dazwischen liegt
            #print 'LOESCHDATUM', rmdatum
            #alle_faelle = FallList(
            #    where = 'zday <= %(year)s and zdam <= %(month)s and zday > 0'
            #    % rmdatum)
            alle_faelle = FallList(
                # where = '(zday < %(year)s or (zday = %(year)s and zdam <= %(month)s)) and zday > 0'
                #    % rmdatum, 
                # order = 'zday, zdam, zdad',
                # cache_related=True)
                where = 'zday = 2010',
                order = 'zday, zdam, zdad',
                cache_related=True)
            # Hier gab es einen Fehler (in Reinickendorf):
            # AttributeError: Could not resolve field 'akte__letzter_fall' for instance of 'Fall'
            # Daher wurde nichts zum lschen angezeigt
            # faelle = [f for f in alle_faelle
            #           if f == f['akte__letzter_fall'] and aufbew == f['akte__aufbew']]
            faelle = []
            for f in alle_faelle:
                try:
                    if f == f['akte__letzter_fall'] and aufbew == f['akte__aufbew']:
                        faelle.append(f)
                except Exception, e:
                    t = sys.exc_info()[0]
                    logging.exception("Interner Fehler: %s: %s", t, e)
                    logging.exception("  Falle ohne Akte? fall_id=%s" % f['id'])

            daten= [[h.String(string=fall['fn']),
                    h.String(string=fall['akte__na']),
                    h.String(string=fall['akte__vn']),
                    h.String(string=fall['akte__gb']),
                    h.Datum(date=fall.getDate('zda')),
                    h.CheckItem(label='',
                                name='rmak',
                                value=fall['akte_id'],
                                checked=alle_markieren,
                                tip='Hier markieren, um die Akte endgltig zu lschen',
                                ),
                    ] for fall in faelle] + [[h.Dummy(n_col=6),
                     h.Button(value="Alle zurcksetzen" if alle_markieren else "Alle markieren",
                           name='op',
                           tip="Auswahl zurcksetzen" if alle_markieren else 'Alle Akten markieren',
                           type='submit',
                                     ) 
                     ]]
            akten = h.FieldsetFormDataTable(
                name="rmakten",
                #action="abfragedef",
                action="rmakten",
                method="post",
                hidden = (),
                legend='Akten zum lschen auswhlen ({}) [Statistiken bleiben erhalten]'.format(len(faelle)),
                headers=('Fallnummer', 'Name', 'Vorname', 'Geburtsdatum', 'Letzter Fallabschluss', '', 'Lschen'),
                daten=daten,
                no_button_if_empty=True,
                button=h.Button(value="Endgltig lschen",
                           name='op',
                           tip="Markierte endgltig Akten lschen",
                           type='submit',
                           class_='buttonbig',
                           n_col=6,
                     ),
                empty_msg="Keine Akten gefunden.",
                )
            
        else:
            akten = None
        res = h.Page(
            title='Akten lschen nach Ablauf der Lschfrist',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  alter_und_kategorie,
                  akten,
                  ),
            )
        return res.display()
        
class rmakten2(Request.Request, akte_share):
    """Lscht die Akten, deren ID im Parameter rmak steht.
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        from ebkus.app.ebapi import check_list
        #print self.form
        op = self.form.get('op')
        rm = check_list(self.form, 'rmak', 'Fehler in markierten Akten', '')
        if op == 'loeschen_confirmed' and rm:
            akte_ids = [int(x) for x in rm]
            if akte_ids:
                from ebkus.app.ebupd import remove_akten
                anzahl_akten_geloescht = remove_akten(akte_ids, statistik_auch=False, aktuell_auch=False)
            else:
                raise EE('Keine Akten zum lschen markiert')
            geloeschte_akten = h.FieldsetDataTable(
                legend='Endgltig gelschte Akten (Statistik bleibt erhalten)',
                headers=('Name', 'Vorname', 'Geburtsdatum', 'Letzter Fallabschluss'),
                daten=[[h.String(string=akte['na']),
                        h.String(string=akte['vn']),
                        h.String(string=akte['gb']),
                        h.Datum(date=akte['zda']),
                        ] for akte in akten],
                )
            
        else:
            geloeschte_akten = None
        res = h.Page(
            title='Akten gelscht',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ('Akten lschen', None),
                           ),
            rows=(self.get_hauptmenu(),
                  geloeschte_akten,
                  ),
            )
        return res.display()

class rmgruppen(Request.Request, akte_share):
    """Gruppen lschen wg. Datenschutz."""
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        #print self.form
        alter = self.form.get('alter')
        if alter == None:
            alter = config.LOESCHFRIST
        else:
            alter = check_int_not_empty(
            self.form, 'alter', 'Fehler im Alter in Monaten')
            if alter < config.LOESCHFRIST:
                raise EE('Lschfrist betrgt mindestens %s Monate' % config.LOESCHFRIST)
        op = self.form.get('op')
        rm = check_list(self.form, 'rmgr', 'Fehler in markierten Gruppen', '')
        if op == 'Endgltig lschen' and rm:
            hidden = [('rmgr', r) for r in rm] + [('op', 'loeschen_confirmed')]
            return h.SubmitOrBack(
                legend='Endgltig lschen',
                action='rmgruppen2',
                method='post',
                hidden=hidden,
                zeilen=("Sollen die markierten Gruppen endgltig gelscht werden?",
                        )
                ).display()
        alter_auswahl = h.FieldsetFormInputTable(
            name="rmgruppenanzeigen",
            action="rmgruppen",
            method="post",
            hidden = (),
            legend='Alter der zu lschenden Gruppen whlen',
            daten=[[h.TextItem(label='Monate seit Beendigung der Gruppe',
                               name='alter',
                               value=alter or '',
                               class_='textboxsmall',
                               maxlength=3,
                               tip='Mindesanzahl der Monate seit Beendigung der Gruppe'
                               ),
                    ],
                   [h.Dummy(n_col=2)],
                   [h.Button(value="Anzeigen",
                             name='op',
                             tip="Gruppendes gewhlten Alters anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    ],
                   ],
            )
        if alter:
            rmdatum = today().add_month(-(alter+1)) # damit immer mindestens die Anzahl Monate
                                                    # dazwischen liegt
            gruppe_list = GruppeList(
                where = '(ey < %(year)s or (ey = %(year)s and em <= %(month)s)) and ey > 0'
                   % rmdatum, 
                order = 'ey, em, ed')
            gruppen = h.FieldsetFormDataTable(
                name="rmgruppen",
                #action="abfragedef",
                action="rmgruppen",
                method="post",
                hidden = (),
                legend='Gruppen zum lschen auswhlen',
                headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                         'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende'),
                daten=[[h.String(string=gruppe['gn']),
                        h.String(string=gruppe['mitarbeiternamen']),
                        h.String(string=gruppe['name']),
                        h.String(string=gruppe['thema']),
                        h.String(string=gruppe['grtyp__name']),
                        h.String(string=gruppe['teiln__name']),
                        h.String(string=gruppe['tzahl']),
                        h.Datum(date=gruppe.getDate('bg')),
                        h.Datum(date=gruppe.getDate('e')),
                        h.CheckItem(label='',
                                    name='rmgr',
                                    value=gruppe['id'],
                                    checked=False,
                                    tip='Hier markieren, um die Gruppe endgltig zu lschen',
                                    ),
                        ] for gruppe in gruppe_list],
                no_button_if_empty=True,
                button=h.Button(value="Endgltig lschen",
                           name='op',
                           tip="Markierte Gruppen endgltig lschen",
                           type='submit',
                           class_='buttonbig',
                           n_col=10,
                     ),
                empty_msg="Keine Gruppen gefunden.",
                )
            
        else:
            gruppen = None
        res = h.Page(
            title='Gruppen lschen nach Ablauf der Lschfrist',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  alter_auswahl,
                  gruppen,
                  ),
            )
        return res.display()
        
class rmgruppen2(Request.Request, akte_share):
    """Lscht die Gruppen, deren ID im Parameter rmgr steht.
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        from ebkus.app.ebapi import check_list
        #print self.form
        op = self.form.get('op')
        rm = check_list(self.form, 'rmgr', 'Fehler in markierten Gruppen', '')
        if op == 'loeschen_confirmed' and rm:
            if rm:
                rmdatum = today().add_month(-(config.LOESCHFRIST+1)) # damit immer mindestens die Anzahl Monate
                                                                     # dazwischen liegt
                gruppe_list = [Gruppe(r) for r in rm]
                for a in gruppe_list:
                    # vor dem Lschen holen!
                    assert (a['ey'] > 0 and 
                            (a['ey'] < rmdatum.year or 
                             (a['ey'] == rmdatum.year) and a['em'] < rmdatum.month))
                from ebkus.app.ebupd import removeakten
                anzahl_akten_geloescht, anzahl_gruppen_geloescht = removeakten(self.form)
                assert anzahl_gruppen_geloescht == len(gruppe_list), 'Fehler beim Lschen von Gruppen'
            else:
                raise EE('Keine Gruppen zum lschen markiert')
            geloeschte_gruppen = h.FieldsetDataTable(
                legend='Endgltig gelschte Gruppen',
                headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                         'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende'),
                daten=[[h.String(string=gruppe['gn']),
                        h.String(string=gruppe['mitarbeiternamen']),
                        h.String(string=gruppe['name']),
                        h.String(string=gruppe['thema']),
                        h.String(string=gruppe['grtyp__name']),
                        h.String(string=gruppe['teiln__name']),
                        h.String(string=gruppe['tzahl']),
                        h.Datum(date=gruppe.getDate('bg')),
                        h.Datum(date=gruppe.getDate('e')),
                        ] for gruppe in gruppe_list],
                )
            
        else:
            geloeschte_gruppen = None
        res = h.Page(
            title='Akten gelscht',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ('Gruppen lschen', None),
                           ),
            rows=(self.get_hauptmenu(),
                  geloeschte_gruppen,
                  ),
            )
        return res.display()
