# coding: latin-1

"""Module fr Akte und Fall."""

import os
import csv
#import logging
from ebkus.db.sql import escape
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, FallList, \
     FeldList, AltdatenList, Altdaten, Zustaendigkeit, Date, today, cc, EE

from ebkus.html.strkat import split_hausnummer, fuellen
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.akte_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share



class altimport(Request.Request, akte_share):
    permissions = Request.ADMIN_PERM

    def get_altdaten_felder(self):
        return [f['feld'] for f in FeldList(
            where="tabelle.tabelle='altdaten'",
            join=[('tabelle', 'tabelle.id=feld.tab_id')])]
        
    def read_data(self, f):
        #print 'FILE: ', f, type(f)
        data = []
        feldnamen = self.get_altdaten_felder()[1:]
        size = len(feldnamen)
        reader = csv.reader(f.readlines(),
                            delimiter=';',
                            doublequote=True,
                            quotechar='"',
                            )
        try:
            erste_zeile = reader.next()
        except StopIteration:
            self.csv_lese_fehler("Keine Daten gefunden")
            
        #print 'ERSTE_ZEILE: ', erste_zeile
        if size != len(erste_zeile):
            self.csv_lese_fehler("Anzahl der Feldnamen in der ersten Zeile stimmt nicht", 1,
                                 erste_zeile)
        for ist, soll in zip(erste_zeile, feldnamen):
            if ist != soll:
                self.csv_lese_fehler("Erste Zeile mit den Feldnamen stimmt nicht "
                                     "mit Feldnamen berein", 1, erste_zeile)
        for i, row in enumerate(reader):
            #print 'ZEILE: ', row
            if size != len(row):
                self.csv_lese_fehler("Anzahl der Felder in Zeile %(znr)s stimmt nicht",
                                     i+2, row)
            dic = dict(zip(erste_zeile, row))
            dic = self.validate_normalize_altdaten(dic, i+2, row)
            dic['id'] = i+2
            altd = Altdaten()
            altd.init(**dic)
            data.append(altd)
        return data

    def validate_normalize_altdaten(self, dic, znr, daten):
        strasse = dic.get('strasse', '')
        if strasse:
            for end in ('trasse', 'trae'):
                if strasse.endswith(end):
                    i = strasse.index(end)
                    strasse = strasse[:i] + 'tr.'
            dic['strasse'] = strasse
        gb = dic.get('geburtsdatum')
        if gb:
            try:
                d,m,j = [int(x) for x in gb.split('.')]
                date = Date(j,m,d)
                if not date.check():
                    raise Exception()
                dic['geburtsdatum'] = str(date)
            except:
                self.csv_lese_fehler('Fehler im Geburtsdatum: %s' % gb, znr, daten)
        gs = dic.get('geschlecht')
        if gs:
            if gs not in ('m', 'w', 'M', 'W'):
                self.csv_lese_fehler('Fehler im Geschlecht: %s' % gs, znr, daten)
            dic['geschlecht'] = gs.lower()
        jahr = dic.get('jahr')
        if jahr:
            try:
                assert 1980 < int(jahr) < today().year
            except:
                self.csv_lese_fehler('Fehler im Jahr: %s' % jahr, znr, daten)
        plz = dic.get('plz')
        if plz:
            try:
                assert 10000 < int(plz) < 99999
            except:
                self.csv_lese_fehler('Fehler in der Postleitzahl: %s' % plz, znr, daten)
        hausnummer = dic.get('hausnummer')
        if hausnummer:
            try:
                nummer, buchstabe, gu = split_hausnummer(hausnummer)
                dic['hausnummer'] = "%s%s" % (nummer, buchstabe.upper())
            except:
                self.csv_lese_fehler('Fehler in Hausnummer: %s' % hausnummer, znr, daten)
        return dic
    
    def csv_lese_fehler(self, msg, znr='', zeile=''):
        werte = ''.join([("%s: %s<br />" % (fuellen(k, 13),v))
                  for k,v in zip(self.get_altdaten_felder()[1:],
                                  zeile)])
        #raise
        raise EE('Fehler beim Lesen der CSV-Datei: <br /><br />' + (msg%locals())
                 + '<br /><br />' +
                 'Zeilennummer: %(znr)s<br />Zeilendaten:<br />%(werte)s' % locals())

    def processForm(self, REQUEST, RESPONSE):
        example = self.form.get('example')
        datei = self.form.get('datei')
        if datei:
            data = self.read_data(datei)
            self.session.data['altdaten'] = data
            return  h.SubmitOrBack(
                legend='Altdaten bernehmen',
                action='altimport',
                method='post',
                hidden=(('altdaten_validiert', '1'),
                        ),
                zeilen=("Kein Fehler gefunden. Altdaten in die Datenbank bernehmen?",
                        )
                ).display()
        altdaten_validiert = self.form.get('altdaten_validiert')
        if altdaten_validiert:
            data = self.session.data['altdaten']
            AltdatenList(where='').deleteall()
            altd_list = AltdatenList(data)
            altd_list.insertall()
            res = h.Meldung(
                legend="Altdaten erfolgreich importiert",
                zeilen=("Altdaten erfolgreich importiert.",
                      "%s Datensaetze eingelesen" % len(altd_list),
                        'Weiter zum  Hauptmen&uuml ...',
                        ),
                onClick="go_to_url('menu')",
                )
            return res.display()
        if self.session.data.get('altdaten'):
            del self.session.data['altdaten']
        fname = 'beispiel_altdaten.csv'
        demo_altdaten = os.path.join(config.EBKUS_HOME, 'sql', fname)
        if example == '1':
            f = open(demo_altdaten)
            content = f.read()
            f.close()
            self.RESPONSE.setHeader('content-type', 'text/plain; charset=iso-8859-1')
            #self.RESPONSE.setHeader('content-disposition', 'attachment; filename=%s' % fname)
            self.RESPONSE.setBody(content)
            return
        elif example == '2':
            f = open(demo_altdaten)
            content = f.read()
            f.close()
            self.RESPONSE.setHeader('content-type', "text/csv; charset=iso-8859-1")
            self.RESPONSE.setHeader('content-disposition',
                                    'attachment; filename=%s' % fname)
            self.RESPONSE.setBody(content)
            return
        erste_zeile = ';'.join(['"%s"' % f for f in self.get_altdaten_felder()][1:])
        hinweise_csv = h.FieldsetDataTable(
            legend='Hinweise zur CSV-Datei',
            daten=[[h.String(string="Die CSV-Datei muss genauso aufgebaut sein wie die " +
                      '<a href="altimport?example=1">Beispieldatei</a>.<br /> ' +
                      "Dieses Format kann direkt mit Open Office oder MS Excel " +
                      "geffnet und geschrieben werden " +
                      '(<a href="altimport?example=2">Beispiel</a>).',
                      n_col=2,
                      )
                    ],
                   [h.String(string='Erste Zeile:'
                             ),
                    h.String(string=
                             "In der ersten Zeile mssen in jeder Spalte "
                             " die entsprechenden Feldnamen stehen: "
                             "%s<em>&lt;Umbruch wg. Lesbarkeit&gt;</em> %s" %
                             (erste_zeile[:60],
                             erste_zeile[60:])
                             ),
                    ],
                   [h.String(string='Feldtrennzeichen:'
                             ),
                    h.String(string=';'
                             ),
                    ],
                   [h.String(string='Texttrennzeichen:'
                             ),
                    h.String(string='" (Anfhrungszeichen). Kann bei einfachen Feldern entfallen, die selbst keine Anfhrungszeichen oder Zeilenumbrche enthalten.'
                             ),
                    ],
                   [h.String(string='Texttrennzeichen im Text:'
                             ),
                    h.String(string='"" (Verdoppelung)'
                             ),
                    ],
                   [h.String(string='Zeilenumbrche in einem Feld:'
                             ),
                    h.String(string='zulssig'
                             ),
                    ],
                   [h.String(string='Kodierung:'
                             ),
                    h.String(string='iso-8859-1, iso-8859-15, latin-1, WinLatin1 '
                             '(Das ist das, '
                             'was im groen und ganzen '
                             'standardmig von Open Office und MS Excel erzeugt wird, zumindest '
                             'was die Umlaute und das EssZett betrifft. Es ist nicht Unicode oder utf8) '
                             ),
                    ],
                   ],
            )
        hinweise_felder = h.FieldsetDataTable(
            legend='Hinweise zu den Feldern',
            daten=[[h.String(string="Felder knnen leer sein, die entsprechenden "
                      'Daten stehen dann nicht fr eine bernahme zur Verfgung.',
                      n_col=2,
                      )
                    ],
                   [h.String(string="Folgende Felder werden in die entsprechenden Felder der neuen "
                             "Akte bernommen, falls sie Werte enthalten: "
                             "vorname, name, geburtsdatum, geschlecht, ort, plz, strasse, hausnummer, "
                             "telefon1, telefon2. <br />"
                             "Die Felder fallnummer, jahr und mitarbeiter werden in das Notizfeld "
                             "der neuen Akte bernommen.",
                      n_col=2,
                      )
                    ],
                   [h.String(string='geburtsdatum:'
                             ),
                    h.String(string=
                             "Tag . Monat . Jahr <br /> "
                             "Das Jahr muss vierstellig sein, Tag und Monat ein- oder zweistellig, <br /> "
                             "z.B. 1.1.2002, 10.01.1999"
                             ),
                    ],
                   [h.String(string='geschlecht:'
                             ),
                    h.String(string='m oder w, alles andere ist ungltig'
                             ),
                    ],
                   [h.String(string='jahr:'
                             ),
                    h.String(string='eine vierstellige Zahl'
                             ),
                    ],
                   [h.String(string='strasse:'
                             ),
                    h.String(string='Endung mit <em>strasse, Strasse, strae, Strae</em> '
                             'werden zu <em>str.</em> bzw. <em>Str.</em> normalisiert.'
                             ),
                    ],
                   [h.String(string='hausnummer:'
                             ),
                    h.String(string='ein- bis dreistellige Zahl und evt. ein Buchstabe als '
                             'Zusatz, der zum grogeschriebenen Buchstaben normalisiert wird.'
                             ),
                    ],
                   [h.String(string='plz:'
                             ),
                    h.String(string='fnfstellige Zahl'
                             ),
                    ],
                   [h.String(string='memo:'
                             ),
                    h.String(string='Hier kann auch lngerer Text stehen, auch ber mehrere Zeilen hinweg.'
                             'Anfhrungsstriche, die im Feld vorkommen, mssen verdoppelt werden, '
                             'damit sie nicht als Ende des Feldes interpretiert werden.'
                             ),
                    ],
                   ],
            )
        altdimport = h.FieldsetFormInputTable(
            legend='Altdaten importieren (evt. bereits vorhandene aus einem frheren Import werden berschrieben)',
            name='altimport',action="altimport",method="post",
            daten=[[h.UploadItem(label='Lokaler Dateiname',
                                 name='datei',
                                 tip='CSV-Datei mit Altdaten hochladen',
                                 class_="textboxverylarge",
                                 ),
                    ]],
                button=h.Button(value="Hochladen",
                                name='op',
                                tip="Gewhlte Datei mit Altdaten hochladen",
                                type='submit',
                                n_col=2,
                                ),
            )
        res = h.Page(
            title='Altdaten importieren',
            breadcrumbs = (('Administratorhauptmen', 'menu'),
                           ),
            hidden=(),
            rows=(self.get_hauptmenu(),
                  altdimport,
                  hinweise_csv,
                  hinweise_felder,
                  ),
            )
        return res.display()

class altlist(Request.Request, akte_share):
    permissions = Request.ABFR_PERM

    def beratungen_fall(self, 
                        mitarbeiter=None,
                        klvorname=None,
                        klnachname=None,
                        ):
        join=[('akte', 'fall.akte_id=akte.id'),]
        # nur die eigene Stelle
        where = "akte.stzbg=%s" % self.stelle['id']
        if klvorname:
            expr = escape('%' + klvorname + '%')
            where += " and (akte.vn like %s)" % (expr,)
        if klnachname:
            expr = escape('%' + klnachname + '%')
            where += " and (akte.na like %s)" % (expr,)
        if mitarbeiter:
            join.append(('zustaendigkeit', 'zustaendigkeit.fall_id=fall.id'))
            join.append(('mitarbeiter', 'zustaendigkeit.mit_id=mitarbeiter.id'))
            where += " and mitarbeiter.id = %s" % mitarbeiter['id']
            # nur die letzte Zustndigkeit deren Endedatum gleich dem ZDA-Datum ist
            where += """ and fall.zday = zustaendigkeit.ey and 
                     fall.zdam = zustaendigkeit.em and 
                     fall.zdad = zustaendigkeit.ed"""
        fall_list = FallList(
            where=where,
            join=join,
            )
        sort = ('akte__na', 'akte__vn', 'bgy', 'bgm', 'bgd')
        fall_list.sort(*sort)
        return fall_list


    def processForm(self, REQUEST, RESPONSE):
        vorname = self.form.get('vorname', '')
        name = self.form.get('name', '')
        fallnummer = self.form.get('fallnummer', '')
        rest = self.form.get('rest', '')
        where = []
        altdaten = []
        if vorname:
            vorname_e = escape('%' + vorname + '%')
            where.append("vorname like %(vorname_e)s" % locals())
        if name:
            name_e = escape('%' + name + '%')
            where.append("name like %(name_e)s" % locals())
        if fallnummer:
            fallnummer_e = escape('%' + fallnummer + '%')
            where.append("fallnummer like %(fallnummer_e)s" % locals())
        if rest:
            rest_e = escape('%' + rest + '%')
            where.append("(strasse like %(rest_e)s or "
                         "ort like %(rest_e)s or "
                         "plz like %(rest_e)s or "
                         "memo like %(rest_e)s or "
                         "mitarbeiter like %(rest_e)s)" % locals())
        if where:
            #logging.info("Altdatensuche where: %s" % ' and '.join(where))
            altdaten = AltdatenList(where=' and '.join(where), order='name, vorname, fallnummer')
            #logging.info("Altdatensuche gefunden: %s Datensaetze" % len(altdaten))
        else:
            #logging.info("Altdatensuche: Keine Suchkriterien")
            pass
                        
        aktuelle_daten = []
        if vorname or name:
            mitarbeiter = None
            if self.mitarbeiter['benr__code'] == 'bearb':
                mitarbeiter = self.mitarbeiter
            #logging.info("Suche in aktuellen Daten: vorname: %s name: %s" % (vorname, name))
            aktuelle_daten = self.beratungen_fall(mitarbeiter=mitarbeiter,
                                                  klvorname=vorname,
                                                  klnachname=name,
                                                  )
            #logging.info("Gefunden in aktuellen Daten: %s Datensaetze" % len(aktuelle_daten))
        else:
            #logging.info("Suche in aktuellen Daten: Kein Vorname oder Nachname")
            pass
            
        auswahl_kriterien = h.FieldsetFormInputTable(
            name="altlist",
            #action="abfragedef",
            action="altlist",
            method="post",
            hidden = (),
            legend='Suchkriterien eingeben',
            daten=[[h.TextItem(label='Vorname enthlt',
                               name='vorname',
                               value=vorname,
                               tip='Gro/Kleinschreibung egal, kann mittendrin erscheinen',
                               ),
                    h.TextItem(label='Nachname enthlt',
                               name='name',
                               value=name,
                               tip='Gro/Kleinschreibung egal, kann mittendrin erscheinen',
                               ),
                    ],
                   [h.TextItem(label='Alte Fallnummer enthlt',
                               name='fallnummer',
                               value=fallnummer,
                               tip='Gro/Kleinschreibung egal, kann mittendrin erscheinen',
                               ),
                    h.TextItem(label='Memo-, Mitarbeiter- oder Adressfeld enthlt',
                               name='rest',
                               value=rest,
                               tip='Zeichenkette erscheint in einem der Felder',
                               ),
                    ],
                   [h.Dummy(n_col=4)],
                   [h.Button(value="Anzeigen",
                             name='op',
                             tip="Altdaten entsprechend den Suchkriterien anzeigen",
                             type='submit',
                             n_col=4,
                             ),
                    ],
                   ],
            )
        daten = []
        for altd in altdaten:
            daten.append([h.String(string=altd['vorname']),
                          h.String(string=altd['name']),
                          h.String(string=altd['geburtsdatum']),
                          h.String(string=altd['geschlecht']),
                          h.String(string=altd['jahr']),
                          h.String(string=altd['fallnummer']),
                          h.String(string=altd['mitarbeiter']),
                          h.String(string="%(strasse)s %(hausnummer)s, %(plz)s %(ort)s" % altd),
                          h.String(string="%(telefon1)s, %(telefon2)s" % altd),
                          h.CheckItem(label='',
                                      name='uebern',
                                      value=altd['id'],
                                      checked=False,
                                      tip='Hier markieren, um die Daten in die Neuaufnahme zu bernehmen',
                                      ),
                          ])
            daten.append([h.String(string=altd['memo'],
                                   n_col=9),
                          h.DummyItem(),
                          ])
            daten.append([h.Dummy(n_col=11),
                          ])
        altdaten_table = h.FieldsetFormDataTable(
            name="altdaten",
            #action="abfragedef",
            action="akteneu",
            method="post",
            hidden = (),
            legend='Altdaten passend zu den Suchkriterien',
            headers=('Vorname', 'Name', 'Geb.', 'm/w', 'Jahr', 'Fallnr.', 'Mitarbeiter',
                     'Adresse', 'Telefon', '', 'bern.'),
            daten=daten,
            no_button_if_empty=False,
            buttons=[daten and h.Button(value="bernehmen",
                              name='op',
                              tip="Markierten Datensatz fr die Neuaufnahme bernehmen",
                              type='submit',
                              ) or None,
                     h.Button(value="Abbrechen",
                              name='op',
                              tip="Zurck zur Neuaufnahme ohne Datenbernahme",
                              type='button',
                              onClick="go_to_url('akteneu')",
                              ),
                     ],
            empty_msg="Keine passenden Altdaten gefunden.",
            )
        aktuelle_daten_table = h.FieldsetDataTable(
            legend='Klienten passend zu den Suchkriterien Vorname/Nachname',
            headers=('Fallnr', 'Vorname', 'Name', 'Geb.', 'Beginn', 'z.d.A.', 'Zustndig',),
            daten=[[h.Link(string=fall['fn'],
                           url='klkarte?akid=%(akte_id)s' % fall),
                    h.String(string=fall['akte__vn']),
                    h.String(string=fall['akte__na']),
                    h.String(string=fall['akte__gb']),
                    h.Datum(date=fall.getDate('bg')),
                    h.Datum(date=fall.getDate('zda')),
                    h.String(string=fall['zuletzt_zustaendig__mit__na']),
            ] for fall in aktuelle_daten],
            empty_msg="Keine Klienten gefunden.",
            )
        res = h.Page(
            title='Altdaten',
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Neue Akte anlegen', 'akteneu'),
                           ),
            rows=(self.get_hauptmenu(),
                  auswahl_kriterien,
                  altdaten_table,
                  aktuelle_daten_table,
                  ),
            )
        #logging.info("Altdatensuche: Seite aufgebaut. %s rows" % len(res.rows))
        return res.display()

