# coding: latin-1

"""Modul Beratungskontakte."""

from ebkus.app import Request
from ebkus.app.ebapi import Code, Akte, Fall, MitarbeiterList, Beratungskontakt, BeratungskontaktList, \
     today, cc, cn, bcode, check_int_not_empty, check_list, EE
from ebkus.app.ebapih import get_codes, make_option_list
from ebkus.config import config

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

class _bkont(Request.Request, akte_share):
    # bkontneu und updbkont mit derselben Routine,
    # nur anders parametrisiert
    bkontdok_feldinfo = [
# name label_erst label_folge cols rows in_erst in_folge
('inhalt', 'Wesentliche Inhalte des Kontakts:', None, "Wesentliche Inhalte:", 75, 10, True, True,),
('diagnose', 'Diagnostische Einschtzung:', 'Ggf. diagnostisch relevante Erkenntnisse:', "Diagnostische Erkenntnisse:", 75, 2, True, True,),
('vorgehen', 'Weiteres Vorgehen:', None, 'Weiteres Vorgehen:', 75, 2, True, True,),
('kontrakt', 'Beratungskontrakt, inkl. Zielvereinbarung(en) und prognostizierte Dauer der Hilfe:', 'nderungen des Beratungskontraktes notwendig?', 'Beratungskontrakt:', 75, 2, True, True,),
('richtung', 'Richtung der Intervention und Hilfeplanung, ggf. formal nach 36 SGB VIII:', 'nderungen in der Richtung der Intervention notwendig? Ggf. benennen:', 'Richtung der Intervention:', 75, 2, True, True,),
('verlauf', 'Ggf. Besonderheiten des Beratungsverlaufes bzw. besondere Vereinbarungen (z.B. Abklrung selbst- bzw. fremdgefhrdenden Verhaltens, Abklrung kinderschutzrechtlicher Aspekte):', 
            'Ggf. Besonderheiten des Beratungsverlaufes (z.B. Abklrung selbst- bzw. fremdgefhrdenden Verhaltens, Abklrung kinderschutzrechtlicher Aspekte):', 'Besonderheiten des Verlaufs:', 75, 2, True, True,),
('vereinbarung', 'Besondere Vereinbarungen:', None, 'Besondere Vereinbarungen:', 75, 2, False, True,),
]
    # zu faul neue kategorie zu definieren, zu fuss
    termin_elements = [{'id':1, 'name': 'Klient meldet sich bei Bedarf'},
                       {'id':2, 'name': 'fester Termin vereinbart'},
                       {'id':3, 'name': 'Fall beendet'}]

    def _process(self, REQUEST, RESPONSE,
                 title,
                 legendtext,
                 fall, # einer der Flle, der am Kontakt teilnahme
                       # von dessen Klientenkarte aus man kam
                 bkont,
                 file,
                 ):
        beratungskontakte = fall['beratungskontakte']
        # wahr, wenn config.BERATUNGSKONTAKT_DOKUMENTATION True ist und wenn alle bereits
        # existierenden Kontakte das erstgesprch nicht None ist.
        #BERATUNGSKONTAKT_DOKUMENTATION = fall['do_bkont_dok']
        is_erstgespraech = False
        hidden = []
        if config.BERATUNGSKONTAKT_DOKUMENTATION:
            # Der Beratungskontakt gilt als Erstkontakt, wenn es sich um den ersten
            # eingetragenen Beratungskontakt fr den Fall berhaupt handelt.
            # Das Feld erstgespraech wird mit 'ja' belegt.
            if len(beratungskontakte) == 0:
                # Erstgespraech
                is_erstgespraech = True
            else:
                for bk in beratungskontakte:
                    # update erstgespraech
                    if bk == bkont and bk['erstgespraech'] == cc('ja_nein', 1):
                        is_erstgespraech = True
            if is_erstgespraech:
                hidden.append(('erstgespraech', cn('ja_nein', 'ja')))
            else:
                hidden.append(('erstgespraech', cn('ja_nein', 'nein')))
        if config.BERATUNGSKONTAKTE_BS:
            if is_erstgespraech:
                hidden.append(('bkfallid', fall['id'])) # wg readonly unten
            beratungs_kontakt_bearbeiten = h.FieldsetInputTable(
                legend = '%s %s %s' % (legendtext, fall['akte__vn'], fall['akte__na']),
                daten = [[h.SelectItem(label='Mitarbeiter',
                                       name='mitid',
                                       options=self.for_mitarbeiter(
                sel=[m['id'] for m in bkont['mitarbeiter']]),
                                       multiple=True,
                                       size=8,
                                       ),
                          h.SelectItem(label='Klienten',
                                       name='bkfallid',
                                       class_='listbox280',
                                       options=self.for_klienten(
                sel=[f['id'] for f in bkont['faelle']]),
                                       multiple=True,
                                       size=8,
                                       # nur ein Klient fr das Erstgesprch
                                       readonly=True if is_erstgespraech else False,
                                       ),
                          ],
                          [h.DatumItem(label='Datum/Uhrzeit',
                                       name='k',
                                       date =  bkont.getDate('k'),
                                       time = bkont.getTime('k'),
                                       ),
                           h.SelectItem(label='Art des Kontaktes',
                                        name='art_bs',
                                        class_='listbox280',
                                        options=self.for_kat('kabs', sel=bkont['art_bs'])
                                        ),
                           ],
                         [h.CheckItem(label='frheres Quartal',
                                       name='fruehquart',
                                       value='1',
                                       checked=False,
                                      tip="Markieren, um Beratungskontakt aus frherem Quartal zuzulassen"
                                       ),
                          h.SelectItem(label='Teilnehmer',
                                       name='teilnehmer_bs',
                                       multiple=True,
                                       size=5,
                                       tip='Wer hat teilgenommen?',
                                       rowspan=3,
                                       options=self.for_kat('teilnbs', sel=bkont['teilnehmer_bs'])
                                       ),
                          ],
                          [h.TextItem(label='Dauer in Minuten',
                                      name='dauer',
                                      value=bkont['dauer'],
                                      tip='Bitte in 10-Schritten angeben, z.B. 10, 20, 40, etc.',
                                      class_='textboxsmall',
                                      maxlength=3,
                                      ),
                           ],
                          [h.TextItem(label='Anzahl',
                                      name='anzahl',
                                      value=bkont['anzahl'],
                                      tip='Anzahl der Teilnehmer',
                                      class_='textboxsmall',
                                      maxlength=2,
                                      ),
                           ],
                          [h.CheckItem(label='In offener Sprechstunde',
                                       name='offenespr',
                                       value=cn('ja_nein', 'ja'),
                                       checked=bkont['offenespr']==cn('ja_nein', 'ja'),
                                       ),
                           ],
                         None if config.BERATUNGSKONTAKT_DOKUMENTATION else 
                          [h.TextareaItem(label='Notiz',
                                      name='no',
                                      value=bkont['no'],
                                      class_='textareanormal100prz',
                                      maxlength=1024,
                                      n_col=4,
                                      ),
                           ],
                          ],
                )
        else:
            beratungs_kontakt_bearbeiten = h.FieldsetInputTable(
                legend = '%s %s %s' % (legendtext, fall['akte__vn'], fall['akte__na']),
                daten = [[h.SelectItem(label='Mitarbeiter',
                                       name='mitid',
                                       options=self.for_mitarbeiter(
                sel=[m['id'] for m in bkont['mitarbeiter']]),
                                       multiple=True,
                                       size=8,
                                       ),
                          h.SelectItem(label='Klienten',
                                       name='bkfallid',
                                       class_='listbox280',
                                       options=self.for_klienten(
                sel=[f['id'] for f in bkont['faelle']]),
                                       multiple=True,
                                       size=8,
                                       n_col=4,
                                       ),
                          ],
                         [h.DatumItem(label='Datum',
                                      name='k',
                                      date =  bkont.getDate('k')),
                          h.SelectItem(label='Art des Kontaktes',
                                     name='art',
                                     options=self.for_kat('fska', sel=bkont['art'])),
                          h.SelectItem(label='Dauer',
                                       name='dauer_kat',
                                       options=self.for_kat('fskd',
                                                            sel=bcode('fskd', bkont['dauer'], 0)['id'])),
                          ],
                         None if config.BERATUNGSKONTAKT_DOKUMENTATION else 
                          [h.TextItem(label='Notiz',
                                      name='no',
                                      value=bkont['no'],
                                      class_='textboxverylarge',
                                      maxlength=1024,
                                      n_col=4,
                                      ),
                           ],
                          ],
                )
        beratung_dokumentation = None
        if config.BERATUNGSKONTAKT_DOKUMENTATION:
            daten = []
            erstellungs_und_aenderungsdatum = [
                h.String(string="Erstellt am:", 
                         class_='normaltext',
                         n_col=2,), 
                h.Datum(date=bkont.getDate('d'), class_='normaltext'),
                h.Dummy(n_col=3),
                ]
            if bkont.getDate('dlc').check(can_be_zero=False) and bkont.getDate('d') < bkont.getDate('dlc'):
                erstellungs_und_aenderungsdatum += [
                    h.String(string="Gendert am:", 
                             class_='normaltext',
                             n_col=2), 
                    h.Datum(date=bkont.getDate('dlc'), class_='normaltext'),
                    h.Dummy(n_col=3),
                    ]
            daten.append(erstellungs_und_aenderungsdatum)
            # TODO
            # else:
            #     noch_wieviel_tage_bearbeiten = config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST
            #     erstellungs_und_aenderungsdatum += [h.String(string="Zu bearbeiten bis zum ", class_='normaltext', n_col=2)]
            
            if is_erstgespraech:
                daten.append([h.SelectItem(label="Aufklrung ber die Arbeitsweise der Beratungsstelle "
                                           "(v.a. hinsichtlich Schweigepflicht, Verhalten bei "
                                           " Kindeswohlgefhrdung, Grundstze der Arbeit) erfolgt?",
                                           name='aufklaerung',
                                           # leeres Element erzwingen wenn noch nicht ausgewhlt
                                           options=self.for_kat('ja_nein', sel=bkont['aufklaerung'], add_empty=True),
                                            n_col=6,
                                           n_label = 5,
                                           class_='listbox60',
                                           )])
            for name, label_erst, label_folge, label_kurz, cols, rows, in_erst, in_folge in self.bkontdok_feldinfo:
                if is_erstgespraech:
                    label = label_erst
                else:
                    label = label_folge if label_folge else label_erst
                if (is_erstgespraech and in_erst) or (not is_erstgespraech and in_folge):
                    daten.append([h.String(string=label,
                                           n_col=6,
                                           class_='labeltext',
                                           )]
                                 )
                    daten.append([h.TextareaItem(label='',
                                                name=name,
                                                value=bkont[name],
                                                class_='textarea',
                                                n_col=6,
                                                cols=str(cols),
                                                rows=str(rows),
                                                n_label=1,
                                                 )]
                                 )
            daten.append([h.CheckItem(label="Fallbesprechung gem.  8a, SGB VIII notwendig?",
                                      name='fall8a',
                                      value=cn('ja_nein', 'ja'),
                                      checked=bkont['fall8a']==cn('ja_nein', 'ja'),
                                      n_label=3,
                                      n_col=4,
                                      ),
                          h.SelectItem(label="Nchster Termin",
                                       name='termin',
                                       class_='listbox195',
                                       options=make_option_list(self.termin_elements,
                                                                'id', 'name',
                                                                selected=bkont['termin'],
                                                                empty_option=True),
                                       ),
                          ])
            beratung_dokumentation =  h.FieldsetInputTable(
                legend = '%s %s %s' % ('Beratungsdokumentation' + (' (Erstgesprch)' if is_erstgespraech else ''), fall['akte__vn'], fall['akte__na']),
                daten = daten)

        #print 'BERATUNGSKONTAKT_DOKUMENTATION', BERATUNGSKONTAKT_DOKUMENTATION
        #print 'is_erstgespraech', is_erstgespraech
        hidden += [("akid", fall['akte_id']),
                   ("stz", bkont['stz']),
                   ("bkontid", bkont['id']),
                   ("fallid", fall['id']),
                   ("file", file),
                   ]
        res = h.FormPage(
            title=title,
            name="beratungskontakt",action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=hidden,
            rows=(beratungs_kontakt_bearbeiten,
                  beratung_dokumentation,
                  h.SpeichernZuruecksetzenAbbrechen(),
                  self.get_beratungskontakte(beratungskontakte, 
                                             akte=fall['akte'],
                                             view_button=True),
                  )
            )
        return res.display()

class bkontneu(_bkont):
    """Neue Beratungskontakt eintragen. (Tabelle: Beratungskontakt.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fr den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        benr = self.mitarbeiter['benr__code']
        if benr in ('verw',):
            # Verwaltungskraft kann Eintrge fr andere Mitarbeiter machen
            mitarbeiter = [fall['zustaendig__mit']]
        else:
            mitarbeiter = [self.mitarbeiter]
        bkont = Beratungskontakt()
        bkont.init(
            id=Beratungskontakt().getNewId(),
            faelle=[fall],
            mitarbeiter=mitarbeiter,
            teilnehmer_bs=cc('teilnbs', '0'),
            anzahl=1,
            art_bs=cc('kabs', '1'),
            art=None,
            dauer='',
            offenespr=cn('ja_nein', 'nein'),
            no='',
            stz=self.stelle['id'],
           )
        bkont.setDate('k', today())
        return self._process(REQUEST, RESPONSE,
                             title="Neuen Beratungskontakt eintragen",
                             legendtext="Neuen Beratungskontakt eintragen fr",
                             fall=fall,
                             bkont=bkont,
                             file='bkonteinf',
                             )
        
class updbkont(_bkont):
    """Beratungskontakt ndern. (Tabelle: Beratungskontakt.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        id = self.form.get('bkontid')
        if not id:
            self.last_error_message = "Keine ID fr den Beratungskontakt erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall_id = self.form.get('fallid')
        if not fall_id:
            self.last_error_message = "Keine Fall-ID fr den Beratungskontakt erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        bkont = Beratungskontakt(id)
        fall = Fall(fall_id)
        #TODO rausnehmen
        assert fall in bkont['faelle']
        return self._process(REQUEST, RESPONSE,
                             title="Beratungskontakt bearbeiten",
                             legendtext="Beratungskontakt bearbeiten von",
                             fall=fall,
                             bkont=bkont,
                             file='updbkont',
                             )

class rmbkont(Request.Request):
    """Beratungskontakt lschen."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        id = self.form.get('bkontid')
        if not id:
            self.last_error_message = "Keine ID fr den Beratungskontakt erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall_id = self.form.get('fallid')
        if not fall_id:
            self.last_error_message = "Keine Fall-ID fr den Beratungskontakt erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        bkont = Beratungskontakt(id)
        fall = Fall(fall_id)
        #TODO rausnehmen
        assert fall in bkont['faelle']
        res = h.SubmitOrBack(
            legend='Beratungskontakt lschen',
            action='klkarte',
            method='post',
            hidden=(('file', 'removebkont'),
                    ('bkontid', bkont['id']),
                    ('fallid', fall_id),
                    ),
            zeilen=('Soll der Beratungskontakt vom %s endgltig gelscht werden?' % bkont.getDate('k'),
                    'Beteiligte Klienten: %s' % ', '.join([f['name'] for f in bkont['faelle']]),
                    'Beteiligte Mitarbeiter: %s' % ', '.join([m['na'] for m in bkont['mitarbeiter']]),
                    ),
            )
        return res.display()

class bkontbsabfr(Request.Request, akte_share):
    permissions = Request.ABFR_PERM
    def _init_res(self):
        netto = {}
        brutto = {}
        anzahl = {}
        anzahl_offenespr = {}
        for ka in get_codes('kabs'):
            netto[ka['code']] = 0
            brutto[ka['code']] = 0
            anzahl[ka['code']] = 0
            anzahl_offenespr[ka['code']] = 0
        netto['offenespr'] = 0
        brutto['offenespr'] = 0
        return netto, brutto, anzahl, anzahl_offenespr
    def _add_res(self, summe, summand):
        for k,v in summand.items():
            summe[k] += v
    def count_row(self, netto, brutto, anzahl, anzahl_offenespr, bkont):
        "Einen Kontakt auszhlen"
        art = bkont['art_bs__code']
        dauer = bkont['dauer']
        netto[art] += dauer
        brutto[art] += bkont['brutto']
        anzahl[art] += 1
        if bkont['offenespr'] == cn('ja_nein', 'ja'):
            # Offene Sprechstunde
            anzahl_offenespr[art] += 1
            netto['offenespr'] += dauer
            brutto['offenespr'] += bkont['brutto']
    def count(self, stz_list, von_jahr, bis_jahr, quartal=None, monat=None):
        """Kontaktzeiten auszhlen.
        Ergebnis ist abhngig von Berechtigungen:
        - verw: alle Mitarbeiter der spezifierten Stellen
                Summe ber alle Mitarbeiter der spezifizierten Stellen
        - bearb: der Mitarbeiter selber

        Berechnung der Summen fr einen Mitarbeiter:
        - es gelten grundstzlich die Zeiten, die ein Mitarbeiter aufwendet,
          unabhngig davon, wieviele Klienten betroffen sind. Es wird also jede
          Zeile der Beratungskontakt-Tabelle, in der der Mitarbeiter aufgefhrt ist,
          bercksichtigt.
        - Berechnung der Summe fr die Beratungsstelle:
          es wird ebenso wie bei Mitarbeitern die Summe des Aufwandes der Beratungsstelle
          berechnet. Dh die Summe fr die Stelle ist gleich der Summe aus allen
          Mitarbeitern.
          Der Aufwand pro Klient spielt hingegen keine Rolle.

        - Fr beide Berechnungen gilt: Jede Zeile der Beratungskontakt-Tabelle geht so
          oft in die Summierung ein, wie die Anzahl der am Kontakt beteiligten Mitarbeiter.

        Daraus ergibt sich, dass die Aufwnde unabhngig davon sind, wieviel Gruppenarbeit
        gemacht wird. 

        Die pro Klient aufgewendete Zeit geht hingegen nur in die Bundesstatistik ein.
        Die Summe der Kontakte in der Bundesstatistik
        ist um so hher, je mehr Gruppenarbeit gemacht wird, bei gleichem Zeiteinsatz von
        Mitarbeitern.
        """
        stellen = ','.join(["%s" % s for s in stz_list])
        where = ("ky is not NULL and "
        "ky >= %(von_jahr)s and ky  <= %(bis_jahr)s and "
        "stz in ( %(stellen)s )") % locals()
        if quartal:
            assert von_jahr == bis_jahr
            monate = range(1,13)[3*quartal-3:3*quartal]
            where += " and km in (%s)" % ','.join([str(i) for i in monate])
        if monat:
            assert not quartal
            assert von_jahr == bis_jahr
            #monate = range(1,13)[3*quartal-3:3*quartal]
            monate = [monat]
            where += " and km in (%s)" % ','.join([str(i) for i in monate])
        bkont_list = BeratungskontaktList(where=where)
        benr = self.mitarbeiter['benr__code']
        mitarbeiter = []
        if benr == 'bearb':
            mitarbeiter = [self.mitarbeiter]
        elif benr == 'verw':
            bearb_benr_id = cc('benr', 'bearb')
            status = cc('status', 'i')
            mitarbeiter = MitarbeiterList(
                where='stz in (%(stellen)s) and stat = %(status)s and benr = %(bearb_benr_id)s' % locals(),
                order='na')
        # fr jeden Mitarbeiter einen Zhler mit netto, brutto
        res = {}
        for m in mitarbeiter:
            #id --> (netto, brutto)
            res[m['id']] = self._init_res()
        for b in bkont_list:
            #for f in b('mit_id', 'mit1_id', 'mit2_id'):
            for m in b['mitarbeiter']:
                # jeder Eintrag wird sooft gezhlt, wie Mitarbeiter darin vorkommen
                try:
                    netto, brutto, anzahl, anzahl_offenespr = res[m['id']]
                    self.count_row(netto, brutto, anzahl, anzahl_offenespr, b)
                except:
                    pass # falls Mitarbeiter in bkont, aber nicht in MitarbeiterList
                         # sollte nicht vorkommen, wenn doch, ignorieren
        summe_netto, summe_brutto, summe_anzahl, summe_anzahl_offenespr = self._init_res()
        for netto, brutto, anzahl, anzahl_offenespr in res.values():
            self._add_res(summe_netto, netto)
            self._add_res(summe_brutto, brutto)
            self._add_res(summe_anzahl, anzahl)
            self._add_res(summe_anzahl_offenespr, anzahl_offenespr)
        res['summe'] = summe_netto, summe_brutto, summe_anzahl, summe_anzahl_offenespr
        return mitarbeiter, res
            
    def processForm(self, REQUEST, RESPONSE):
        #print 'FORM', self.form
        von_jahr = self.form.get('von_jahr')
        bis_jahr = self.form.get('bis_jahr')
        if bis_jahr:
            bis_jahr = check_int_not_empty(self.form, 'bis_jahr', "Jahr fehlt")
        else:
            bis_jahr = today().year
        von_jahr = check_int_not_empty(self.form, 'von_jahr', "Jahr fehlt", bis_jahr)
        if von_jahr > bis_jahr:
            von_jahr = bis_jahr
#         stellen_ids = [int(id) for id in check_list(self.form, 'stz', 'Keine Stelle',
#                                                     [self.stelle['id']])]
        stellen_ids = [self.stelle['id']]
        quartal = self.form.get('quartal')
        if quartal:
            quartal = check_int_not_empty(self.form, 'quartal', 'Fehler im Quartal')
            if von_jahr != bis_jahr:
                raise EE('Quartalsauswertungen nur in einem Jahr mglich')
        else:
            quartal == None
        monat = self.form.get('monat')
        if monat:
            monat = check_int_not_empty(self.form, 'monat', 'Fehler im Monat')
            if von_jahr != bis_jahr:
                raise EE('Monatsauswertungen nur in einem Jahr mglich')
        else:
            monat == None
        if monat and quartal:
            raise EE('Monats- und Quartalsauswertung nicht gleichzeitig mglich')
        mitarbeiter, res = self.count(stellen_ids, von_jahr, bis_jahr,
                                      quartal, monat)
        kontakt_arten = [c['code'] for c in get_codes('kabs')]
        def row(name, tupl):
            netto, brutto, anzahl, anzahl_offenespr = tupl
            row = [h.String(string=name)]
            summe_netto = 0
            summe_brutto = 0
            summe_anzahl = 0
            summe_anzahl_offenespr = 0
            for ka in kontakt_arten:
                summe_netto += netto[ka]
                summe_brutto += brutto[ka]
                summe_anzahl += anzahl[ka]
                summe_anzahl_offenespr += anzahl_offenespr[ka]
                row.append(h.String(string="%s / %s / %s / %s" %
                                    (netto[ka], brutto[ka], anzahl[ka], anzahl_offenespr[ka]),
                                    tip='Netto Minuten/Brutto Minuten/Anzahl/Anzahl in offener Sprechstunde')
                           )
            row.append(h.String(string="%s / %s / %s / %s" %
                                    (summe_netto, summe_brutto, summe_anzahl, summe_anzahl_offenespr),
                                    tip='Netto Minuten/Brutto Minuten/Anzahl/Anzahl in offener Sprechstunde')
                           )
            return row
        mitarbeiter_daten = [row(m['name'], res[m['id']])
                             for m in mitarbeiter]
        stellen_row = row(', '.join([Code(s)['name'] for s in stellen_ids]),
                          res['summe'])
        headers = [''] + [c['name'] for c in get_codes('kabs')] + ['Summe']
        fuer = ''
        if von_jahr < bis_jahr:
            fuer += " fr %(von_jahr)s bis %(bis_jahr)s" % locals()
        else:
            fuer += " fr %(bis_jahr)s" % locals()
        if quartal:
            fuer += " Quartal %(quartal)s" % locals()
        if monat:
            fuer += " Monat %(monat)s" % locals()
        tabelle_mitarbeiter = h.FieldsetDataTable(
            legend='Beratungskontaktzeiten Mitarbeiter' + fuer,
            headers=headers,
            daten=mitarbeiter_daten,
            )
        if self.mitarbeiter['benr__code'] == 'bearb':
            tabelle_stellen = None
        else:
            tabelle_stellen = h.FieldsetDataTable(
                legend='Beratungskontaktzeiten summiert ber Mitarbeiter' + fuer,
                headers=headers,
                daten=[stellen_row],
                )
        res = h.FormPage(
            title='Auswertung Beratungskontaktzeiten',
            name="bkontform",action="bkontbsabfr",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden = (),
            rows=(self.get_auswertungs_menu(),
                  self.grundgesamtheit(von_jahr=von_jahr,
                                       bis_jahr=bis_jahr,
                                       quartal=quartal,
                                       monat=monat,
                                       show_monat=True,
                                       show_stelle = False,
                                       stellen_ids=stellen_ids,
                                       legend='Zeitraum whlen',
                                       submit_value='Anzeigen'),
                  tabelle_mitarbeiter,
                  tabelle_stellen,
                  ),
            )
        return res.display()

def hatte_kontakt_im_abschlussjahr(fall, jahr):
    # nur auf Flle anwenden, wo es berhaupt Beratungskontakte gibt
    kontakte_im_jahr, kontakte_insgesamt = get_jgh_kontakte(fall, jahr)
    #print 'KONTAKTE', kontakte_insgesamt, kontakte_im_jahr,
    if not kontakte_insgesamt:
        return True
    if kontakte_im_jahr:
        return True
    return False
    

def get_jgh_kontakte(fall, jahr):
    """Ermittelt Anzahl der Kontakte sowohl fr den ganzen Fall
    als auch fr das gegebene Jahr.

    In Braunschweig anders als sonst. Berechnung ein ebapi,
    attributemethod 'jghkontakte'
    """
    bkont_list = fall['beratungskontakte']
    kontakte_im_jahr = kontakte_insgesamt = 0
    if config.BERATUNGSKONTAKTE:
        for row in bkont_list:
            k = row['jghkontakte'] # definiert in ebapi.py und ber art_bs__dok
            if row['ky'] == jahr:
                kontakte_im_jahr += k
            kontakte_insgesamt += k
    else:
        kontakte_im_jahr = 0
        kontakte_insgesamt = 0
    return kontakte_im_jahr, kontakte_insgesamt

def get_fs_kontakte(fall, fs):
    """Anzahl der Kontakte fr jede Kontaktart aus den Beratungskontakten
    in das Fachstatistikobjekt eintragen.
    """
    code2feld = {'1': 'kkm', '2': 'kkv', '3': 'kki', '4': 'kpa', '5': 'kfa',
                 '6': 'ksoz', '7': 'kleh', '8': 'kerz', '9': 'kkonf', '10': 'kson',
                 }
    if config.BERATUNGSKONTAKTE and not config.BERATUNGSKONTAKTE_BS:
        for f in  code2feld.values():
            fs[f] = 0
        bkont_list = []
        if fall:
            bkont_list = fall['beratungskontakte']
        summe = 0
        for bkont in bkont_list:
            # 0, 1, 2, 3 entsprechend dem code von fskd
            anzahl = int(bcode('fskd', bkont['dauer'], 0)['code'])
            code = bkont['art__code']
            if code not in [str(i) for i in range(1,11)]:
                # Falls nicht-standard Kontaktarten bei Beratungskontakten hinzu
                # gefgt wurden, als Sonstige (code='10') behandeln.
                code = '10'
            feld = code2feld[code]
            fs[feld] += anzahl
            summe += anzahl
        fs['kat'] = summe


class bkontdruck(Request.Request, akte_share):
    permissions = Request.ABFR_PERM
    def processForm(self, REQUEST, RESPONSE):
        #print 'FORM', self.form
        fall_id = self.form.get('fallid')
        if not fall_id:
            self.last_error_message = "Keine Fall-ID fr Beratungskontakte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fall_id)
        beratungskontakt_list = fall['beratungskontakte']
        if not beratungskontakt_list:
            raise EE('XXXKeine Beratungskontakte vorhanden')
        titelstr = ("Beratungskontakte fr %s " % (fall['name']) +
                    "bis %(day)d.%(month)d.%(year)d " % today() +
                    '(Fallnummer: %s)' % fall['fn'])
        ueberschrift = h.TableDataTable(
            daten=[[h.String(string=titelstr,
                                   class_='tabledatabold'),
                    ],
                   [h.Dummy()],
                   ],
            )

        beratungskontakte = self.get_beratungskontakte(
            beratungskontakt_list,
            cls=h.TableDataTable,
            header_class='tabledatabold',
            )
        res = h.Page(
            title='Beratungskontakte %(fn)s ' % fall + \
                'bis %(day)d.%(month)d.%(year)d)' % today(),
            ueberschrift=ueberschrift,
            rows=(ueberschrift,
                  beratungskontakte,
                  ),
            )
        return res.display()



class viewbkont(_bkont):
    permissions = Request.KLKARTE_PERM
    """Einen Beratungskontakt readonly auf einer Seite darstellen mit Links
    zum vorherigen und dem nchsten Kontakt (beschrnkt auf den aktuellen Fall).
    Relevant fr Kontakte mit Beratungsdokumentation."""
    def processForm(self, REQUEST, RESPONSE):
        bkont_id = self.form.get('bkontid')
        if not bkont_id:
            self.last_error_message = "Keine ID fr den Beratungskontakt erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        bkont = Beratungskontakt(bkont_id)
        # Der Teilnehmer, fr den der Beratungskontakt aufgerufen wird.
        # Da ein Beratungskontakt mehrere Klienten als Teilnehmer haben kann,
        # kann derselbe Beratungskontakt fr verschiedene Klienten aufgerufen werden.
        fall_id = self.form.get('fallid')
        fall = Fall(fall_id)
        if not fall_id:
            self.last_error_message = "Keine ID fr den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        beratungskontakte = fall['beratungskontakte']
        if bkont not in beratungskontakte:
            raise EE("Beratungskontakt gehrt nicht zum Fall.")
        is_erstgespraech = bkont['erstgespraech'] == cn('ja_nein','ja')
        prev_ = next_ = count = None
        for i, b in enumerate(beratungskontakte):
            if b == bkont:
                count = i + 1
                prev_ = None if i-1 < 0 else beratungskontakte[i-1]
                next_ = None if i+1 >= len(beratungskontakte) else beratungskontakte[i+1]
                
        titelstr = ("%s. Beratungskontakt mit %s " % (count, fall['name']) +
                    "vom %(day)d.%(month)d.%(year)d " % bkont.getDate('k') +
                    '(Fallnummer: %s)' % fall['fn'])
        ueberschrift = h.TableDataTable(
            daten=[[h.String(string=titelstr,
                                   class_='legendtext'),
                    ],
                   # [h.Dummy()],
                   ],
            )

        beratungskontakt = self.get_beratungskontakte(
            BeratungskontaktList([bkont]),
            cls=h.TableDataTable,
            header_class='tabledatabold',
            )
        daten = [[h.Dummy(n_col=2)],
                 [h.String(string="Beratungsdokumentation",
                           class_='legendtext'),
                  h.String(string="(Erstgesprch)" if  is_erstgespraech else '',
                           class_='legendtext'),],
                 [h.Dummy(n_col=2)],
                 [h.String(string="Erstellt am:",
                           class_='tabledatabold'),
                          h.String(string=bkont.getDate('d') if bkont['dy'] else '')],
                 [h.String(string="Gendert am:",
                           class_='tabledatabold'),
                  h.String(string=bkont.getDate('dlc'))]
                 if bkont.getDate('d') < bkont.getDate('dlc') else None,
                 [h.String(string="Aufklrung ber Arbeitsweise:",
                           class_='tabledatabold'),
                  h.String(string=bkont['aufklaerung__name'])] if is_erstgespraech else None,
                 ]
        termin_id = bkont.get('termin')
        for name, _, _, label_kurz, _, _, _, _ in self.bkontdok_feldinfo:
            daten.append([h.String(string=label_kurz,
                           class_='tabledatabold'),
                  h.StringPre(string=bkont[name],
                              split_long_lines=True)])
        daten.append([h.String(string="Fallbesprechung gem.  8a, SGB VIII notwendig?",
                               class_='tabledatabold'),
                      h.String(string=bkont['fall8a__name'])])
        daten.append([h.String(string="Terminabsprache:",
                               class_='tabledatabold'),
                      h.String(string='' if termin_id is None else 
                               [x for x in self.termin_elements if x['id'] == termin_id][0]['name'])])
        beratungsdokumentation = h.TableDataTable(
            daten=daten,
            )
        zurueck = h.TableDataTable(
            daten = [[h.Link(class_='legendtext', string='<<< zurck', url="viewbkont?bkontid=%s&fallid=%s" % (prev_['id'], fall_id)) if prev_ else h.String(class_='legendtext', string='---'),
                      ]]
            )
        vor = h.TableDataTable(
            daten = [[h.Link(class_='legendtext_right', string='vor &gt;&gt;&gt;', url="viewbkont?bkontid=%s&fallid=%s" % (next_['id'], fall_id)) if next_ else h.String(class_='legendtext_right', string='---'),
                      ]]
            )
        res = h.Page(
            title='Beratungskontakt %(fn)s ' % fall + \
                'vom %(day)d.%(month)d.%(year)d' % bkont.getDate('k'),
            ueberschrift=ueberschrift,
            rows=(ueberschrift,
                  h.Pair(left=zurueck, right=vor),
                  beratungskontakt,
                  beratungsdokumentation,
                  h.Schliessen(),
                  ),
            )
        return res.display()



class bkontpdf(Request.Request):
    """Alle Beratungskontakte als PDF ausgeben"""
    permissions = Request.KLKARTE_PERM
    def processForm(self, REQUEST, RESPONSE):
        mitarbeiterliste = self.getMitarbeiterliste()
        user = self.user
        akte_id = self.form.get('akid')
        if not akte_id:
            self.last_error_message = "Keine ID fuer die Akte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        akte = Akte(akte_id)
        doc = AkteBeratungskontaktPDF(akte, self.mitarbeiter)
        #doc = BeratungskontaktPDF(akte)
        datei = doc.create_pdf()
        RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        RESPONSE.setHeader('filename', 'beratungskontakte_%d.pdf' % (akte['id'],))
        RESPONSE.setBody(datei)    

from reportlab.lib.units import inch, cm, mm
from reportlab.lib import colors
from reportlab.platypus import Paragraph, PageBreak, Spacer, Flowable
from reportlab.platypus.tables import Table, TableStyle
from ebkus.app.pdfgen import CustomPDF

class AkteBeratungskontaktPDF(CustomPDF):
    # Margins
    lm = 20*mm
    rm = 10*mm
    tm = 15*mm
    bm = 15*mm
    # Show boundary around frame
    sb = False
    topline = False # explizite Linie auf dem topMargin
    bottomline = False # explizite Linie auf dem bottomMargin
    header_on_first_page = False
    header_left = ''
    header_center = ''
    header_right = ''
    def __init__(self, akte, mitarbeiter, standalone=True):
        CustomPDF.__init__(self)
        self.akte = akte
        self.faelle = akte['faelle']
        self.faelle.sort('bgy', 'bgm', 'bgd')
        self.mitarbeiter = mitarbeiter
        self.standalone = standalone
    def add_content(self):
        akte = self.akte
        if self.standalone:
            titel = self.formatzelle(("Beratungskontakte fr {name} geb. {gb}",), ebkus_objekt=akte)[0]
            self.addp(titel, 'Heading2')
        last = len(self.faelle) - 1
        for i, fall in enumerate(self.faelle):
            fall_dok = BeratungskontaktPDF(fall, self.mitarbeiter)
            fall_dok.add_content()
            self.flowables += fall_dok.flowables
            if i < last:
                self.addf(PageBreak())

class BeratungskontaktPDF(CustomPDF):
    def __init__(self, fall, mitarbeiter):
        CustomPDF.__init__(self)
        self.fall = fall
        self.mitarbeiter = mitarbeiter
    def add_content(self):
        self.beratungskontakte = self.fall['beratungskontakte']
        # kein inhalt, keine doku
        doku_vorhanden = bool([bk['inhalt'] for bk in self.beratungskontakte if bk['inhalt']])
        if config.BERATUNGSKONTAKT_DOKUMENTATION and doku_vorhanden:
            #print 'BERATUNGSKONTAKT_DOKUMENTATION'
            self.add_content_fuer_fall_mit_beratungsdoku()
        else:
            #print 'BERATUNGSKONTAKT'
            self.add_content_fall()
    def add_content_fall(self):
        fall = self.fall
        titel = self.formatzelle(("Fallnummer {fn}",), ebkus_objekt=fall)[0]
        self.addp(titel, 'MyHeading4')
        daten = self.get_metadaten(self.beratungskontakte)
        self.addt(daten, 
                  #colWidths=(None,None,2*cm,2*cm)
                  )
    def add_content_fuer_fall_mit_beratungsdoku(self):
        fall = self.fall
        titel = self.formatzelle(("Fallnummer {fn}",), ebkus_objekt=fall)[0]
        self.addp(titel, 'Heading4')
        last = len(self.beratungskontakte)
        for laufende, bkont in enumerate(self.beratungskontakte, 1):
            self.add_content_fuer_eine_beratungsdoku(fall, bkont, laufende)

    def add_content_fuer_eine_beratungsdoku(self, fall, bkont, laufende):
        titel = u"{laufende}. Beratungskontakt am {datum}".format(
            laufende=laufende, 
            datum=bkont['k'])
        metadaten = self.get_metadaten([bkont])
        beratungsdokudaten = self.get_beratungsdokudaten(fall, bkont)
        self.addp(titel, 'MyHeading4')
        self.addt(metadaten)
        self.addpropt1(beratungsdokudaten, colWidths=(63*mm,None))

    def get_beratungsdoku_template(self, erstgespraech=False):
        templates = [
            #('Dokumentation {erstgespr}' if erstgespraech else '', ''),
            ('Dokumentation', '{erstgespr}' if erstgespraech else ''),
            ('Erstellt am:', '{d}'),
            ('Gendert am:', '{dlc}'),
            ]
        if erstgespraech:
            templates.append(("Aufklrung ber Arbeitsweise:", '{aufklaerung__name}'))
        for (name, label_erst, label_folge, label_kurz, 
             cols, rows, in_erst, in_folge) in _bkont.bkontdok_feldinfo:
            if ((in_erst and erstgespraech) or 
                (in_folge and not erstgespraech)):
                templates.append((label_kurz, "{%s}" %(name,)))
        templates.append(("Fallbesprechung gem.  8a, SGB VIII notwendig?",
                          '{fall8a__name}'))
        templates.append(("Terminabsprache:", '{terminabsprache}'))
        #print templates
        return templates
                         
    def get_beratungsdokudaten(self, fall, bkont):
        is_erstgespraech = bkont['erstgespraech'] == cn('ja_nein','ja')
        #titelstr = ("%s. Beratungskontakt mit %s " % (count, fall['name']) +
        daten = []
        termin_id = bkont.get('termin')
        prio = dict(erstgespr='(Erstgesprch)' if is_erstgespraech else '',
                    terminabsprache=('' if termin_id is None else 
                                     [x for x in _bkont.termin_elements 
                                      if x['id'] == termin_id][0]['name']))
        # Zeilen umbrechen fr alle Textfelder in bkontdok_feldinfo
        for (name, label_erst, label_folge, label_kurz, 
             cols, rows, in_erst, in_folge) in _bkont.bkontdok_feldinfo:
            s = bkont[name]
            # print s
            # print 'Newlines:', s.count('\r\n')
            if s:
                s = s.replace('\r\n', '<br/>')
                #t = '' if s is None else s
                prio[name] = s
            #prio[name] = h.break_lines(t, 65)
        for t in self.get_beratungsdoku_template(erstgespraech=is_erstgespraech):
            daten += [self.formatzelle(t, ebkus_objekt=bkont, prio=prio)]
        return daten
    def get_metadaten(self, beratungskontakte):
        """Tabelle mit Header und eine Zeile fr jeden Beratungskontakt."""
        BS = config.BERATUNGSKONTAKTE_BS
        BD = config.BERATUNGSKONTAKT_DOKUMENTATION
        if BS:
            art_feld = 'art_bs'
            headers=('Datum', 'Mitarbeiter', 'Klienten', 'Art',
                       'Teilnehmer', 'Dauer in Minuten', 'BS', 'Notiz')
        else:
            art_feld = 'art'
            headers=('Datum', 'Mitarbeiter', 'Klienten', 'Art',
                     'Dauer', 'BS', 'Notiz')
        daten = [headers]
        for  b in beratungskontakte:
            prio = dict(
                datumzeit = "{} {}".format(b.getDate('k'), b.getTime('k')),
                mitarbeiter = ', '.join([m['na']
                                         for m in b['mitarbeiter']]),
                klienten = ', '.join([f['akte__na']
                                      for f in b['faelle']]),
                art = b['%s__name'% art_feld],
                teilnehmer = (', '.join([Code(i)['name']
                                        for i in b['teilnehmer_bs'].split()])
                              if BS else None),
                dauer = (h.String(string="%(dauer)s / %(brutto)s" % b,
                                 tip='Netto/Brutto')
                         if BS else
                         h.String(string=bcode('fskd', b['dauer'])['name'])),
                jghkontakte = b['jghkontakte'],
                notiz = b['no'])
            if BS:
                templates = ('{datumzeit}','{mitarbeiter}','{klienten}','{art}',
                             '{teilnehmer}','{dauer}','{jghkontakte}','{notiz}')
            else:
                templates = ('{datumzeit}','{mitarbeiter}','{klienten}','{art}',
                             '{dauer}','{jghkontakte}','{notiz}')
            #print prio
            daten += [self.formatzelle(templates, prio=prio)]
        return daten


