# coding: latin-1

"""Module fr den Im- und Export aus der Datenbank."""

import time, os
import re
from operator import itemgetter
from ebkus.utils import Bunch
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Code, JugendhilfestatistikList, today, cc, EE, ER, EW, Fall, \
    check_int_not_empty, check_str_not_empty, FallList
from ebkus.html.akte_share import akte_share
import ebkus.html.htmlgen as h
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.datenaustausch_templates import *
from ebkus.app.jghexport import jghexport as get_jghexport_datensaetze, \
    get_export_datei_name, get_export_datei_daten, \
    get_daten_fuer_uebersicht, get_daten_fuer_uebersicht_sql
from ebkus.db.sql import SQL, getDBHandle

EXPORT_DIR = os.path.join('daten', 'export')

def align_right(daten):
    "Man msste htmlgen erweiteren ..."
    if daten:
        for zeile in daten:
            if zeile:
                for zelle in zeile[1:]:
                    if zelle:
                        zelle.class_ = 'tabledata_right'

class jghexportform(Request.Request, akte_share):
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = self.form.get('jahr')
        if jahr and jahr.isdigit():
            jahr = int(jahr)
        else:
            # default: Berichtsjahr ist das Vorjahr
            jahr = today().year - 1 
        res = h.Page(
            title='Bundesjugendhilfestatistik exportieren fr das Berichtsjahr {}'.format(jahr),
            breadcrumbs = (('Administratorhauptmen', 'menu'),),
            rows=(self.get_hauptmenu(),
                  self.get_berichtsjahr_auswahl(jahr),
                  self.get_uebersicht(jahr),
                  self.get_export_liste_mit_button(jahr, 'abgeschlossen'),
                  self.get_export_liste_mit_button(jahr, 'andauernd'),
                  # self.get_addressen_ausserhalb_pruefen(),
                  ),
            )
        return res.display()
    # def get_berichtsjahr_auswahl(self, jahr):
    #     anzeige = h.FieldsetFormInputTable(
    #         legend='Berichtsjahr auswhlen',
    #         name='jghexportform',
    #         action='jghexportform',
    #         method='post',
    #         hidden = (),
    #         daten=[[h.SelectItem(label='Berichtsjahr',
    #                              name='jahr',
    #                              class_='listbox60',
    #                              options=self.for_jahre(sel=jahr),
    #                              ),
    #                 h.Button(value="Anzeigen",
    #                           name='submitop',
    #                           tip="Exportformular fr gewhltes Jahr anzeigen",
    #                           type='submit',
    #                           ),
    #                 ],
    #                ],
    #         )
    #     return anzeige

    def get_berichtsjahr_auswahl(self, berichtsjahr):
        res = getDBHandle().query("select min(bgy) from fall")
        minjahr, maxjahr = res[0][0], today().year
        url_name_pairs = [('jghexportform?jahr={}'.format(jahr), "{}".format(jahr)) 
                          for jahr in range(maxjahr, minjahr-1, -1)]
        anzeige = h.FieldsetInputTable(
            legend='Berichtsjahr auswhlen',
            daten=[[h.DummyItem(label_width="80%"),
                    h.DropdownMenu(name="Berichtsjahr", url_name_pairs=url_name_pairs,
                                   tip="Sinnvollerweise macht man den Datenexport fr die "
                                   "Bundesjugendhilfestatistik am Beginn des Folgejahres, "
                                   "wenn die andauernden Flle zum Ende des Berichtsjahrs noch nicht "
                                   "abgeschlossen sind. Fr abgeschlossene Flle kann man aber auch "
                                   "fr frhere Jahre den Export wiederholen, da EBKuS die Statistik "
                                   "fr abgeschlossene Flle nicht berschreibt."), 
                    h.String(string=berichtsjahr, class_='largetextbold'),
                    h.DummyItem(label_width="80%")]],
            )
        return anzeige

    def get_uebersicht(self, berichtsjahr):
        """
        Im Berichtsjahr 2018 gab es 2772 Flle, davon 1150 bernahmen aus 2017 und 1650 
        Neuaufnahmen.

                                       bernahmen  Neuaufnahmen  Insgesamt ohne Fall
        Flle im Berichtsjahr:

                                               mit zdA  ohne zdA  exportieren  zu klren  insgesamt ohne Fall (nur falls vorhanden)
                                                                   
        Bundesstatistik mit Eintrag 
        Ende der Hilfe/Beratung 2018              234     123         236         123         432      2        

        Bundesstatistik mit Eintrag
        Hilfe/Beratung dauert an fr 2018           0     344         345           0         345      1
        
        Keine Bundesstatistik vorhanden             0      23           0          23          23

        Sonstige                                  123      23           0          23         146

        Summe                                    


                           Button        Problematische Flle anzeigen

                                                                                        
                           Fallnummer    Beginn (Anmeldung) letzte Zustndigkeit    zdA   Bundesstatistik  fortdauernd
                                                                                          Berichtsjahr
                           123-2016HAU     02.08.2016              Tester          offen         2017           ja


        Abgeschlossene Flle im Berichtsjahr:

                                                      
        Fordauernde Flle am Ende des Berichtsjahrs: 

        """
        jahr = berichtsjahr
        #d = get_daten_fuer_uebersicht(berichtsjahr)
        d = get_daten_fuer_uebersicht_sql(berichtsjahr)
        tip_ohne_fall = "Statistik vorhanden, Falldaten aber gelscht, z.B. auf Verlangen des Klienten oder nach Ablauf der Lschfrist"
        tip_uebernahme = "siehe 'Neumeldungen und Abschlsse', Zeile 1, Spalte 2"
        tip_neuaufnahme = "siehe 'Neumeldungen und Abschlsse', letzte Zeile, Spalte 3"
        tip_summe = "Anzahl der Flle, die irgenwann im Berichtsjahr mal aktiv waren"
        headers = (
            ('bernahmen aus dem Vorjahr', tip_uebernahme),
            ('Neuaufnahmen', tip_neuaufnahme),
            ('ohne Fall', tip_ohne_fall),
            ('Summe', tip_summe)
            )
        #if d.sum_BS_ohne_fall > 0:
        # if True:
        #     headers.append(('ohne Fall', "verbliebene Statistik bei gelschten Falldaten"))

        daten = [[h.String(string='Flle im Berichtsjahr:',
                           class_='normaltext'),
                  h.String(string=d.sum_uebernahmen, tip=tip_uebernahme),
                  h.String(string=d.sum_neuaufnahmen, tip=tip_neuaufnahme),
                  h.String(string=d.sum_BS_ohne_fall, tip=tip_ohne_fall),
                  h.String(string=d.sum_faelle + d.sum_BS_ohne_fall, tip=tip_summe),
                  ],
               ]
        align_right(daten)
        faelle = h.TableDataTable(
            headers=headers,
            noheaders=1,
            daten=daten,
            )
        headers = (
            ('mit zdA', "abgeschlossene Flle, zdA erfolgt"),
            ('ohne zdA', "offene Flle, zdA noch nicht erfolgt"),
            ('zu exportieren', "Flle, fr die zum jetzigen Zeitpunkt Datenstze "
             "fr die Bundesjugendhilfstatistik erstellt werden knnen"),
            ('zu klren', "Flle, fr die potentiell bei entsprechender Aktualisierung "
             "Datenstze fr die Bundesjugendhilfstatistik erstellt werden knnten"),
            ('ohne Fall', tip_ohne_fall),
            ('Summe', "Zeilensumme"),
            )
        daten=[[h.String(string='Bundesstatistik mit Eintrag\n'
                            'Ende der Hilfe/Beratung {}:'.format(jahr),
                            class_='normaltext'),
                h.String(string=d.sum_BS_Berichtsjahr_beendet_mit_zda),
                h.String(string=d.sum_BS_Berichtsjahr_beendet -
                         d.sum_BS_Berichtsjahr_beendet_mit_zda,
                         tip="Diese Flle knnen erst nach erfolgtem Fallabschluss (zdA) "
                         "fr die Bundesjugendhilfstatistik bernommen werden. "),
                h.String(string=d.sum_BS_Berichtsjahr_beendet_mit_zda +
                         d.sum_BS_ohne_fall_beendet),
                h.String(string=d.sum_BS_Berichtsjahr_beendet -
                         d.sum_BS_Berichtsjahr_beendet_mit_zda,
                         tip="Diese Flle knnen erst nach erfolgtem Fallabschluss (zdA) "
                         "fr die Bundesjugendhilfstatistik bernommen werden. "),
                h.String(string=d.sum_BS_ohne_fall_beendet),
                h.String(string=d.sum_BS_Berichtsjahr_beendet + 
                         d.sum_BS_ohne_fall_beendet),
                ],
               [h.String(string='Bundesstatistik mit Eintrag\n'
                            'Hilfe/Beratung dauert an fr {}:'.format(jahr),
                            class_='normaltext'),
                h.String(string='0'),
                h.String(string=d.sum_BS_Berichtsjahr_andauernd),
                h.String(string=d.sum_BS_Berichtsjahr_andauernd +
                         d.sum_BS_ohne_fall_andauernd),
                h.String(string='0'),
                h.String(string=d.sum_BS_ohne_fall_andauernd), 
                h.String(string=d.sum_BS_Berichtsjahr_andauernd + 
                         d.sum_BS_ohne_fall_andauernd),
                ],
               [h.String(string='Keine Bundesstatistik vorhanden:',
                         class_='normaltext',
                         tip="Logisch mglich sind hier nur Flle ohne zdA, "
                         "da EBKuS das Vorhandensein der Statistik bei Fallabschluss prft"),
                h.String(string='0', 
                         tip="Logisch unmglich, da EBKuS den Fallabschluss ohne Statistik nicht zulsst"),
                h.String(string=d.sum_BS_fehlt),
                h.String(string='0'),
                h.String(string=d.sum_BS_fehlt,
                         tip="Offene Flle, fr die die Bundesstatistik noch nachgetragen werden kann."),
                h.String(string='0'),
                h.String(string=d.sum_BS_fehlt),
                ],
               [h.String(string='Sonstige:',
                         class_='normaltext',
                         tip="Fortdauernde Flle, die inzwischen aber abgeschlossen wurden, "
                         "und offene Flle, fr die die Bundesstatistik noch aktualisiert werden kann."),
                h.String(string=d.sum_BS_sonstige_zda,
                         tip="Diese Flle waren am Ende des Berichtsjahr fortdauernd, inzwischen ist "
                         "aber zdA erfolgt und die Bundesstatistik damit berschrieben. "
                         "Datenstze fr fortdauernde Flle mssen rechtzeitig exportiert werden!"
                         ""),
                h.String(string=d.sum_BS_sonstige_ohne_zda,
                         tip="Fr diese offenen Flle kann die Bundesstatistik noch aktualisiert werden"),
                h.String(string='0'),
                h.String(string=d.sum_BS_sonstige_ohne_zda,
                         tip="Fr diese offenen Flle kann die Bundesstatistik noch aktualisiert werden"),
                h.String(string='0'),
                h.String(string=d.sum_BS_sonstige),
                ],
               ]
        summen_zeile = [h.String(string='Summe:',
                                 tip='Spaltensumme',
                                 class_='normaltext')]
        for spalten_index in range(1,7):
            summe = 0
            for zeile in daten:
                summe += int(zeile[spalten_index].string)
            summen_zeile.append(h.String(string=str(summe)))
        daten.append(summen_zeile)
        button_zu_klaeren = h.Button(value='Zu klrende Flle anzeigen',
                                     class_='buttonhuge',
                                     onClick="go_to_url('zuerfaelle?jahr={}')".format(jahr),
                                     tip="Die Flle aus der Spalte 'zu klren' anzeigen. "
                                     "Sinnvoll ist dies hauptschlich am Beginn des neuen "
                                     "Jahres, um den Datenexport fr das vergangene Berichtsjahr "
                                     "vorzubereiten.",
                                     )
        align_right(daten)
        statistik_uebersicht = h.TableDataTable(
            headers=headers,
            noheaders=1,
            daten=daten,
            )
        uebersicht = h.Fieldset(
            legend='bersicht fr das Berichtsjahr {}'.format(jahr),
            rows=(faelle,
                  statistik_uebersicht,
                  button_zu_klaeren,
            ),
            )
        return uebersicht

# h.SelectItem(label='Berichtsjahr',
#                                  name='jahr',
#                                  class_='listbox60',
#                                  options=self.for_jahre(sel=jahr),
#                                  ),
#                     h.Button(value="Exportieren",
#                               name='submitop',
#                               tip="Datei mit Datenstzen erstellen ",
#                               type='submit',
#                               ),
                            

    def get_export_liste_mit_button(self, jahr, welche):
        """
        Datentabelle mit Exportdateien zum runterladen

        Dateiname Log-Datei Exportdatum Anzahl der Datenstze

        Immer nur die Dateien fr das Berichtsjahr.

            name='jghexport',
            action='jghexport',
            method='post',
            hidden = (('welche', 'abgeschlossen'),),
        
        """
        dateien = get_export_datei_daten(jahr, welche)
        headers=('Dateiname', 'Log-Datei', 'Exportdatum', 'Anzahl Datenstze')
        daten=[[h.Link(string=d.name, url="jghexport?jahr={}&file=download&filename={}".format(jahr, d.name)),
                h.Link(string=d.name_log, url="jghexport?jahr={}&file=download&filename={}".format(jahr, d.name_log)),
                h.String(string=d.datum),
                h.String(string=d.count),
                ] for d in dateien
               ]
        export_dateien = h.TableDataTable(
            headers=headers,
            daten=daten,
            )
        button_exportieren = h.Button(value="Exportieren",
                                      name='submitop',
                                      tip="Datei mit Datenstzen erstellen ",
                                      type='submit',
                                      )

        export_liste = h.FieldsetForm(
            legend='{} Flle fr das Berichtsjahr {}'.format(
                'Andauernde' if welche == 'andauernd' else 'Abgeschlossene', jahr),
            action='jghexport',
            method='post',
            name='jghexport',
            hidden=(
                ('file', 'export'),
                ('welche', welche),
                ('jahr', jahr),
                ),
            rows=(
                export_dateien,
                button_exportieren,
                ),
            )
        return export_liste
    def get_fortdauernde_faelle(self, jahr):
        anzeige = h.FieldsetFormInputTable(
            legend='Fortdauernde Flle fr das Berichtsjahr {}'.format(jahr),
            name='jghexport',
            action='jghexport',
            method='post',
            hidden = (('welche', 'fortdauernd'),),
            daten=[[h.SelectItem(label='Berichtsjahr',
                                 name='jahr',
                                 class_='listbox60',
                                 options=self.for_jahre(sel=jahr),
                                 ),
                    h.Button(value="Exportieren",
                              name='submitop',
                              tip="Datei mit Datenstzen erstellen ",
                              type='submit',
                              ),
                    ],
                   ],
            )
        return anzeige
        

            
class zuerfaelle(Request.Request, akte_share):
    """Flle anzeigen, die potentiell fr den Datenexport in Frage kommen nach
    entsprechender Aktualisierung.
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Jahr fehlt")
        # welche sind zu klren? nur offene Flle
        offene_faelle = FallList(where = 'bgy <= {jahr} and zday = 0'.format(jahr=jahr), 
                                 cache_related=True)
        # ganz ohne statistik -> knnten fortdauernd sein
        faelle_ohne_statistik = [f for f in offene_faelle if f['jgh'] is None]
        # mit statistik ende eingetragen, aber kein zda
        faelle_ohne_zda = [f for f in offene_faelle 
                           if f['jgh'] and f['jgh__ey']==jahr]
        # kein zda, statistik vorhanden aber falsches jahr
        faelle_mit_statistik_sonstige = (
            [f for f in offene_faelle if f['jgh'] and 
             ((f['jgh__ey']!=None and f['jgh__ey']!=jahr) or 
              (f['jgh__ey']==None and f['jgh__jahr']!=jahr))])
        

        inhalt = h.FieldsetDataTable(
            legend='Inhalt',
            daten=[[h.String(string="Zu klrende Flle fr den Bundesstatistikexport fr " 
                             "das Berichtsjahr {}".format(jahr),
                             class_='largetextbold')],
                   [h.Link(string='Offene Flle ohne Bundesstatistik ({})'.
                           format(len(faelle_ohne_statistik)),
                           url='#ohne_statistik')],
                   [h.Link(string='Ende eingetragen, aber kein zdA ({})'.
                           format(len(faelle_ohne_zda)),
                           url='#ohne_zda')],
                   [h.Link(string='Berichtsjahr oder Endejahr falsch ({})'.
                           format(len(faelle_mit_statistik_sonstige)),
                           url='#sonstige')],
                   ],
            )

        # Felder:
        # fallnummer, beginn, letzte zustndigkeit
        
        headers=('Fallnummer', 'Beginn', 'zdA', 
                 ('Berichtsjahr', "Das in der Bundesstatistik eingetragene Jahr."),
                 ('Ende Monat/Jahr bzw. fortdauernd', "'Ende der Hilfe/Beratung' ist in "
                  "die Bundesstatistik eingetragen oder 'Hilfe/Beratung dauert an'."),
                 # ('Fortdauernd', "'Hilfe/Beratung dauert an' ist in "
                 #  "die Bundesstatistik eingetragen."),
                 ('Mitarbeiter', "Zuletzt zustndige(r) Mitarbeiter(in)"),
                 )
        def zeile(f):
            return [h.String(string=f['fn']),
                    h.Datum(date=f.getDate('bg')),
                    h.Datum(date=f.getDate('zda')),
                    h.String(string=f['jgh__jahr']),
                    h.String(string="{}/{}".
                             format(f['jgh__em'],f['jgh__ey']) 
                             if f['jgh__ey'] else 
                             ('fortdauernd' if f['jgh__hda__code']=='1'
                             else '')),
                    #h.String(string=f['jgh__hda__name']),
                    h.String(string=f['zuletzt_zustaendig__mit__na']),
                    ]
        ohne_statistik = h.FieldsetDataTable(
            legend='Offene Flle ohne Bundesstatistik ({})'.format(len(faelle_ohne_statistik)),
            anchor='ohne_statistik',
            headers=headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_ohne_statistik],
            )
        kein_zda = h.FieldsetDataTable(
            legend='Ende eingetragen, aber kein zdA ({})'.format(len(faelle_ohne_zda)),
            anchor='ohne_zda',
            headers = headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_ohne_zda],
            )
        sonstige = h.FieldsetDataTable(
            legend='Berichtsjahr oder Endejahr falsch ({})'.format(len(faelle_mit_statistik_sonstige)),
            anchor='sonstige',
            headers = headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_mit_statistik_sonstige],
            )
        res = h.Page(
            title='Zu klrende Flle fr den Bundesstatistikexport {}'.format(jahr),
            breadcrumbs = (('Administratorhauptmen', 'menu', ),
                           ('Bundesjugendhilfstatistik exportieren', 
                            'jghexportform?jahr={}'.format(jahr)),
                           ),
            rows=(self.get_zurueck(),
                  inhalt,
                  ohne_statistik,
                  kein_zda,
                  sonstige,
                  self.get_zurueck(),
                ),
            )
        return res.display()

        
        


class jghexport(Request.Request):
    """Die action von post request
    file=download
    evt. file=delete
    file=export
    am Ende immer ein redirect auf jghexportform
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        #print 'JGHEXPORT:', self.form
        file = self.form.get('file')
        jahr = check_int_not_empty(self.form, 'jahr', 'Kein Berichtsjahr')
        if not file or file not in ('download', 'export'):
            raise EE('Parameter file fehlt oder falsch')
        if file == 'download':
            filename = check_str_not_empty(self.form, 'filename', 'Kein Dateiname')
            self.download(filename)
            return ''
        if file == 'export':
            welche = check_str_not_empty(self.form, 'welche', 'Kein welche')
            self.export(jahr, welche)
            RESPONSE.redirect('jghexportform?jahr={}'.format(jahr), status=303)
    def export(self, jahr, welche):
        assert welche in ('abgeschlossen','andauernd')
        andauernd = (welche == 'andauernd')
        if andauernd and jahr < 2007:
            raise EE('Bundesstatistik fr andauernde Flle erst ab 2007 mglich')
        daten_saetze, log_daten_saetze = get_jghexport_datensaetze(jahr, andauernd)
        if not daten_saetze:
            raise EE('Keine Bundesstatistik fr {}e Flle fr {} vorhanden'.format(
                    welche,
                    jahr))
        try:
            filename = get_export_datei_name(jahr, log=False, andauernd=andauernd) 
            log_filename = get_export_datei_name(jahr, log=True, andauernd=andauernd) 
            path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, filename)
            f = open(path, 'wb')
            f.write(daten_saetze)
            f.close()
            log_path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, log_filename)
            f_log = open(log_path, 'w')
            f_log.write(log_daten_saetze)
            f_log.close()
        except Exception, e:
            raise EE("Fehler beim Exportieren: %s" % str(e) ) 
    def download(self, filename):
        #print 'DOWNLOAD:'
        path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, filename)
        f = open(path, 'rb')
        content = f.read()
        f.close()
        self.RESPONSE.setHeader('content-type', 'text/plain')
        self.RESPONSE.setHeader('content-disposition', 'attachment; filename=%s' % filename)
        self.RESPONSE.setBody(content)
        
class jghexportuebersicht(Request.Request):
    """bersicht ber vorhandene Exportdateien."""
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        return 'Not yet implemented'

class formabfrjghexport(Request.Request):
    """Auswahlformular fr den Export der Jugendhilfestatistik."""
    
    permissions = Request.ADMIN_PERM
    
    def processForm(self, REQUEST, RESPONSE):
        bestaetigung = {'titel': "Bundesjugendhilfestatistik: Exportdatei erstellen",
                        'legende':'Bundesjugendhilfestatistik: Exportdatei erstellen',
                        'zeile':'Bitte das Exportjahr eingeben und mit Ok best&auml;tigen!',
                        'jahr' : '%s' % (today().year),
                        'dest_url':'menu'}
        return bestaetigung_t % bestaetigung
        
        
class jghexportfeedback(Request.Request):
    """Aufruf zum Export der Jugendhilfestatistik und Feedback."""
    permissions = Request.ADMIN_PERM
    def _keine_statistik_liste(self, jahr):
        """Liste aller Flle, die in jahr oder davor begonnen haben,
        fr die keine Jugendhilfestatistik existiert.

        Wir brauchen nur offene Flle zu betrachten, da alle geschlossenen
        Flle eine Statistik haben mssen.
        """
        # zustaendigkeit, mitarbeiter nur zum sortieren
        sql = """
SELECT fall.id
FROM fall JOIN zustaendigkeit JOIN mitarbeiter
LEFT JOIN jghstat07 ON fall.id=jghstat07.fall_id
LEFT JOIN jghstat on fall.id=jghstat.fall_id
WHERE jghstat07.fall_id IS NULL AND
      jghstat.fall_id IS NULL AND
      fall.bgy<=%d AND
      fall.zday=0 AND
      zustaendigkeit.fall_id=fall.id AND
      mitarbeiter.id=zustaendigkeit.mit_id
ORDER BY mitarbeiter.na, fall.bgy, fall.bgm""" % jahr
        q = SQL(sql)
        faelle_ids_ohne = q.execute()
        res = []
        res.append(head_normal_ohne_help_t % "Flle ohne Bundesjugendhilfestatistik")
        res.append(thkeinestatistik_t)
        for fall_id_t in faelle_ids_ohne:
            fall = Fall(fall_id_t[0])
            res.append(keinestatistikliste_t % fall)
        res.append(jghexportliste_ende_t)
        return ''.join(res)
        
    def processForm(self, REQUEST, RESPONSE):
        jahr = self.form.get('jahr')
        if jahr and jahr.isdigit():
            jahr = int(jahr)
        else:
            self.last_error_message = "Kein gltiges Jahr"
            return self.EBKuSError(REQUEST, RESPONSE)
        
        if  self.form.get('welche') == 'keine':
            return self._keine_statistik_liste(jahr)
        if  self.form.get('welche') == 'adressen':
            RESPONSE.redirect('jgh_wohnt_ausserhalb_check?jahr=%s' % jahr, status=303)
            return ''
        andauernd = self.form.get('welche') == 'andauernd'
        if andauernd and jahr < 2007:
            self.last_error_message = \
               'Bundesstatistik fr andauernde Flle erst ab 2007 mglich'
            return self.EBKuSError(REQUEST, RESPONSE)
        daten_saetze, log_daten_saetze = get_jghexport_datensaetze(jahr, andauernd)
        if not daten_saetze:
            self.last_error_message = 'Keine Bundesstatistik f&uuml;r %s vorhanden' % jahr
            return self.EBKuSError(REQUEST, RESPONSE)
        try:
            filename = get_export_datei_name(jahr, log=False, andauernd=andauernd) 
            log_filename = get_export_datei_name(jahr, log=True, andauernd=andauernd) 
            path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, filename)
            f = open(path, 'wb')
            f.write(daten_saetze)
            f.close()
            log_path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, log_filename)
            f_log = open(log_path, 'w')
            f_log.write(log_daten_saetze)
            f_log.close()
        except Exception, e:
            raise EE("Fehler beim Exportieren: %s" % str(e) ) 
            
        res = []
        res.append(head_normal_ohne_help_t %("Exportdatei f&uuml;r das Jahr %s" % jahr))

        ausgabe = {
            'jahr' : jahr,
            'jgh_filename' : filename,
            'jgh_url' : 'jghexportlist?file=%s' % filename,
            'jgh_log_filename' : log_filename,
            'jgh_log_url' : 'jghexportlist?file=%s' % log_filename,
            }
        res.append(jghexportfeedback_t % ausgabe)
        return ''.join(res)
        

class jghexportlist(Request.Request):
    """Listet die exportierten Jugendhilfestatistiken.

    Falls ein Parameter file vorhanden ist, wird die entsprechende
    Jugendhilfestatistikdatei direkt zurckgegeben, so dass der Anwender
    sie bei sich abspeichern kann.
    """
    permissions = Request.ADMIN_PERM
    

    def processForm(self, REQUEST, RESPONSE):
        import os
        #print REQUEST
        if self.form.has_key('file'):
            file = self.form.get('file')
            try:
                filename = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, file)
                f = open(filename)
                content = f.read()
                f.close()
                self.RESPONSE.setHeader('content-type', 'text/plain')
                self.RESPONSE.setHeader('content-disposition', 'attachment; filename=%s' % file)
                self.RESPONSE.setBody(content)
                #print RESPONSE
                return
            except Exception, e:
                raise EE("Fehler beim Download: %s" % str(e) ) 
        
        dateiliste = os.listdir(os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR) )
        abgeschlossen, andauernd = _ordne_dateinamen(dateiliste)
        
        res = []
        res.append(head_normal_ohne_help_t % "Bundesjugendhilfestatistik: Liste der Exportdateien")
        res.append(thjghexportliste_t)
        for d, l in abgeschlossen:
            res.append(jghexportliste_t % ('jghexportlist?file=%s' % d, d,
                                           'jghexportlist?file=%s' % l, l))
        res.append(jghexportliste_trenner_t)
        for d, l in andauernd:
            res.append(jghexportliste_t % ('jghexportlist?file=%s' % d, d,
                                           'jghexportlist?file=%s' % l, l))
        res.append(jghexportliste_ende_t)
        return ''.join(res)

def _ordne_dateinamen(liste):
    an = [n for n in liste if n.find('andauernd') >= 0 and n.startswith('jgh_')]
    daten = [n for n in an if n.find('log') == -1]
    log = [n for n in an if n.find('log') >= 0]
    daten.sort()
    log.sort()
    andauernd = zip(daten, log)

    ab = [n for n in liste if n.find('andauernd') == -1 and n.startswith('jgh_')]
    daten = [n for n in ab if n.find('log') == -1]
    log = [n for n in ab if n.find('log') >= 0]
    daten.sort()
    log.sort()
    abgeschlossen = zip(daten, log)
    return abgeschlossen, andauernd
        
