# coding: latin-1

import string
from ebkus.config import config
from ebkus.app import ebapi
from ebkus.app import Request
from ebkus.app.ebapi import Akte, Fall, Fachstatistik, FachstatistikList, \
     LeistungList,cc,today,calc_age, bcode, EE, Code
from ebkus.html.strkat import get_strasse

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share
from ebkus.html.fskonfig import fs_customize

class _fachstatistik(Request.Request, akte_share):
    # fs und options wird von customize_item gebraucht
    def _customize(self, fs, rows):
        """bertrgt die in fskonfig definierten Anpassungen der Fachstatistik
        auf den Aufbau des Formulars. Die eigentliche Arbeit findet in
        _customize_item statt.

        Das aus fskonfig importierte fs_customize-Objekt enthlt die fr die
        Anpassung ntigen Informationen.
        """
        self._fs = fs
        res = []
        for r in rows:
            rc = self._customize_fieldset(r)
            if rc:
                res.append(rc)
        return tuple(res)
    def _customize_fieldset(self, f):
        res = []
        self._legend = None # _customize item kann hier was reinschreiben
        if f:
            for zeile in f.daten:
                if zeile:
                    z = self._customize_zeile(zeile)
                    if z:
                        res.append(z)
        if res:
            f.daten = res
            if self._legend:
                f.legend = self._legend
            return f
        return None # es kann sein, dass ein fieldset wegfllt, wenn es keine gltigen Zeilen hat
    def _customize_zeile(self, zeile):
        res = []
        valid_item = False # mindestens 1 gltiges item pro Zeile, sonst verschwindet Zeile
        for item in zeile:
            i = self._customize_item(item)
            #print 'CUSTOMIZE ITEM: ', item.name, i
            if i and not isinstance(i, h.DummyItem):
                valid_item = True
                res.append(i)
            else:
                res.append(h.DummyItem())
        if valid_item:
            return res
        return None
    def _customize_item(self, item):
        # Readonly Items werden nicht aktiviert/deaktiviert, 
        # da item.name=='xxx' und nicht der richtige Name.
        if fs_customize.deaktiviert(item.name):
            return None
        if fs_customize.jokerfeld(item.name):
            feld = fs_customize.fd[item.name]
            kat_code = feld['kat_code']
            if kat_code:
                #multiple = feld['verwtyp'] == cc('verwtyp', 'm')
                multiple = fs_customize.multifeld(item.name) # nur wenn verwtyp=='m' und flag hat keine 2
                if fs_customize.jokerfeld_eigenstaendig(item.name):
                    self._legend = feld['name']
                    item.label = ''
                else:
                    item.label = feld['name']
                if multiple:
                    item.multiple = True
                    item.size = 8
                if self._fs: # fachstat Objekt vorhanden, wir sind in updfs
                    #print 'FS CUSTOMIZE', self._fs[item.name], type(self._fs[item.name])
                    item.options = self.for_kat(kat_code, self._fs[item.name])
                elif multiple: # initialisieren
                    item.options = self.for_kat(kat_code, None)
                else:
                    item.options = self.for_kat(kat_code, ' ')
        return item

    def _process(self,
                 title,
                 file,
                 fs,
                 fsjok=None,
                 pageonly=False
                 ):
        try:
            fall = Fall(fs['fall_id'])
            akte = fall['akte']
            fall_aktuell = fall['aktuell']
            anm = fall['anmeldung']
        except:
            fall = None
            akte = None
            anm = None
            fall_aktuell = False
        # regionaldaten
        rd = regional_daten(fs, akte, fall)
        # gs, fs, zm, eleistungen
        von_akte_readonly = False
        # anmprobleme,qualij,bkm,bkv
        von_akte_readonly_anmproblem_bei_anmeldung = False
        _gs = fs['gs']
        _zm = fs['zm']
        _fs = fs['fs']
        _eleistungen = fs['eleistungen']
        _anmprobleme = fs['anmprobleme']
        _qualij = fs['qualij'] or ' '
        _bkm = fs['bkm'] or ' '
        _bkv = fs['bkv'] or ' '
        if fall_aktuell: 
            # Betrifft nur fs, zm, eleistungen und 
            # konfigurationsabhngig auch anmprobleme, qualij, bkm, bkv
            # Weitere Items werden bei der Neuanlage der FS bernommen,
            # siehe fsneu
            if config.FACHSTATISTIK_AKTE_DISKREPANZ_NICHT_ZULASSEN:
                # Immer die 3 (bzw. 7) Werte direkt von der Akte, Fall bzw. Anmeldung setzen
                # Damit ist gewhrleistet, dass keine Diskrepanz entstehen kann,
                # wenn die Fachstatistik gespeichert wird.
                # Wenn die Items in der Akte bzw. Anmeldung gendert werden ohne
                # die FS neu abzuspeichern kann allerdings eine Diskrepanz
                # entstehen.
                # TODO evt. in ebupd immer auch in die FS bertragen.
                von_akte_readonly = True
                _gs = akte['gs']
                _fs = akte['fs']
                _eleistungen = ' '.join([
                        str(leist['le']) for leist in fall['leistungen']])
                if anm:
                    anm = anm[0]
                    _zm = anm['zm']
                    if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
                        #if anm['anmprobleme'] and
                        von_akte_readonly_anmproblem_bei_anmeldung = True
                        _anmprobleme = anm['anmprobleme'] or ''
                        _qualij = akte['qualij'] or ' '
                        _bkm = akte['bkm'] or ' '
                        _bkv = akte['bkv'] or ' '
        else:            
            # Wenn der Fall nicht aktuell ist, kann alles editiert werden, die Akte ist ja nicht editierbar.
            # Ob das sinnvoll ist, ist eine andere Frage. Aber bisher war es immer so. Da sollten auch die
            # die Sachen editierbar sein, die aus der Akte bernommen wurden
            # (aber *nicht* ort, plraum, plz Geschlecht gs; diese werden sofort aus der Akte gesetzt,
            # wenn die Fachstatistik angelegt wird und sind dann immer readonly.
            pass
        if fsjok == None:
            fsjok = fs
        abschluss_info = None
        if not fall_aktuell:
            # In die Falldaten ganz oben eine Information einfgen
            # wenn der Fall abgeschlossen ist.
            abschluss_info = [h.TextItem(label='',
                                         n_col=4,
                                         name='xxx',
                                         readonly=True,
                                         class_='textbox310',
                                         value=(("Der Fall wurde am %s abgeschlossen." % fall.getDate('zda')) if fall else 
                                                "Der Fall ist abgeschlossen.")),
                              h.DummyItem(n_col=2)]
        falldaten = h.FieldsetInputTable(
            legend='Falldaten',
            daten = [[h.TextItem(label='Fallnummer',
                                 name='xxx',
                                 value=fs['fall_fn'],
                                 readonly=True,
                               ),
                      h.TextItem(label='Mitarbeiter',
                                 name='xxx',
                                 value=fs['mit__ben'],
                                 readonly=True,
                               ),
                      h.TextItem(label='Jahr',
                                 name='jahr',
                                 value=fs['jahr'],
                                 class_='textboxmid',
                                 maxlength=4,
                                 readonly=False if fall else True,
                               ),
                      ],
                     abschluss_info],
            )
        regional_info = "Ort: %s" % rd['ort']
        for f in config.STRASSENSUCHE.split():
            if f != 'ort' and rd.get(f):
                regional_info += " %s: %s" % (f.capitalize(), rd[f])
        tip_ro1 = (von_akte_readonly and 
                   "Von der Klientenkarte bernommen" or '')
        tip_ro2 = (von_akte_readonly_anmproblem_bei_anmeldung and 
                   "Von der Klientenkarte bernommen" or '')
        #print 'REGIONALDATEN', rd
        angabenklient = h.FieldsetInputTable(
            legend="Angaben zum Klienten und dessen Angehrige",
            daten = [[h.TextItem(label='Planungsraum',
                                 name='xxx',
                                 value=rd['plraum'],
                                 readonly=True,
                                 ),
                      h.TextItem(label='PLZ',
                                 name='xxx',
                                 value=rd['plz'],
                                 readonly=True,
                                 tip=regional_info,
                                 ),
                      ],
                     # [h.TextItem(label='Geschlecht',
                     #             name='xxx',
                     #             value=fs['gs__name'],
                     #             class_='textbox13',
                     #             readonly=True,
                     #             ),
                     [h.SelectItem(label='Geschlecht',
                                 name='gs',
                                 #value=fs['gs__name'],
                                 #class_='textbox13',
                                 readonly=von_akte_readonly,
                                 options=self.for_kat('gs', sel=_gs,),
                                 #tip=gs_tip,
                                 ),
                      h.SelectItem(label='Alter Kind/Jugendliche(r)',
                                   name='ag',
                                   options=self.for_kat('fsag', fs['ag']),
                                   aktiviert=True,
                                   tip='Alter bei Fallbeginn',
                                    ),
                      ],
                     [h.SelectItem(label='Lebensmittelpunkt Kind/Jugendliche(r)',
                                   readonly=von_akte_readonly,
                                   name='fs',
                                   options=self.for_kat('fsfs', _fs),
                                   tip=tip_ro1,
                                   ),
                      h.SelectItem(label='Empfohlen von',
                                   name='zm',
                                   readonly=von_akte_readonly,
                                   options=self.for_kat('fszm', _zm),
                                   tip=tip_ro1,
                                   ),
                      ],
                     [h.SelectItem(label='Beschftigung Jugendliche(r)',
                                   readonly=von_akte_readonly_anmproblem_bei_anmeldung,
                                   name='qualij',
                                   options=self.for_kat('fsqualij', _qualij),
                                   tip=tip_ro2,
                                   ),
                      h.DummyItem(),
                      ],
                     [h.SelectItem(label='Qualifikation Mutter',
                                   name='qualikm',
                                   options=self.for_kat('fsquali', fs['qualikm']),
                                   ),
                      h.SelectItem(label='Qualifikation Vater',
                                   name='qualikv',
                                   options=self.for_kat('fsquali', fs['qualikv']),
                                   ),
                      ],
                     [h.SelectItem(label='Beschftigungsverhltnis Mutter',
                                   readonly=von_akte_readonly_anmproblem_bei_anmeldung,
                                   name='bkm',
                                   options=self.for_kat('fsbe', _bkm),
                                   tip=tip_ro2,
                                   ),
                      h.SelectItem(label='Beschftigungsverhltnis Vater',
                                   readonly=von_akte_readonly_anmproblem_bei_anmeldung,
                                   name='bkv',
                                   options=self.for_kat('fsbe', _bkv),
                                   tip=tip_ro2,
                                   ),
                      ],
                     [h.SelectItem(label='Herkunftsland Mutter',
                                   name='hkm',
                                   options=self.for_kat('fshe', fs['hkm']),
                                   ),
                      h.SelectItem(label='Herkunftsland Vater',
                                   name='hkv',
                                   options=self.for_kat('fshe', fs['hkv']),
                                   ),
                      ],
                     [h.SelectItem(label='Alter Mutter',
                                   name='agkm',
                                   options=self.for_kat('fsagel', fs['agkm']),
                                   ),
                      h.SelectItem(label='Alter Vater',
                                   name='agkv',
                                   options=self.for_kat('fsagel', fs['agkv']),
                                   ),
                      ],
                     [h.SelectItem(label='Joker 1',
                                   name='joka1',
                                   options=self.for_kat('fsjoka1', fsjok['joka1']),
                                   ),
                      h.SelectItem(label='Joker 2',
                                   name='joka2',
                                   options=self.for_kat('fsjoka2', fsjok['joka2']),
                                   ),
                      ],
                     [h.SelectItem(label='Joker 3',
                                   name='joka3',
                                   options=self.for_kat('fsjoka3', fsjok['joka3']),
                                   ),
                      h.SelectItem(label='Joker 4',
                                   name='joka4',
                                   options=self.for_kat('fsjoka4', fsjok['joka4']),
                                   ),
                      ],
                     ]
            )
        label_width = "20%" # richtet die Select-Elemente Fieldset-bergreifend aus
        ba1 = h.FieldsetInputTable(
            legend='Problem 1 bei der Anmeldung',
            daten=[[h.SelectItem(label='',
                                 name='ba1',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsba', fs['ba1']),
                                 )
                    ]],
            )
        ba2 = h.FieldsetInputTable(
            legend='Problem 2 bei der Anmeldung',
            daten=[[h.SelectItem(label='',
                                 name='ba2',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsba', fs['ba2']),
                                 )
                    ]],
            )
        pbk = h.FieldsetInputTable(
            legend='Hauptproblematik Kind/Jugendliche(r)',
            daten=[[h.SelectItem(label='',
                                 name='pbk',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fspbk', fs['pbk']),
                                 )
                    ],
                   [h.TextItem(label='Sonstige',
                               name='no2',
                               value=fs['no2'],
                               class_="textbox310",
                               ),
                    ]
                   ],
            )
        pbe = h.FieldsetInputTable(
            legend='Hauptproblematik der Eltern',
            daten=[[h.SelectItem(label='',
                                 name='pbe',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fspbe', fs['pbe']),
                                 )
                    ],
                   [h.TextItem(label='Sonstige',
                               name='no3',
                               value=fs['no3'],
                               class_="textbox310",
                                  ),
                    ]
                   ],
            )
        if von_akte_readonly_anmproblem_bei_anmeldung:
            cell = []
            for name in [Code(int(i))['name'] for i in _anmprobleme.split()]:
                cell.append(name)
            daten = [[h.String(tip=tip_ro2, string='<br/>'.join(cell))]]
        else:
            daten = [[h.SelectItem(label='',
                                   name='anmprobleme',
                                   options=self.for_kat('fsba', _anmprobleme),
                                   multiple=True,
                                   class_='listbox310',
                                   label_width=label_width,
                                   size=8,
                                   tip=tip_ro2,
                                   ),
                    ]]
        anmprobleme = h.FieldsetInputTable(
            legend='Problem(e) bei der Anmeldung',
            daten=daten,
            )
        kindprobleme = h.FieldsetInputTable(
            legend='Problemspektrum Kind/Jugendliche(r)',
            daten=[[h.SelectItem(label='',
                                 name='kindprobleme',
                                 options=self.for_kat('fspbk', fs['kindprobleme']),
                                 multiple=True,
                                 class_='listbox310',
                                 label_width=label_width,
                                 size=8,
                                 ),
                    ],
                   ],
            )
        elternprobleme = h.FieldsetInputTable(
            legend='Problemspektrum Eltern',
            daten=[[h.SelectItem(label='',
                                 name='elternprobleme',
                                 options=self.for_kat('fspbe', fs['elternprobleme']),
                                 multiple=True,
                                 class_='listbox310',
                                 label_width=label_width,
                                 size=8,
                                 ),
                    ],
                   ],
            )

        jokf5 = h.FieldsetInputTable(
            legend='Joker 5',
            daten=[[h.SelectItem(label='',
                                 name='jokf5',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsjokf5', fsjok['jokf5']),
                                 )
                    ]],
            )
        jokf6 = h.FieldsetInputTable(
            legend='Joker 6',
            daten=[[h.SelectItem(label='',
                                 name='jokf6',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsjokf6', fsjok['jokf6']),
                                 )
                    ]],
            )
        jokf7 = h.FieldsetInputTable(
            legend='Joker 7',
            daten=[[h.SelectItem(label='',
                                 name='jokf7',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsjokf7', fsjok['jokf7']),
                                 )
                    ]],
            )
        jokf8 = h.FieldsetInputTable(
            legend='Joker 8',
            daten=[[h.SelectItem(label='',
                                 name='jokf8',
                                 class_='listbox310',
                                 label_width=label_width,
                                 options=self.for_kat('fsjokf8', fsjok['jokf8']),
                                 )
                    ]],
            )
        if von_akte_readonly:
            cell = []
            for name in [Code(int(i))['name'] for i in _eleistungen.split()]:
                cell.append(name)
            daten = [[h.String(tip=tip_ro1, string='<br/>'.join(cell))]]
        else:
            daten = [[h.SelectItem(label='',
                                   readonly=von_akte_readonly,
                                   name='eleistungen',
                                   class_='listbox310',
                                   label_width=label_width,
                                 multiple=True,
                                   tip=tip_ro1,
                                   options=self.for_kat('fsle', _eleistungen),
                                   size=8,
                                   ),
                    ]]
            
        eleistungen = h.FieldsetInputTable(
            legend='Erbrachte Leistungen',
            daten=daten,
            )
        #### anfang get_termine_daten
        def get_termine_daten():
            label = ('KiMu', 'KiVa', 'Kind', 'Paar', 'Familie',
                     'Soz.', 'Lehrer', 'Erz.', 'Hilfebespr.', 'Sonst.', 'Summe')
            namen = ('kkm', 'kkv', 'kki', 'kpa', 'kfa',
                     'ksoz', 'kleh', 'kerz', 'kkonf', 'kson', 'kat')

            items = [h.TextItem(label=lab,
                                name=name,
                                value=0 if fs[name] is None else fs[name],
                                class_='textboxmid',
                                onBlur="set_term_sum_fachstat('%s')" % name,
                                )
                     for lab, name in zip(label, namen)]
            kat = items[-1]
            kat.bold_label=True
            kat.bold_value=True
            if config.BERATUNGSKONTAKTE and not config.BERATUNGSKONTAKTE_BS:
                # wenn die Terminsumme aus den Beratungskontakten gleich der
                # Terminsumme in der Datenbank ist, nehmen wir an, dass die Daten
                # bernommen wurden und markieren das Kstchen.
                # Wenn nicht, sieht der Benutzer, dass die Daten nicht bernommen sind
                # und kann das Kstchen markieren, um sie erneut zu bernehmen.
                from ebkus.html.beratungskontakt import get_fs_kontakte
                pseudo_fs = {}
                fall_id = fs['fall_id']
                if fall_id:
                    fall = Fall(fall_id)
                    get_fs_kontakte(fall, pseudo_fs)
                    if pseudo_fs['kat'] == fs['kat'] == 0:
                        # bernahme ist nicht default bei neuer FS
                        checked = False
                    else:
                        checked = pseudo_fs['kat'] == fs['kat']
                else:
                    checked = False
                checkitem = h.CheckItem(label="Aus Beratungskontakten bernehmen",
                                        name="uebernehmen",
                                        value='1',
                                        checked=checked,
                                        tip="Markieren, um Terminsummen aus Beratungskontakten zu bernehmen",
                                        )
            else:
                checkitem = h.DummyItem()
            return [items[0:5]+[checkitem], items[5:10]+[kat]]
        ### ende get_termine_daten
        termine = h.FieldsetInputTable(
            legend='Terminsumme',
            daten=get_termine_daten(),
            )
        notiz = h.FieldsetInputTable(
            legend='Notiz',
            daten=[[h.TextareaItem(label='Notiz',
                                   label_width='10%',
                                   name='no',
                                   value=fs['no'],
                                   class_='textareanormal100prz',
                                   ),
            ]],
            )
        diskrepanz_akzeptieren = None
        if (fall_aktuell and not config.FACHSTATISTIK_AKTE_DISKREPANZ_NICHT_ZULASSEN and 
            config.WARNUNG_BEI_FACHSTATISTIK_AKTE_DISKREPANZ):
            # Bei abgeschlossenen Fllen darf es immer Diskrepanzen geben.
            # Es ist aber fraglich, ob die Fachstatistik abgeschlossener Flle
            # wirklich editierbar sein soll. TODO Vielleicht nachfragen, ob
            # das wirklich gewnscht wird?
            # Sinnlos, wenn FACHSTATISTIK_AKTE_DISKREPANZ_NICHT_ZULASSEN 
            # gesetzt ist, da kann es wg. direkter bernahme per definitionem
            # keine Diskrepanz geben.
            diskrepanz_akzeptieren = h.FieldsetInputTable(
                legend="Diskrepanz zur Akte zulassen",
                daten=[[h.CheckItem(label="Diskrepanz zur Akte zulassen",
                                    label_class="labeltextred",
                                    name="diskr",
                                    value='1',
                                    checked=False,
                                    tip="Markieren, um Diskrepanz zur Akte hinsichtlich "
                                        "Lebensmittelpunkt/Empfohlen von/erbrachte Leistungen zuzulassen",
                                    ),
                        h.DummyItem(),
                        ]],
                )
        fs_fuer_abgeschlossenen_fall_modifizieren = self.modifikation_zulassen(fall, "Fachstatistik")
        rows = self._customize(fsjok, # wird nur fr joker verwendet
                               (
                falldaten,
                angabenklient,
                ba1, ba2,
                pbk, pbe,
                anmprobleme,
                kindprobleme,
                elternprobleme,
                jokf5, jokf6, jokf7, jokf8, 
                eleistungen,
                termine,
                notiz,
                diskrepanz_akzeptieren,
                fs_fuer_abgeschlossenen_fall_modifizieren,
                ))
        hidden = ()
        # Die readonly Items bertragen ihre Werte nicht an die Form.
        # Diese mssen extra per hidden angefgt werden.
        if von_akte_readonly:
            hidden += (('zm', _zm),
                      ('fs', _fs),
                       ('gs', _gs),
                      )
            for le in _eleistungen.split():
                hidden += (('eleistungen', le),)
            if von_akte_readonly_anmproblem_bei_anmeldung:
                hidden += (('qualij', _qualij),)
                hidden += (('bkm', _bkm),)
                hidden += (('bkv', _bkv),)
                try:
                    ids = _anmprobleme.split()
                    for id in ids:
                        hidden += (('anmprobleme', id),)
                except:
                    pass
        hidden += (('file', file),
                   ('stz', fs['stz']),
                   ('fallid', fs['fall_id']),
                   ('fall_fn', fs['fall_fn']),
                   ('mitid', fs['mit_id']),
                   ('fsid', fs['id']),
                   )
        res = h.FormPage(
            title=title,
            help=False,
            name="fachstatform",action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(fall__akte_id)s' % fs),
                           ),
            rows=rows + (h.SpeichernZuruecksetzenAbbrechen(),),
            hidden=hidden,
            )
        if pageonly:
            return res
        return res.display()


class fsneu(_fachstatistik):
    """Neue Fachstatistik eintragen. (Tabelle: Fachstatistik)"""
    permissions = Request.STAT_PERM
    def processForm(self, REQUEST, RESPONSE):
        fallid = self.form.get('fallid')
        if not fallid:
            raise EE('Erstellen einer Fachstatistik nur f&uuml;r einen Fall moeglich.')
        fall = Fall(fallid)
        akte = fall['akte']
        geschlecht = akte['gs']
        if not geschlecht:
            raise EE("Bitte zuerst das Geschlecht in die Klientenkarte eintragen!")
        # Ich kanns mir nicht anders vorstellen:
        if akte['aktueller_fall']:
            assert fall['id'] == akte['aktueller_fall']['id'] == akte['letzter_fall']['id']
        # das geht nicht immer weil Gleichheit nicht richtig definiert ist
        #assert fall == akte['letzter_fall'] == akte['aktueller_fall']
        leistungen = fall['leistungen']
        jahresl = ebapi.FachstatistikList(where = "fall_fn = '%s'" % fall['fn'])
        if jahresl:
            raise EE('Fachstatistik f&uuml;r den Fall schon vorhanden')
        fs = Fachstatistik()
        # Alter relativ zum Fallbeginn
        alter = calc_age(akte['gb'], fall.getDate('bg'))
        def altersgruppe():
            ag = cc('fsag','999')
            try:
                ag = bcode('fsag', alter)['id']
            except:
                pass # kein bereich gefunden
            return ag
        def altersgruppeeltern(verwcode):
            # verwcode == '1' Mutter
            # verwcode == '2' Vater
            ag = cc('fsagel','999')
            try:
                for b in akte['bezugspersonen']:
                    if b['verw__code'] == verwcode:
                        if b['gb']:
                            alter = calc_age(b['gb'], fall.getDate('bg'))
                            ag = bcode('fsagel', alter)['id']
            except:
                pass # kein valides Geburtsdatum gefunden
            return ag
                        
        if config.STRASSENKATALOG:
            strasse = get_strasse(akte)
        else:
            strasse = {}
        fs.init(
            id=Fachstatistik().getNewId(),
            mit_id=fall['zustaendig__mit_id'],
            fall_id=fall['id'],
            fall_fn=fall['fn'],
            jahr=str(today().year),
            stz=akte['stzbg'],
            plraum=akte['plraum'],
            ort=akte['ort'],
            plz=akte['plz'],
            ortsteil=strasse.get('ortsteil', ''),
            bezirk=strasse.get('bezirk', ''),
            samtgemeinde=strasse.get('samtgemeinde', ''),
            gs=akte['gs'],
            ag=altersgruppe(),
            agkm=altersgruppeeltern('1'), # code klerv fr Mutter
            agkv=altersgruppeeltern('2'), # code klerv fr Vater
            fs=akte['fs'],
            anmprobleme=None,
            kindprobleme=None,
            elternprobleme=None,
            )
        fs['eleistungen'] = ' '.join([str(leist['le']) for leist in leistungen])
        single_kat_felder = ('zm', 'qualij', 'hkm', 'hkv', 'bkm', 'bkv',
                             'qualikm', 'qualikv',
                             'ba1', 'ba2', 'pbe', 'pbk', )
        for f in single_kat_felder:
            fs[f] = ' ' # leere, selektierte Option, es muss aktiv ausgewhlt werden

        # falls unter 14 Beschftigung vorbelegen
        try:
            if alter < 14:
                fs['qualij'] = cc('fsqualij', '7')
        except:
            pass
        anmeldungen = fall['anmeldung']
        if anmeldungen:
            anm = anmeldungen[0] # Es darf nur eine geben
            fs['zm'] = anm['zm']
            if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
                fs['anmprobleme'] = anm.get('anmprobleme')
        if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
            for f in ('qualij', 'bkm', 'bkv',):
                fs[f] = akte.get(f) or ' '
        joker_felder = ('joka1', 'joka2', 'joka3', 'joka4',
                        'jokf5', 'jokf6', 'jokf7', 'jokf8',)
        for f in joker_felder:
            if fs_customize.multifeld(f):
                fs[f] = None
            else:
                fs[f] = ' ' # leere, selektierte Option, es muss aktiv ausgewhlt werden
        termin_felder = ('kkm', 'kkv', 'kki', 'kpa', 'kfa',
                         'ksoz', 'kleh', 'kerz', 'kkonf', 'kson', 'kat',)
        if config.BERATUNGSKONTAKTE and not config.BERATUNGSKONTAKTE_BS:
            from ebkus.html.beratungskontakt import get_fs_kontakte
            get_fs_kontakte(fall, fs)
        else:
            for f in termin_felder:
                fs[f] = 0
        #print '***********FSNEU', fs
        return self._process(title='Neue Fachstatistik erstellen',
                             file='fseinf',
                             fs=fs,
                             )

# kann auch als updfsform angesprochen werden (siehe EBKuS.py)
# wird in menu_templates.py verwendet
class updfs(_fachstatistik):
    permissions = Request.STAT_PERM
    def processForm(self, REQUEST, RESPONSE):
        fsid = self.form.get('fsid')
        fallid = self.form.get('fallid')
        if fsid:
            fs = Fachstatistik(fsid)
        elif fallid:
            fall = Fall(fallid)
            fs_list = fall['fachstatistiken']
            if not fs_list:
                raise EE('Noch keine Fachstatistik f&uuml;r den Fall vorhanden.')
            else:
                fs = fs_list[-1] # msste eigentlich immer nur eine sein ...
        else:
            self.last_error_message = "Keine ID fr die Fachstatistik erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        joker_felder = ('joka1', 'joka2', 'joka3', 'joka4',
                        'jokf5', 'jokf6', 'jokf7', 'jokf8',)

        # Fr den Fall, dass ein Joker aktiviert wird und Fachstatistik
        # bereits vorhanden ist.
        # Diese Lsung fhrt dazu, dass alte Statistiken nicht mehr
        # abspeicherbar sind, wenn man keinen Wert eintrgt. Das ist aber
        # akzeptabel.
        fsjok = {} # Ersatzobjekt, da echtem fs keine Werte zugewiesen werden drfen
        for f in joker_felder:
            #print 'FS: ', f, fs[f], fs_customize.multifeld(f)
            if fs[f] == None and not fs_customize.multifeld(f):
                #print 'FS SINGLE: ', f, fs[f]
                fsjok[f] = ' ' # leere, selektierte Option, es muss aktiv ausgewhlt werden
            else:
                #print 'FS MULTI: ', f, fs[f]
                fsjok[f] = fs[f] # None ist ok bei multi
        return self._process(title='Fachstatistik &auml;ndern',
                             file='updfs',
                             fs=fs,
                             fsjok=fsjok,
                             )

          
class viewfs(_fachstatistik):
    permissions = Request.STAT_PERM
    def processForm(self, REQUEST, RESPONSE):
        fsid = self.form.get('fsid')
        if fsid:
            fs = Fachstatistik(fsid)
        else:
            self.last_error_message = "Keine ID fr die Fachstatistik erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        title='Fachstatistik fr Fallnummer %(fall_fn)s ' % fs + \
            'am %(day)d.%(month)d.%(year)d' % today()
        ueberschrift = h.TableDataTable(
            daten=[[h.String(string=title,
                                   class_='tabledatabold'),
                    ],
                   [h.Dummy()],
                   ],
            )
        page = self._process(title='Fachstatistik ansehen',
                             file='updfs',
                             fs=fs,
                             pageonly=True,
                             )
        data = []
        page.get_data(data)
        daten = []
        for label, value in data:
            if not label and not value:
                continue
            if label == 'legend':
                daten.append( [h.String(string=value,
                                   class_='tabledatabold'),
                          h.String(string='')])
            elif value and isinstance(value, (list, tuple)):
                daten.append( [h.String(string=label),
                          h.String(string=value[0]),
                          ])
                for v in value[1:]:
                    daten.append( [h.String(string=''),
                              h.String(string=v),
                              ])
            else:
                daten.append( [h.String(string=label),
                          h.String(string=value),
                          ])
        tabelle = h.TableDataTable(
            headers=('Item', 'Wert'),
            daten=daten,
            )
        res = h.Page(
            title = title,
            ueberschrift=ueberschrift,
            rows=(h.Schliessen(),
                  ueberschrift,
                  tabelle,
                  h.Schliessen(),
                  ),
            )
        return res.display()

def regional_daten(fs=None, akte=None, fall=None):
    """Liefert eine dict mit alle Regionaldaten, entweder mit Wert oder ''.
    Beim aktuellen Fall werden die Daten immer aus der Akte bzw. dem Straenkatalog
    genommen, beim nicht-aktuellen Fall nur dann, wenn in der Fachstatistik
    noch kein Ort angegeben war.

    Aktuelle Flle bekommen also immer die aktuellen Daten aus der Akte, alte Flle bekommen
    diese Daten nur, wenn sie vorher keine hatten.
    """
    #print 'REGIONALDATEN FALL AKTUELL', fall['aktuell']
    daten = {}
    for f in ('ort', 'plz', 'plraum', 'ortsteil', 'bezirk', 'samtgemeinde'):
        if fs:
            daten[f] = fs[f]
        else:
            daten[f] = ''
    if (akte and fall and fall['aktuell']) or (fs and not fs['ort']):
        #print 'REGIONALDATEN UEBERNEHMEN', fall['aktuell'], daten['plraum'], akte['plraum']
        if config.STRASSENKATALOG:
            strasse = get_strasse(akte)
        else:
            strasse = {}
        daten['plraum'] = akte['plraum']
        daten['ort'] = akte['ort']
        daten['plz'] = akte['plz']
        daten['ortsteil'] = strasse.get('ortsteil', '')
        daten['bezirk'] = strasse.get('bezirk', '')
        daten['samtgemeinde'] = strasse.get('samtgemeinde', '')
    return daten

