# coding: latin-1

"""Module fr Gruppenloeschen"""

import logging

from ebkus.db.sql import SQL
from ebkus.db.dbapp import DBAppError
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, Gruppe, FallList, GruppeList, Altdaten, Anmeldung, \
     Zustaendigkeit, today, cc, check_list, check_code, check_int_not_empty, \
     check_str_not_empty, check_date, EE
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.akte_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share




class rmgruppenf(Request.Request, akte_share):
    """Fehlerhafte Gruppen lschen.
    """
    permissions = Request.ADMIN_PERM


    def processForm(self, REQUEST, RESPONSE):
        gruppe = self.form.get('gruppe')            # Gruppe zum lschen anbieten, sonst Akte
        fn = self.form.get('fn')                    # Diesen Fall zum Lschen prfen
        gn = self.form.get('gn')                    # Diese Gruppe zum Lschen prfen
        if fn and gn:
            raise EE("Es kann entweder ein Fall oder eine Gruppe gelscht werden, nicht beides zugleich")
        statistik_auch = (self.form.get('statistik_auch') == '1')
        ok = self.form.get('ok')                    # ok zum endgltigen Lschen
        fall_id = self.form.get('fallid')           # Diesen Fall endgltig lschen
        gruppe_id = self.form.get('gruppeid')       # Diese Gruppe endgltig lschen
        if gn and not ok and not gruppe_id:
            gruppen = GruppeList(where="gn = '%s'" % gn)
            if len(gruppen) < 1:
                raise EE("Kein Gruppe mit dieser Gruppennummer gefunden.")
            elif len(gruppen) > 1:
                # darf nicht passieren
                raise EE("Mehrere passende Gruppen gefunden.")
            else:
                gruppe = gruppen[0]
            if gruppe['ey']:
                raise EE(("Gruppe <em>%s</em> hat eine Endedatum, ist also abgeschlossen. " +
                         "Es knnen nur laufende Gruppen gelscht werden.")
                         % gn)
            if gruppe['faelle'] or gruppe['bezugspersonen']:
                raise EE(("Die Gruppe <em>%s</em> hat Teilnehmer. " +
                         "Es knnen nur Gruppen ohne Teilnehmer gelscht werden.")
                         % gn)
            legend = 'Gruppe endgltig lschen'
            zeilen=("Soll die Gruppe <em>%(gn)s (%(name)s)</em> endgltig gelscht werden?" % gruppe,
                        )
            hidden = (('gruppeid', gruppe['id']),
                      ('ok', '1'),
                )
            return h.SubmitOrBack(
                legend=legend,
                action='rmgruppenf',
                method='post',
                hidden=hidden,
                zeilen=zeilen,
                ).display()
        elif gruppe_id and ok == '1':
            gruppe = Gruppe(gruppe_id)
            gn = gruppe['gn']
            name = gruppe['name']
            from ebkus.app.ebupd import remove_gruppe
            remove_gruppe(gruppe)
            legend = "Gruppe gelscht"
            geloeschte_gruppe = h.FieldsetDataTable(
                legend=legend,
                headers=('Gruppennummer', 'Name'),
                daten=[[h.String(string=gn),
                        h.String(string=name),
                        ]],
                )
            res = h.Page(
                title=legend,
                breadcrumbs = (('Aministratorhauptmen', 'menu'),
                               ('Einzelne Gruppe lschen', None),
                               ),
                rows=(self.get_hauptmenu(),
                      geloeschte_gruppe,
                      ),
                )
            return res.display()
            
        auswahl_gruppe = h.FieldsetFormInputTable(
            name="rmgruppeanzeigen",
            action="rmgruppenf",
            method="post",
            legend='Gruppennummer der zu lschenden Gruppe',
            daten=[[h.TextItem(label='Gruppennummer',
                               name='gn',
                               value='',
                               tip='Gruppennummer der zu lschenden Gruppe'
                               ),
                    h.DummyItem(label_width="50%", class_="textbox280"), # breites Dummy aus optischen Grnden
                    ],
                   [h.Dummy(n_col=4)],
                   ],
            button=h.Button(value="Lschen",
                            name='op',
                            tip="Gruppe mit der angegebenen Nummer endgltig lschen",
                            type='submit',
                            n_col=4,
                            ),
            )
        res = h.Page(
            title= 'Gruppe lschen wg. Fehler',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  auswahl_gruppe,
                  ),
            )
        return res.display()


class rmgruppen(Request.Request, akte_share):
    """Gruppen lschen wg. Datenschutz."""
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        #print self.form
        alter = self.form.get('alter')
        if alter == None:
            alter = config.LOESCHFRIST
        else:
            alter = check_int_not_empty(
            self.form, 'alter', 'Fehler im Alter in Monaten')
            if alter < config.LOESCHFRIST:
                raise EE('Lschfrist betrgt mindestens %s Monate' % config.LOESCHFRIST)
        op = self.form.get('op')
        rm = check_list(self.form, 'rmgr', 'Fehler in markierten Gruppen', '')
        if op == 'Endgltig lschen' and rm:
            hidden = [('rmgr', r) for r in rm] + [('op', 'loeschen_confirmed')]
            return h.SubmitOrBack(
                legend='Endgltig lschen',
                action='rmgruppen2',
                method='post',
                hidden=hidden,
                zeilen=("Sollen die markierten Gruppen endgltig gelscht werden?",
                        )
                ).display()
        alter_auswahl = h.FieldsetFormInputTable(
            name="rmgruppenanzeigen",
            action="rmgruppen",
            method="post",
            hidden = (),
            legend='Alter der zu lschenden Gruppen whlen',
            daten=[[h.TextItem(label='Monate seit Beendigung der Gruppe',
                               name='alter',
                               value=alter or '',
                               class_='textboxsmall',
                               maxlength=3,
                               tip='Mindesanzahl der Monate seit Beendigung der Gruppe'
                               ),
                    ],
                   [h.Dummy(n_col=2)],
                   [h.Button(value="Anzeigen",
                             name='op',
                             tip="Gruppendes gewhlten Alters anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    ],
                   ],
            )
        if alter:
            rmdatum = today().add_month(-(alter+1)) # damit immer mindestens die Anzahl Monate
                                                    # dazwischen liegt
            gruppe_list = GruppeList(
                where = '(ey < %(year)s or (ey = %(year)s and em <= %(month)s)) and ey > 0'
                   % rmdatum, 
                order = 'ey, em, ed')
            gruppen = h.FieldsetFormDataTable(
                name="rmgruppen",
                #action="abfragedef",
                action="rmgruppen",
                method="post",
                hidden = (),
                legend='Gruppen zum lschen auswhlen',
                headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                         'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende'),
                daten=[[h.String(string=gruppe['gn']),
                        h.String(string=gruppe['mitarbeiternamen']),
                        h.String(string=gruppe['name']),
                        h.String(string=gruppe['thema']),
                        h.String(string=gruppe['grtyp__name']),
                        h.String(string=gruppe['teiln__name']),
                        h.String(string=gruppe['tzahl']),
                        h.Datum(date=gruppe.getDate('bg')),
                        h.Datum(date=gruppe.getDate('e')),
                        h.CheckItem(label='',
                                    name='rmgr',
                                    value=gruppe['id'],
                                    checked=False,
                                    tip='Hier markieren, um die Gruppe endgltig zu lschen',
                                    ),
                        ] for gruppe in gruppe_list],
                no_button_if_empty=True,
                button=h.Button(value="Endgltig lschen",
                           name='op',
                           tip="Markierte Gruppen endgltig lschen",
                           type='submit',
                           class_='buttonbig',
                           n_col=10,
                     ),
                empty_msg="Keine Gruppen gefunden.",
                )
            
        else:
            gruppen = None
        res = h.Page(
            title='Gruppen lschen nach Ablauf der Lschfrist',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  alter_auswahl,
                  gruppen,
                  ),
            )
        return res.display()
        
class rmgruppen2(Request.Request, akte_share):
    """Lscht die Gruppen, deren ID im Parameter rmgr steht.
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        from ebkus.app.ebapi import check_list
        #print self.form
        op = self.form.get('op')
        rm = check_list(self.form, 'rmgr', 'Fehler in markierten Gruppen', '')
        if op == 'loeschen_confirmed' and rm:
            if rm:
                rmdatum = today().add_month(-(config.LOESCHFRIST+1)) # damit immer mindestens die Anzahl Monate
                                                                     # dazwischen liegt
                gruppe_list = [Gruppe(r) for r in rm]
                for a in gruppe_list:
                    # vor dem Lschen holen!
                    assert (a['ey'] > 0 and 
                            (a['ey'] < rmdatum.year or 
                             (a['ey'] == rmdatum.year) and a['em'] < rmdatum.month))
                from ebkus.app.ebupd import removeakten
                anzahl_akten_geloescht, anzahl_gruppen_geloescht = removeakten(self.form)
                assert anzahl_gruppen_geloescht == len(gruppe_list), 'Fehler beim Lschen von Gruppen'
            else:
                raise EE('Keine Gruppen zum lschen markiert')
            geloeschte_gruppen = h.FieldsetDataTable(
                legend='Endgltig gelschte Gruppen',
                headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                         'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende'),
                daten=[[h.String(string=gruppe['gn']),
                        h.String(string=gruppe['mitarbeiternamen']),
                        h.String(string=gruppe['name']),
                        h.String(string=gruppe['thema']),
                        h.String(string=gruppe['grtyp__name']),
                        h.String(string=gruppe['teiln__name']),
                        h.String(string=gruppe['tzahl']),
                        h.Datum(date=gruppe.getDate('bg')),
                        h.Datum(date=gruppe.getDate('e')),
                        ] for gruppe in gruppe_list],
                )
            
        else:
            geloeschte_gruppen = None
        res = h.Page(
            title='Akten gelscht',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ('Gruppen lschen', None),
                           ),
            rows=(self.get_hauptmenu(),
                  geloeschte_gruppen,
                  ),
            )
        return res.display()
