# coding: latin-1

import pytest
import os
import sys

from os.path import dirname, realpath, join
# Testdatenbank initialisieren

#print sys.path

#@pytest.fixture(scope='package')
@pytest.fixture(scope='package')
def ebkus_home():
    d = dirname
    return d(d(d(d(realpath(__file__)))))


#@pytest.fixture(scope='module')
@pytest.fixture
def ebkus_control(ebkus_home):
    sys.path.insert(0, ebkus_home)
    from ebkusctl import EBKuSControl, ServiceException
    ectl = EBKuSControl(ebkus_home)
    return ectl 


@pytest.fixture(autouse=True)
def initialize_test_database(ebkus_home):
    """Die Testinstanz initialisieren."""
    from ebkus.Install import InstallFromHome
    installer = InstallFromHome(ebkus_home)
    installer.init()
    test_instance = installer.ebkus['test']
    assert test_instance.name == 'test'
    standard_sql = join(ebkus_home, 'sql', 'standard.sql')
    test_instance.create_database(sql_file=standard_sql)
    print 'Database was initialized'
    return 1

@pytest.fixture
def test_config(ebkus_control):
    """Sicherstellen, dass nach 'from ebkus.config import config'
    config auf die Konfiguration der Testinstanz gesetzt ist.
    config zurückgebe,
    """
    #res = initialize_test_database
    from ebkus.config import init
    config = init('test', ebkus_control.ebkus_conf)
    return config

# @pytest.fixture
# def test_config(ebkus_control):
#     """Sicherstellen, dass nach 'from ebkus.config import config'
#     config auf die Konfiguration der Testinstanz gesetzt ist.
#     config zurückgebe,
#     """
#     test_instanz = 'test'
#     assert test_instanz in ebkus_control.instances, "Testinstanz '{0}' ist nicht konfiguriert".format(test_instanz)
#     from ebkus.config import init
#     config = init(test_instanz, ebkus_control.ebkus_conf)
#     # print dir(config)
#     # config.show()
#     #print 'EBKuSTestCase.config:', id(self.config)
#     assert config.INSTANCE_NAME == test_instanz
#     return config

@pytest.fixture
def test_daten(ebkus_home, test_config):
    #print 'SYS:PATH', sys.path
    config = test_config
    from ebkus.gen import test_daten
    from ebkus.db.sql import opendb, closedb
    opendb()
    test_daten = test_daten.TestDaten(profil='minimal',
                                      instance_name='test',
                                      seed=101, 
                                      #seed=None, 
                                      von_jahr=None,
                                      #heute="2016-01-05",
                                      open_database=False,
                                      )
    yield test_daten
    closedb()
    

@pytest.fixture
def interact(test_config):
    from ebkus.test.interactforms import InteractForms
    interact = InteractForms(test_config)
    return interact


@pytest.fixture
def interact_testdaten_testinput(test_config):
    """
    Eine Testklasse

    Testdaten erzeugt forms (dicts), mit denen eine EBKuS-Instanz
    gefüttert werden kann (direkt über ebupdfunktionen, über Request.process
    von außen über http post).
    zB die Methoden miteinf oder akteeinf, die 1:1 ebupd Funktionen
    zugeordnet sind.
    Dabei kann man alle Daten festlegen, oder nur einen Teil, der Rest wird 
    zufällig aufgefüllt. Es gibt (fast) immer Pflichtdaten, die
    explizi angegeben werden müssen.

    Testinput erzeugt über submit komplexe Eingabenfolgen.
    zB mitarbeiter() führ eine Reihe von Mitarbeitern für
    Test- und Demosysteme ein. Dabei wird eine Instanz
    der Testdatenklasse verwendet.

    Am logischsten und verständlichsten wäre eine Interact-Klasse,
    die über login, post, get, logout mit EBKuS kommuniziert.
    Sie verwendet dabei convenience-Methoden aus Testdaten und Testinput.
    
    Geht das oder muss ich was ändern?
    class Testinteraction(InteractForms):
       def __init__(self, testdaten, testinput):
          testinput.testdaten = testdaten
          testinput.interact = self
       def submit(self, file, **kw):
          testinput.submit(file, **kw)
       def submit(self, file, redirect=True, **kw):
          form = getattr(self.test_daten, file)(file=file, **kw)
          path = self.path[file]
          r = self.session.post(path, data=form)
          if redirect and r.status_code == 301:
             rr = self.get(r.location)
          self.check_error(r, is_in_title=self.title[file])

    """
    from ebkus.test.interactforms import InteractForms
    interact = InteractForms(test_config)
    


    return interact
    


# mit bottle starten

@pytest.fixture
def bottle_started():
    pass
