# coding: latin-1

"""Datenbank fr Testzwecke aufblhen.

"""


import sys,os
import re
#from os.path import join, split, dirname, basename, exists, isdir, isfile, normpath, normcase, abspath, realpath, splitext, getmtime
import datetime
import traceback

import ebkus.app.ebapigen as apiclasses
from ebkus.config import config
from ebkus.app.ebapi import register_get, register_set, FeldList, Tabelle
from ebkus.db.sql import SQL, getDBHandle

class DuplicateDatabase(object):
    """
    Vorhandene Datenbank mit Mitarbeitern und Klientendaten ist Batch0.
    Diese kann um Batch1 ... Batchn erweitert werden, wobei Batchx
    eine Kopie von Batch0 ist, bis auf folgende Anpassungen:
    - Die Ids der Objekte werden angepasst, so dass FK-Beziehungen erhalten bleiben. 
    - An alle Nachnamen (Feld 'na') wird in den Kopien die Batchnummer (batch_index) 
      angehngt.
    - In die Fallnummern wird die Batchnummer eingefgt, so dass diese eindeutig bleiben.
    - An die Benutzerkennungen wir die Batchnummer angehngt (test --> test1, verw --> verw1, ...).
    
    Weitere Stellen lasse ich im ersten Schritt weg, vielleicht spter. 
    Die Verwaltungsrollen werden sehr viele offene Flle sehen.
    """
    dont_duplicate = {
        'register',
        'altdaten',
        'ags',
        'strkatalog',
        'kategorie',
        'code',
        'tabelle',
        'feld',
        'protokoll',
        }
    def get_existing_batches(self):
        key = 'batch_info'
        #nbatches=3;akte=33,99;fall=70,210
        batch_info = register_get(key)
        if batch_info:
            pass
        else:
            return None



    def __init__(self):
        self.batch_classes = self._get_batch_classes()
        self.tables = {table for table,_,_ in self.batch_classes}
        self.table2max_index_of_batch0 = self._get_table2max_index_of_batch0()
        self.dont_duplicate_ids = [Tabelle(tabelle=t)['id'] for t in self.dont_duplicate]
        for k,v in self.table2max_index_of_batch0.items():
            print k, v
    def _get_table2max_index_of_batch0(self):
        res = {}
        for table in self.tables:
            max_index = SQL("select max(id) from {}".format(table)).execute()[0][0]
            max_index = 0 if max_index is None else max_index
            print 't2max:', table, max_index
            res[table] = max_index
        return res
    def _get_batch_classes(self):
        import inspect
        import ebkus.app.ebapigen as apiclasses
        from ebkus.db.dbapp import Container
        container_classes = [v for k,v in vars(apiclasses).items() if inspect.isclass(v) and issubclass(v, Container) and v is not Container]
        batch_classes = [(v.resultClass.table, v.resultClass, v) for v in container_classes if v.resultClass.table not in self.dont_duplicate]
        return batch_classes
    def do_batches(self, n):
        """Dupliziere die Datenbank n-mal.
        n: how many additional batches to do
        """
        for batch_index in range(1, n+1):
            self.do_batch(batch_index)
    def do_batch(self, batch_index):
        """Eine Duplizierung fr einen bestimmten Index durchfhren.
        """
        for table, dbobject, container in self.batch_classes:
            self.do_batch_for_one(table, dbobject, container, batch_index)
    def do_batch_for_one(self, table, dbobject, container, batch_index):
        """Eine Tabelle fr einen Index duplizieren.
        """
        print 'Batch fuer:', table
        max_index_of_batch0 = self.table2max_index_of_batch0[table]
        fk_max_index_list = [(f['feld'], self.table2max_index_of_batch0[Tabelle(f['ftab_id'])['tabelle']]) 
                             for f in FeldList(where=
                                               'tab_id = {} and not ftab_id in ({})'.
                                               format(Tabelle(tabelle=table)['id'],
                                                      ','.join([str(i) for i in self.dont_duplicate_ids])))]
        print fk_max_index_list
        # batch0 enthlt zu duplizierende Objekte aus der Tabelle
        batch0 = container(where='id <= {}'.format(max_index_of_batch0))
        print len(batch0)
        for obj in batch0:
            new_obj = dbobject()
            # In data kann man erstmal alles reinpacken ohne Fehlermeldung
            # Alle folgende nderungen in dieser dict.
            u = new_obj.data
            u.update(obj)
            del u['__p__']
            # Id anpassen.
            # Ergibt sich aus der alten Id, der maximalen Id aus Batch0 und dem Index.
            u['id'] = obj['id'] + max_index_of_batch0*batch_index
            # FKs anapassen
            for feld, max_index_of_fk in fk_max_index_list:
                old_id = u[feld]
                if old_id is None:
                    new_id = None
                else:
                    new_id = old_id + max_index_of_fk*batch_index
                    u[feld] = new_id
                print "{}: {} {} --> {}".format(table, feld, old_id, new_id)
            # An Name batchnummer dranhngen
            if 'na' in u and u['na']:
                old_name = u['na']
                new_name = "{}{}".format(old_name, batch_index)
                u['na'] = new_name
                print "{}: {} {} --> {}".format(table, 'na', old_name, new_name)
            # Fallnummer fixen: <lfd>-<jahr><stelle> --> <lfd>-<batch_index>-<jahr><stelle>
            for feld in ('fn', 'fall_fn'):
                if feld in u:
                    old_fn = u[feld]
                    lfd, rest = old_fn.split('-')
                    new_fn = "{}-{}-{}".format(lfd, batch_index, rest)
                    u[feld] = new_fn
                    print "{}: {} {} --> {}".format(table, feld, old_fn, new_fn)
            # Benutzerkennung eindeutig machen
            if 'ben' in u:
                feld = 'ben'
                old_ben = u[feld]
                new_ben = "{}{}".format(old_ben, batch_index)
                u[feld] = new_ben
                print "{}: {} {} --> {}".format(table, feld, old_ben, new_ben)
            new_obj.insert()
        print 'Done:', table




# class NamenUmbenennen(object):
#     """Alle Namen in der Datenbank durch zufllig ausgewhlte Namen ersetzen.
#     Aber so, jeder einzelne Namen immer durch denselben zuflligen ersetzt wird.
#     Bezugsperson und Klient: vn, na
#     Telefonnummern, es gengt die ersten Ziffern zu ndern.
#     Straennamen: name, ort
#     Bentigte Pools:
#       Vornamen, Nachnamen, Straennamen, Ortsnamen
#     Mitarbeiter: vn, na, ben, aber getrennt behandeln (test, verw)
#     """
#     def __init__(self):
#         vornamen_pool = set(self.get_vornamen())
#         vornamen_map = {} # exist. vorname --> aus dem pool gemappter vorname
#     def vornamen_umbenennen(self):
#         for klass in (AkteList, BezugspersonList):
#             for obj in klass(where=''):
#                 vn = obj['vn']
#                 for sep in (' ', '-'):
#                     namen = vn.split(sep)
#                     namen_new = []
#                     for name in namen:
#                         name_new = self.vornamen_map[name]
#                         if not name_new:
#                             name_new = self.vornamen_pool.pop()
#                             self.vornamen_map[name] = name_new
#                         namen_new.append(name_new)
#     def vn_umbenennen(self, vn):
#         vn = vn.strip()
#         vn_list = [vn]
#         sep = ''
#         for x in (' ', '-'):
#             if x in vn:
#                 vn_list = vn.split(x)
#                 sep = x
#         for i,vn in enumerate(vn_list):
#             self.repla
                        
# class Umbenenner(object):
#     def umbenennen(name):
#         return self._get_replacement(name)
# class PoolUmbenenner(Umbenenner):
#     def __init__(self, pool):
#         self.pool = pool
#         self.map = {}
#     def _get_replacement(name):
#         if name in self.map:
#             return self.map[name]
#         new_name = self.pool.pop()
#         self.map[name] = new_name
#         return new_name
# class NamenUmbenenner(PoolUmbenenner):
#     _re_suffix = re.compile(r'^(.+?)(--[0-9]+)*$')
#     def umbenennen(name):
#         name = name.strip()
#         name, suffix = self._re_suffix.match(name)
#         name_list = [name]
#         sep = ''
#         for x in (' ', '-'):
#             if x in name:
#                 name_list = name.split(x)
#                 sep = x
#         new_name_list = []
#         for n in name_list:
#             new_name_list.append(self._get_replacement(n))
#         new_name = sep.join(new_name_list) + suffix if suffix else ''
#         return new_name
# class TelefonUmbenenner(Umbenenner):
#     def _get_replacement(name):
#         # ersten 5 Ziffern durch 09999
#         return '09999' + name[5:]
# class StrassenUmbenenner(PoolUmbenenner):
#     pass

## Ab hier Code zum rumspielen und testen
## Wird fr EBKuS nicht eingesetzt.


def getallsql(where='', limit=100000, dictionary=False):
    sql = """
SELECT a.*,f.*,z.*,j.*,fs.*
FROM fall f
     INNER JOIN akte a ON f.akte_id = a.id
     INNER JOIN zustaendigkeit z ON f.id = z.fall_id
     INNER JOIN mitarbeiter m ON m.id = z.mit_id
     LEFT JOIN jghstat07 j ON f.id = j.fall_id
     LEFT JOIN fachstat fs ON f.id = fs.fall_id
WHERE 1=1
ORDER BY f.bgy, f.bgm, f.bgd
""".format(where=where, limit=limit)
    res = getDBHandle().query(sql, dictionary=dictionary)
    return res

def join_fall_mit_neuester_zustaendigkeit():
    """
    concat(bgy, lpad(bgm, 2, '0'), lpad(bgd, 2, '0')) as date,
    
    """
    sql = """
SELECT f.id, zall.mit_id, zall.date, zall.bgy, zall.bgm, zall.bgd
FROM fall f
     INNER JOIN 
     ( 
       SELECT fall_id, MAX(CONCAT(bgy, LPAD(bgm, 2, '0'), LPAD(bgd, 2, '0'))) date
       FROM zustaendigkeit
       GROUP BY fall_id
     ) zmax ON f.id = zmax.fall_id
     INNER JOIN
     (
       SELECT fall_id, CONCAT(bgy, LPAD(bgm, 2, '0'), LPAD(bgd, 2, '0')) date, mit_id, bgy, bgm, bgd
       FROM zustaendigkeit
     ) zall ON zmax.fall_id = zall.fall_id AND
               zmax.date = zall.date
ORDER BY f.id;
"""

    sql = """
SELECT f.id, zall.mit_id, zall.date, zall.bgy, zall.bgm, zall.bgd
FROM fall f
     INNER JOIN 
     ( 
       SELECT fall_id, MAX(bgy) date
       FROM zustaendigkeit
       GROUP BY fall_id
     ) zmax ON f.id = zmax.fall_id
     INNER JOIN
     (
       SELECT fall_id, bgy date, mit_id, bgy, bgm, bgd
       FROM zustaendigkeit
     ) zall ON f.id = zall.fall_id AND
               zmax.date = zall.date
ORDER BY f.id;
"""


# akte a ON f.akte_id = akte.id
#               INNER JOIN akte a ON f.akte_id = akte.id
# """    



#def beratungen(self, welche, 
def beratungen_sql(welche, 
        mitarbeiter=None,
        stelle=None,
        ab_jahr=None,
        ab_fallnummer=None,
        bis_jahr=None,
        bis_fallnummer=None,
        ab_jahr_zda=None,
        bis_jahr_zda=None,
        #erledigt=None,
        #letzter_kontakt_vor=None, # ebapi.Date
        sort=(),
        limit=1000000):
    print ','.join(sort)
    # bgy ist ambig in der SQL-Query unten
    sort = tuple([('f.bgy' if x=='bgy' else x) for x in sort])
    assert welche in ('laufend', 'abgeschlossen', 'alle')
    #assert erledigt in (None, 'nicht erledigt', 'erledigt', 'alle')
    assert ab_jahr and ab_fallnummer or not ab_fallnummer
    # join = [('zustaendigkeit', 'zustaendigkeit.fall_id=fall.id'),
    #         ('akte', 'fall.akte_id=akte.id'),
    #         ('mitarbeiter', 'zustaendigkeit.mit_id=mitarbeiter.id')]

    where = 'f.zday %s 0' % (welche=='laufend' and '=' or
                                welche=='abgeschlossen' and '>' or
                                welche=='alle' and '>='
                                )
    # nur die letzte Zustndigkeit deren Endedatum gleich dem ZDA-Datum ist
    # where += """ and f.zday <= zustaendigkeit.ey and 
    # f.zdam <= zustaendigkeit.em and 
    # f.zdad <= zustaendigkeit.ed"""

    if mitarbeiter:
        where += ' and m.id=%s' % mitarbeiter['id']
        # Damit wird gewhrleistet, dass bei offenen Fllen nur der zuletzt Zustndige genommen wird:
        where += ' and (z.ed = 0 or f.zdad > 0)'
    if stelle:
        where += ' and a.stzbg=%s' % stelle['id']
    if ab_jahr:
        where += ' and f.bgy >= %s' % ab_jahr
    if bis_jahr:
        where += ' and f.bgy <= %s' % bis_jahr
    if ab_jahr_zda:
        where += ' and f.zday >= %s' % ab_jahr_zda
    if bis_jahr_zda:
        where += ' and f.zday <= %s and f.zday > 0' % bis_jahr_zda
    # if erledigt:
    #     ja = cc('ja_nein', '1')
    #     if erledigt == 'erledigt':
    #          where += ' and f.erledigt = %s' % ja
    #     elif erledigt == 'nicht erledigt':
    #         where += ' and (f.erledigt != %s or f.erledigt is NULL)' % ja
    # fall_list = FallList(
    #     where=where,
    #     join=join)
    #  ab jahr ab Fallnummer und bis jahr bis Fallnummer
    #   f.bgy > {ab_jahr} or ({ab_jahr} = f.bgy and SUBSTRING_INDEX(f.fn, '-', 1) >= {ab_fallnummer})) and 
    #  (f.bgy < {bis_jahr} or ({bis_jahr} = f.bgy and substring_index(f.fn,'-',1) <= {bis_fallnummer}))
    if ab_jahr and ab_fallnummer:
        where += " and (f.bgy > {ab_jahr} or ({ab_jahr} = f.bgy and SUBSTRING_INDEX(f.fn, '-', 1) >= {ab_fallnummer}))".format(**locals())
    if bis_jahr and bis_fallnummer:
        where += " and (f.bgy < {bis_jahr} or ({bis_jahr} = f.bgy and SUBSTRING_INDEX(f.fn,'-',1) <= {bis_fallnummer}))".format(**locals())
    # if letzter_kontakt_vor:
    #     #print 'OHNE FILTER', len(fall_list)
    #     def letzter_kontakt_aelter_als(fall):
    #         # Flle ohne Beratungskontakte werden immer ausgeschlossen
    #         beratungskontakte = fall['beratungskontakte']
    #         if beratungskontakte:
    #             if beratungskontakte[-1].getDate('k') <= letzter_kontakt_vor:
    #                 return True
    #         return False
    #     fall_list = fall_list.filter(letzter_kontakt_aelter_als)
        #print 'MIT FILTER', len(fall_list)
    #print 'SORT', sort

                # daten=[[h.Link(string=fall['fn'],
                #                url='klkarte?akid=%(akte_id)s' % fall),
                #         h.String(string=fall['akte__vn_xxx']),
                #         h.String(string=fall['akte__na_xxx']),
                #         h.String(string=fall['akte__gb']),
                #         h.Datum(date=fall.getDate('bg')),
                #         h.Datum(date=fall.getDate('zda')),
                #         h.String(string=fall['zuletzt_zustaendig__mit__na']),
                #         h.String(string=fall['has_fachstatistik']),
                #         h.String(string=fall['has_jghstatistik'])



    sql = """
SELECT f.id as fall_id,
       z.id as zust_id,
       f.fn as fn,
       CAST(SUBSTRING_INDEX(f.fn, '-', 1) AS UNSIGNED) as fn_count,
       a.id as akte_id,
       a.na as akte__na,
       IF(c.code='ja','xxx',a.vn) as akte__vn_xxx,
       IF(c.code='ja','xxx',a.na) as akte__na_xxx,
       a.gb as akte__gb,
       CONCAT(LPAD(f.bgd, 2, '0'),'.',LPAD(f.bgm, 2, '0'),'.',LPAD(f.bgy, 4, '0')) as bg,
       IF(f.zday=0,'offen',CONCAT(LPAD(f.zdad, 2, '0'),'.',LPAD(f.zdam, 2, '0'),'.',LPAD(f.zday, 4, '0'))) as zda,
       m.na as zuletzt_zustaendig__mit__na,
       IF(f.zday=0,IF(fs.jahr IS NOT NULL, fs.jahr, 'nv'),'-') as has_fachstatistik,
       IF(f.zday=0,IF(j.jahr IS NOT NULL, j.jahr, 'nv'),'-') as has_jghstatistik
FROM fall f
     INNER JOIN akte a ON f.akte_id = a.id
     INNER JOIN zustaendigkeit z ON f.id = z.fall_id AND
                                   f.zday = z.ey AND  
                                   f.zdam = z.em AND  
                                   f.zdad = z.ed
     INNER JOIN mitarbeiter m ON m.id = z.mit_id
     LEFT JOIN jghstat07 j ON f.id = j.fall_id
     LEFT JOIN fachstat fs ON f.id = fs.fall_id
     LEFT JOIN code c ON a.sperre = c.id
WHERE {where}
ORDER BY {sort}
LIMIT {limit}
""".format(where=where, sort=','.join(sort), limit=limit)
    res = getDBHandle().query(sql, dictionary=True)
    # mehrfache Zustndigkeiten eliminieren, die mit hchster id behalten
    fallnummern = {} # fall_id --> zust_id
    for x in res:
        fall_id = x['fall_id']
        zust_id = x['zust_id']
        # grere zust_id fr fall gefunden, merken
        # nach einem Durchgang bleibt nur die grte stehen
        if fall_id not in fallnummern or zust_id > fallnummern[fall_id]:
            fallnummern[fall_id] = zust_id
    # beim 2. Durchgang alle weglassen, deren zust_id nicht drin steht
    res_fixed = [x for x in res if x['zust_id'] == fallnummern[x['fall_id']]]        
    print len(res), len(res_fixed)
    return res_fixed



def get_akte(**kw):
    """kw: Klassenname --> id
    groe join fr all objekte in der klasse
    akte, fall, anmeldun
    """
    pass

def getakten():
    """Alle wesentlichen fallbezogenen Objekte mit einer einzigen Query holen:
    Akte, Fall, Einrichtung, Anmeldung, Jugendhilfestatistik, Fachstatistik, aktuelle Zustndigkeit, Mitarbeiter.
    """

    # Hier gibt es immer noch doppelte Zustndigkeiten, weil mehrer Zustndigkeit mit
    # demselben Endedatum eingetragen wurden (= fall.zda).
    # Jedenfalls geht diese Query schnell.
    #SELECT f.*, a.*, j.*, fs.*, m.*
    sql = """
SELECT f.id,z.id
FROM fall f
     INNER JOIN akte a ON f.akte_id = a.id
     INNER JOIN zustaendigkeit z ON f.id = z.fall_id AND
                                   f.zday = z.ey AND  
                                   f.zdam = z.em AND  
                                   f.zdad = z.ed
     INNER JOIN mitarbeiter m ON m.id = z.mit_id
     LEFT JOIN jghstat07 j ON f.id = j.fall_id
     LEFT JOIN fachstat fs ON f.id = fs.fall_id
ORDER BY f.id,z.id
"""

def get_letzter_beratungskontakt_datum(fall_ids):
    """ fall_id, datum als string dd.mm.yyyy
    """
    fall_ids_string = ','.join(str(id) for id in fall_ids)

    sql = """
SELECT f.id,
       CONCAT(LPAD(b.ky, 4, '0'),LPAD(b.km, 2, '0'),LPAD(b.kd, 2, '0'))
FROM fall f
     INNER JOIN fallberatungskontakt fb ON fb.fall_id = f.id
     INNER JOIN beratungskontakt b ON fb.bkont_id = b.id
WHERE f.id in ({})
ORDER BY fall_id, b.ky, b.km, b.kd, b.kh, b.kmin
""".format(fall_ids_string)
    res = getDBHandle().query(sql)
    f2d = {}
    for fall_id, lk in res:
        if fall_id in f2d:
            v = f2d[fall_id]
            f2d[fall_id] = max(lk, v)
        else:
            f2d[fall_id] = lk
    return f2d

glbd = get_letzter_beratungskontakt_datum
