# coding: latin-1

import re
import urlparse

# fake bobo REQUEST and RESPONSE objects
class REQUEST(object):
    def __init__(self, formssession, form, method):
        self.form = form
        self.formssession = formssession
        self._environ =  {'REMOTE_ADDR': self.formssession.ip,
                          'REQUEST_METHOD': method}
    def get(self, name): # 
        value = self.formssession.cookies.get(name)
        #print 'getCookie', name, value
        return value
    def __getitem__(self, key):
        return self.environ[key]
    
    @property
    def environ(self):
        return self._environ
    # @property
    # def form(self):
    #     return self.form

class RESPONSE(object):
    def __init__(self, formssession):
        self.formssession = formssession
        self.location = None
        self.headers = {}
        self.status = 200
    def setCookie(self, name, value, max_age):
        #print 'setCookie', name, value, max_age
        self.formssession.cookies[name] = value
    def expireCookie(self, name):
        #print 'expireCookie', name
        try:
            del self.formssession.cookies[name]
        except KeyError:
            pass
    def setStatus(self, name):
        assert name == 'InternalError'
        self.status = 500
    def setHeader(self, name, value):
        self.headers[name] = value
    def redirect(self, url):
        self.location = url
        self.status = 303



class FormsSession(object):
    def __init__(self):
        from ebkus.ebs import bobo_application
        self.ebkus_app = bobo_application
        self.cookies = {}
        self.ip = '192.168.22.22'
    # def login(self, user, password, ip):
    #     process = self.ebkus_app['login']
    #     form = {}
    #     res = process(REQUEST, RESPONSE)
    def post(self, path, data={}):
        return self.get(path, params=data, method='POST')
    def get(self, path, params={}, method='GET'):
        process = self.ebkus_app[path]
        requ = REQUEST(formssession=self, form=params, method=method)
        resp = RESPONSE(self)
        assert requ
        assert resp
        res = process(requ, resp)
        return FormsResponse(status=resp.status, headers=resp.headers, location=resp.location, text=res)
    def login(self, user, password):
        data = {'username': user, 'pass': password}
        r = self.post('login', data=data)
        return r
    def close(self):
        from ebkus.db.sql import closedb
        closedb()

class FormsResponse(object):
    def __init__(self, status, headers, location, text):
        self.status_code = status
        self.headers = headers
        self.text = text
        self.location = location

class TestError(Exception):
    pass
class WrongData(TestError):
    pass
class WrongPage(TestError):
    pass
class WrongRedirect(TestError):
    pass
class NoTitle(TestError):
    pass
class UnknownStatus(TestError):
    pass
class EBKuSError(TestError):
    pass
class EBKuSInternalError(TestError):
    pass
class NoAccess(TestError):
    pass
class NotAuthorized(TestError):
    pass
class InteractForms(object):
    _re_akid_fallid_persneu = re.compile(ur'persneu\?akid=(?P<akte_id>\d+)&fallid=(?P<fall_id>\d+)&klerv=')
    _re_akid_fallid_waufnneu = re.compile(ur'waufnneu\?akid=(?P<akte_id>\d+)&fallid=(?P<fall_id>\d+)"')
    _re_akid_fallid_zdar = re.compile(ur"go_to_url\('zdar\?akid=(?P<akte_id>\d+)&fallid=(?P<fall_id>\d+)'")
    _re_fehler = re.compile(ur'<fieldset><legend class="legendtext">Fehlerbeschreibung</legend>')
    _re_title = re.compile(ur'<title>(?P<title>.+?)</title>', re.IGNORECASE)
    _re_zugriff_verweigert = re.compile(ur'<fieldset><legend class="legendtext">Zugriff verweigert</legend>')
    _re_konfig_name_konfig_var = re.compile(ur"""updkonfig\?konfig_name=(?P<konfig_name>)">.+</td>.*<td class="tabledata">(?P=konfig_name)</td><td class="tabledata">(?P<konfig_value>)</td>""", re.MULTILINE)
    _re_konfig_var = re.compile(ur"""updkonfig\?konfig_name={konfig_name}">.+</td>.*<td class="tabledata">(konfig_name)</td><td class="tabledata">(?P<konfig_value>)</td>""")

    path = {
        'miteinf': 'mitausw',
        'updmit': 'mitausw',
        'akteeinf': 'klkarte',
        'leisteinf': 'klkarte',
        'fseinf': 'klkarte',
        'jgh07einf': 'klkarte',
        'zdaeinf': 'klkarte',
        'zdareinf': 'klkarte',
        'waufneinf': 'klkarte',
        'perseinf': 'klkarte',
        'einreinf': 'klkarte',
        'bkonteinf': 'klkarte',
        'codeeinf': 'admin',
        'updcode': 'admin',
        'updfskonfig': 'admin',
        }
    title = {
        'miteinf': 'Mitarbeiter',
        'updmit': 'Mitarbeiter',
        'akteeinf': 'Klientenkarte',
        'leisteinf': 'Klientenkarte',
        'fseinf': 'Klientenkarte',
        'jgh07einf': 'Klientenkarte',
        'zdaeinf': 'Klientenkarte',
        'zdareinf': 'Klientenkarte',
        'waufneinf': 'Klientenkarte',
        'perseinf': 'Klientenkarte',
        'einreinf': 'Klientenkarte',
        'bkonteinf': 'Klientenkarte',
        'codeeinf': 'Kategorien und Merkmalslisten',
        'updcode': 'Kategorien und Merkmalslisten',
        #'updfskonfig': 'Administratorhauptmen',
        #'updfskonfig': u'Administratorhauptmen',
        'updfskonfig': 'nderung durchgef',
        }
    def __init__(self, config, verbose=0):
        self.verbose = verbose
        self.config = config
        self.test_instanz = self.config.INSTANCE_NAME
        self.session = None
    def submit(self, file, test_daten=None, redirect=True, **kw):
        if test_daten:
            form = getattr(test_daten, file)(file=file, **kw)
        else:
            form = kw
            kw['file'] = file
        path = self.path[file]
        if self.verbose > 1:
            print path, file.upper(), form
        r = self.session.post(path, data=form)
        print 'POST r', r.status_code, r.location, r.text
        if redirect and r.status_code in (301, 303):
            print 'REDIRECT:', r.location
            r = self.get(r.location)
        self.check_error(r, is_in_title=self.title[file])
        r.form = form
        return r
    def set_today(self, date):
        self.login('Admin', 'Admin')
        self.set_config_vars(today=date, login=False)
        self.logout()
    # def submit(self, test_daten, file, **kw):
    #     form = getattr(test_daten, file)(file=file, **kw)
    #     if self.verbose > 1:
    #         print file.upper(), form
    #     r = self.session.post(file, data=form)
    #     print r.text
    #     #self.check_error(r, is_in_title=self.title[file])
    #     return form
    def post(self, path, data):
        return self.session.post(path, data=data)
    def get(self, path, params={}):
        "path kann parameter enthalten, params muss dann aber leer sein"
        res = urlparse.urlsplit(path)
        if res.query:
            assert not params
            params = urllib.parse_qs(res.query)
        params = dict([(k, v[0] if isinstance(v, list) and len(v) == 1 else v) for k,v in params.items()])
        return self.session.get(res.path, params=params)
    def altdaten(self, n_altdaten):
        raise NotImplementedError("altdaten()")
    def set_config_vars(self, login=True, **kw):
        if login:
            self.login('Admin', 'Admin')
        from ebkus.app.ebapi import Date
        for k, v in kw.items():
            setattr(self.config, k.upper(), v)
            if isinstance(v, bool):
                v = {True: 'true', False: 'false'}[v]
            if isinstance(v, (int, long)):
                v = str(v)
            if isinstance(v, (int, long, Date)):
                v = str(v)
            kl = k.lower()
            r = self.session.post('konfigausw', data={
                    'konfig_name': kl, 
                    'konfig_value': v, 
                    'konfig_all': '1', 
                    'file': 'updkonfig',
                    })
            self.check_error(r, is_in_title='Konfiguration')
            #print r.text.encode('utf8')
        if login:
            self.logout()
    def login(self, user, passw):
        if self.session is None:
            self.session = FormsSession()
        data = {'username': user, 'pass': passw}
        r = self.session.post('login', data=data)
        #print 'R.TEXT', r.text
        self.check_error(r, redirect=('menu', 'pwchange'))
        return True
    def logout(self):
        r = self.session.get('logout')
        self.check_error(r, redirect=('login', ))
        self.session.close()
        self.session = None
        return True
    def close(self):
        self.logout()
    def check_error(self, r, is_in_title=None, redirect=None):
        if r.status_code not in  (200, 301, 303, 500):
            raise UnknownStatus("HTTP Status: {0}".format(r.status_code))
        if r.status_code in (301, 303):
            if not redirect or r.location not in redirect:
                raise WrongRedirect(r.location)
            return True
        if self._re_zugriff_verweigert.search(r.text):
            raise NoAccess(r.text)
        res = self.match(self._re_title, r.text, title=True)
        if res:
            title = res['title']
        else:
            raise NoTitle(r.text)
        if self._re_fehler.search(r.text):
            if title == 'Interner Fehler':
                raise EBKuSInternalError(r.text)
            elif title == 'Fehlermeldung':
                raise EBKuSError(r.text)
        if is_in_title:
            if is_in_title not in title:
                raise WrongPage(' [String "' + is_in_title + '" <ist nicht in> "' + title + '"]\n' + r.text)
        return True
    def get_letzter_fall_id(self, akte_id):
        r = self.session.get('klkarte', params={'akid': akte_id})
        res = self.match(self._re_akid_fallid_persneu, r.text, akte_id=str(akte_id), fall_id=True)
        if not res:
            res = self.match(self._re_akid_fallid_waufnneu, r.text, akte_id=str(akte_id), fall_id=True)
        if not res:
            res = self.match(self._re_akid_fallid_zdar, r.text, akte_id=str(akte_id), fall_id=True)
        return int(res['fall_id'])
    def match(self, regex, text, **kw):
        """Jeder Parameter in kw ist der Name einer Gruppe in regex.
        Falls der Wert True ist, muss es einen nicht leeren Match geben.
        Sonst muss der Wert ein String sein. Der muss gleich sein mit
        dem Wert der matchenden Gruppe.
        Alle matchenden Gruppen werden als dict zurckgegeben.
        Wenn kein match gefunden, None zurckgeben.
        """
        #aT = self.assertTrue
        res = {}
        match = regex.search(text)
        # print '*********************MATCHED TEXT (type: {0})***********************'.format(type(text))
        # print 'Pattern:', regex.pattern, 'type: ', type(regex.pattern)
        # print text
        # print '*********************END MATCHED TEXT*******************'
        if not match:
            #print text
            return None
        for k,v in kw.items():
            val = match.group(k)
            res[k] = val
            if v is True:
                assert val
            else:
                assert isinstance(v, basestring)
                assert val == v
        return res
    


