# coding: latin-1



import sys,os
import re
from os.path import join, split, dirname, basename, exists, isdir, isfile, normpath, normcase, abspath, realpath, splitext, getmtime
import datetime
import unittest
import traceback
# from ebkus.config import config
# print config
# config.show()
from ebkus.gen import test_input

class EBKuSTestCase(test_input.TestInput):
    verbose = 0
    test_instanz = 'test'
    def ic(self, s):
        "inspect config"
        from ebkus.config import config
        print s
        if config is None:
            print ' ', None
        else:
            try:
                print ' Instanzen:', config.get_instances()
                print ' Instanz:', config.INSTANCE_NAME
            except:
                traceback.print_exc()
    def ic(self, s):
        pass
    def setUp(self):
        self.session = None
        self.set_ebkus_control()
        self.ectl.stop(self.test_instanz)
        self.initialize_database()
        # config erst hier initialisieren, da initialize wg. Installer alles durcheinanderbringt
        self.set_config()
        from ebkus.db.dbapp import cache_off, cache_on
        cache_on() # mit Cache etwas schneller
        #cache_off() # geht auch
        if not self.ectl.status(self.test_instanz):
            self.ectl.start(self.test_instanz)
        assert self.ectl.status(self.test_instanz), 'Testinstanz nicht gestartet'
        from ebkus.gen import test_daten
        self.td = test_daten.TestDaten(profil='minimal',
                            instance_name=self.test_instanz,
                            seed=101, 
                            #seed=None, 
                            von_jahr=None,
                            #heute="2016-01-05",
                            )
        self.interact = getattr(test_daten, self.InteractClass)(verbose=self.verbose)
        self.interact.assertTrue = self.assertTrue
    def initialize_database(self):
        """Sicherstellen, dass die Datenbank der Testinstanz mit standard.sql initialisiert ist.
        Falls die standard.sql zu alt ist oder fehlt, neu erstellen.
        Evt. nur die im Test erstellten Daten lschen, ohne neu zu laden (nicht realisiert).
        """
        template_dir = join(self.home_dir, 'templates')
        from ebkus import __date__ as version_date
        version_date = datetime.datetime.strptime(version_date, "%Y-%m-%d").date()
        standard_sql = join(self.home_dir, 'sql', 'standard.sql')
        recreate = False
        if isfile(standard_sql):
            # Falls lter als die EBKuS-Version, immer neu erstellen
            file_date = datetime.datetime.fromtimestamp(int(getmtime(standard_sql))).date()
            #print file_date, version_date
            if file_date < version_date:
                recreate = True
        else:
            recreate = True
        from ebkus.Install import InstallFromHome
        # Datenbank initialisieren
        if self.verbose < 2:
            out = sys.stdout
            #sys.stdout = open('/dev/null', 'wb')
            sys.stdout = open(os.devnull, 'wb')
            
        installer = InstallFromHome(self.home_dir)
        installer.init()
        test_instance = installer.ebkus[self.test_instanz]
        assert test_instance.name == self.test_instanz
        if recreate:
            merkmale_standard = join(self.home_dir, 'sql', 'merkmale_standard.py')
            test_instance.generate_initial_database(merkmale_standard, standard_sql)
        test_instance.create_database(sql_file=standard_sql)
        if self.verbose < 2:
            sys.stdout = out
        
    def set_ebkus_control(self):
        if not hasattr(self, 'ectl'):
            lib_dir = dirname(dirname(dirname(normcase(realpath(abspath(__file__))))))
            #print lib_dir
            assert isdir(lib_dir) and basename(lib_dir) == 'lib', "Kein lib-Verzeichnis: {0}".format(lib_dir)
            if lib_dir != sys.path[0]:
                sys.path.insert(0, lib_dir)
            from ebkus.ebkus_control import EBKuSControl
            self.ectl = EBKuSControl(dirname(lib_dir))
        self.home_dir = self.ectl.ebkus_home
    def set_config(self):
        """Sicherstellen, dass nach 'from ebkus.config import config'
        config auf die Konfiguration der Testinstanz gesetzt ist.
        Auerdem steht die config als self.config zur Verfgung.
        """
        assert self.test_instanz in self.ectl.instances, "Testinstanz '{0}' ist nicht konfiguriert".format(self.test_instanz)
        from ebkus.config import init
        self.config = init(self.test_instanz, join(self.home_dir, 'ebkus.conf'))
        #print 'EBKuSTestCase.config:', id(self.config)
        assert self.config.INSTANCE_NAME == self.test_instanz
    def tearDown(self):
        from ebkus.db.sql import closedb
        from ebkus.db.dbapp import cache_off
        closedb()
        self.td = None
        cache_off() # um Wechselwirkungen auf den folgenden Test zu vermeiden

    def _test_klient_neu(self):
        t1 = '12.08.2015'
        self.set_config_vars(beratungskontakte=True,
                             beratungskontakte_bs=True,
                             anmeldungsdaten_obligatorisch=True,
                             anmeldeprobleme_und_beschaeftigung_bei_anmeldung_erfassen=True,
                             today=t1,
                             #cache_on=False,
                             )
        self.einrichten()
        for i in range(1):
            self.eine_akte()
    def _test_einrichten_eine_akte(self):
        self._test_klient_neu()
        #print 'BERATUNGSKONTAKTE_BS', self.config.BERATUNGSKONTAKTE_BS
        from ebkus.update import UpdateDB
        u = UpdateDB()
        u.mehrfach_joker_felder_reparieren()

class EBKuSHttpTest(EBKuSTestCase, 
                    unittest.TestCase,
                    ):
    InteractClass = 'InteractHttp'
    def test_einrichten_eine_akte(self):
        self._test_einrichten_eine_akte()
        #print 'BERATUNGSKONTAKTE_BS', self.config.BERATUNGSKONTAKTE_BS
class EBKuSEbupdTest(EBKuSTestCase, 
                     unittest.TestCase,
                     ):
    InteractClass = 'InteractEbupd'
    def test_einrichten_eine_akte(self):
        self._test_einrichten_eine_akte()
    def xtest_fskonfig(self):
        pass


"""
Wie will ich tests schreiben?
Es sollte einen Ausgangspunkt geben, der sich sowohl fr den direkten Test 
ber ebupd(form) eignet, als auch fr den Blackbox-Test ber http. Daten 
fr beides sollten aus demselben Korpus generierbar sein.


Kat/Code: <kat>:<code>  -- wird intern in die Id bersetzt
Andere Entity Ids:      -- 
  Wenn ich im Test neue Entitten einfhre, kann ich einfach 
- getNewId aufrufen


Mit dem Konzept von demo_daten kann man statische Testdaten erzeugen.
Das sind dicts, die man als form direkt an die ebupd Schnittstelle geben kann.
Es sollte auch mglich sein, dieselben dicts fr Tests ber http und requests zu machen.

Statische Testdaten sind aber erstmal nicht ntig. (Wre effizienter bei Tests mit groen Datenmengen.)


Zwei Testarten:
- ber ebupd, ohne httpd
- Blackbox, von auen ber http und requests.

Fr beide Flle Daten ber eine Funktion erzeugen, die dicts erzeugt, 
die fr beide Flle verwendet werden knnen.
Fr beide Flle wird die Instanz 'test' verwendet, die mit standard.sql 
initialisiert wird.
Dann wird eine oder mehrere Akten durch den Test erzeugt und geprft.
Das kann fr verschiedene Konfigurationen wiederholt werden.
Der Test sollte ein setUp mit verschiedenen Konfigurationen ermglichen.
"""

if __name__ == '__main__':
    #print argparse.ArgumentParser
    unittest.main()

